@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,12 @@
 #  include "config.h"
 #endif
 
+#include <glibconfig.h>
+
+#ifdef G_OS_WIN32
+#  include <glib/gwin32.h>
+#endif
+
 #if HAVE_PATHS_H
 #  include <paths.h>
 #endif
@@ -37,7 +43,12 @@
 #define QUEUE_DIR		"queue"
 #define DRAFT_DIR		"draft"
 #define TRASH_DIR		"trash"
-#define RC_DIR			".sylpheed"
+#ifdef G_OS_WIN32
+#  define RC_DIR		"Sylpheed"
+#else
+#  define RC_DIR		".sylpheed-2.0"
+#endif
+#define OLD_RC_DIR		".sylpheed"
 #define NEWS_CACHE_DIR		"newscache"
 #define IMAP_CACHE_DIR		"imapcache"
 #define MIME_TMP_DIR		"mimetmp"
@@ -53,6 +64,7 @@
 #define COMMAND_HISTORY		"command_history"
 #define TEMPLATE_DIR		"templates"
 #define TMP_DIR			"tmp"
+#define UIDL_DIR		"uidl"
 #define NEWSGROUP_LIST		".newsgroup_list"
 #define ADDRESS_BOOK		"addressbook.xml"
 #define MANUAL_HTML_INDEX	"sylpheed.html"
@@ -61,10 +73,14 @@
 #define FOLDER_LIST		"folderlist.xml"
 #define CACHE_FILE		".sylpheed_cache"
 #define MARK_FILE		".sylpheed_mark"
-#define CACHE_VERSION		18
+#define CACHE_VERSION		0x21
 #define MARK_VERSION		2
 
-#define DEFAULT_SIGNATURE	".signature"
+#ifdef G_OS_WIN32
+#  define DEFAULT_SIGNATURE	"signature.txt"
+#else
+#  define DEFAULT_SIGNATURE	".signature"
+#endif
 #define DEFAULT_INC_PATH	"/usr/bin/mh/inc"
 #define DEFAULT_INC_PROGRAM	"inc"
 /* #define DEFAULT_INC_PATH	"/usr/bin/imget" */
@@ -101,11 +117,6 @@
 #define SESSION_TIMEOUT_INTERVAL	60	/* sec */
 #define MAX_HISTORY_SIZE		16
 
-#define DEFAULT_NORMAL_FONT	"-*-helvetica-medium-r-normal--12-*-*-*-*-*-*-*," \
-				"-*-*-medium-r-normal--12-*-*-*-*-*-*-*,*"
-#define DEFAULT_BOLD_FONT	"-*-helvetica-bold-r-normal--12-*-*-*-*-*-*-*," \
-				"-*-*-bold-r-normal--12-*-*-*-*-*-*-*,*"
-#define DEFAULT_SMALL_FONT	"-*-helvetica-medium-r-normal--10-*-*-*-*-*-*-*," \
-				"-*-*-medium-r-normal--10-*-*-*-*-*-*-*,*"
+#define DEFAULT_MESSAGE_FONT	"Monospace 12"
 
 #endif /* __DEFS_H__ */
