@@ -30,6 +30,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
@@ -37,15 +38,23 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkhbbox.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtkhseparator.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtkstatusbar.h>
+#include <gtk/gtkcheckbutton.h>
+#include <gtk/gtkfilesel.h>
 
-#include "intl.h"
 #include "addressbook.h"
 #include "prefs_common.h"
 #include "addressitem.h"
 #include "jpilot.h"
 #include "mgutils.h"
+#include "filesel.h"
 #include "gtkutils.h"
+#include "codeconv.h"
 #include "manage_window.h"
 
 #define ADDRESSBOOK_GUESS_JPILOT "MyJPilot"
@@ -57,14 +66,13 @@ static struct _JPilotEdit {
 	GtkWidget *file_entry;
 	GtkWidget *custom_check[JPILOT_NUM_CUSTOM_LABEL];
 	GtkWidget *custom_label[JPILOT_NUM_CUSTOM_LABEL];
+	GtkWidget *hbbox;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
 	GtkWidget *statusbar;
 	gint status_cid;
 } jpilotedit;
 
-static struct _AddressFileSelection jpilot_file_selector;
-
 /*
 * Edit functions.
 */
@@ -83,11 +91,12 @@ static gint edit_jpilot_delete_event( GtkWidget *widget, GdkEventAny *event, gbo
 	return TRUE;
 }
 
-static void edit_jpilot_key_pressed( GtkWidget *widget, GdkEventKey *event, gboolean *cancelled ) {
+static gboolean edit_jpilot_key_pressed( GtkWidget *widget, GdkEventKey *event, gboolean *cancelled ) {
 	if (event && event->keyval == GDK_Escape) {
 		*cancelled = TRUE;
 		gtk_main_quit();
 	}
+	return FALSE;
 }
 
 static void edit_jpilot_ok( GtkWidget *widget, gboolean *cancelled ) {
@@ -155,16 +164,18 @@ static void edit_jpilot_read_check_box( JPilotFile *pilotFile ) {
 static void edit_jpilot_file_check( void ) {
 	gint t = -1;
 	gchar *sFile;
+	gchar *sFSFile;
 	gchar *sMsg;
 	gboolean flg;
 
 	flg = FALSE;
 	sFile = gtk_editable_get_chars( GTK_EDITABLE(jpilotedit.file_entry), 0, -1 );
 	if( sFile ) {
-		g_strchomp( sFile ); g_strchug( sFile );
-		if( *sFile != '\0' ) {
+		sFSFile = conv_filename_from_utf8( sFile );
+		if( sFSFile && *sFSFile != '\0' ) {
 			/* Attempt to read file */
-			JPilotFile *jpf = jpilot_create_path( sFile );
+			JPilotFile *jpf;
+			jpf = jpilot_create_path( sFSFile );
 			t = jpilot_read_data( jpf );
 			if( t == MGU_SUCCESS ) {
 				/* Set check boxes */
@@ -174,12 +185,13 @@ static void edit_jpilot_file_check( void ) {
 			jpilot_free( jpf );
 			jpf = NULL;
 		}
+		g_free( sFSFile );
+		g_free( sFile );
 	}
 	if( ! flg ) {
 		/* Clear all check boxes */
 		edit_jpilot_fill_check_box_new();
 	}
-	g_free( sFile );
 
 	/* Display appropriate message */
 	if( t == MGU_SUCCESS ) {
@@ -194,55 +206,19 @@ static void edit_jpilot_file_check( void ) {
 	edit_jpilot_status_show( sMsg );
 }
 
-static void edit_jpilot_file_ok( GtkWidget *widget, gpointer data ) {
-	gchar *sFile;
-	AddressFileSelection *afs;
-	GtkWidget *fileSel;
-
-	afs = ( AddressFileSelection * ) data;
-	fileSel = afs->fileSelector;
-	sFile = gtk_file_selection_get_filename( GTK_FILE_SELECTION(fileSel) );
-
-	afs->cancelled = FALSE;
-	gtk_entry_set_text( GTK_ENTRY(jpilotedit.file_entry), sFile );
-	gtk_widget_hide( afs->fileSelector );
-	gtk_grab_remove( afs->fileSelector );
-	edit_jpilot_file_check();
-	gtk_widget_grab_focus( jpilotedit.file_entry );
-}
-
-static void edit_jpilot_file_cancel( GtkWidget *widget, gpointer data ) {
-	AddressFileSelection *afs = ( AddressFileSelection * ) data;
-	afs->cancelled = TRUE;
-	gtk_widget_hide( afs->fileSelector );
-	gtk_grab_remove( afs->fileSelector );
-	gtk_widget_grab_focus( jpilotedit.file_entry );
-}
-
-static void edit_jpilot_file_select_create( AddressFileSelection *afs ) {
-	GtkWidget *fileSelector;
-
-	fileSelector = gtk_file_selection_new( _("Select JPilot File") );
-	gtk_file_selection_hide_fileop_buttons( GTK_FILE_SELECTION(fileSelector) );
-	gtk_signal_connect( GTK_OBJECT (GTK_FILE_SELECTION(fileSelector)->ok_button),
-                             "clicked", GTK_SIGNAL_FUNC (edit_jpilot_file_ok), ( gpointer ) afs );
-	gtk_signal_connect( GTK_OBJECT (GTK_FILE_SELECTION(fileSelector)->cancel_button),
-                             "clicked", GTK_SIGNAL_FUNC (edit_jpilot_file_cancel), ( gpointer ) afs );
-	afs->fileSelector = fileSelector;
-	afs->cancelled = TRUE;
-}
-
 static void edit_jpilot_file_select( void ) {
 	gchar *sFile;
+	gchar *sUTF8File;
 
-	if (! jpilot_file_selector.fileSelector )
-		edit_jpilot_file_select_create( & jpilot_file_selector );
+	sFile = filesel_select_file( _("Select JPilot File"), NULL, GTK_FILE_CHOOSER_ACTION_OPEN );
 
-	sFile = gtk_editable_get_chars( GTK_EDITABLE(jpilotedit.file_entry), 0, -1 );
-	gtk_file_selection_set_filename( GTK_FILE_SELECTION( jpilot_file_selector.fileSelector ), sFile );
-	g_free( sFile );
-	gtk_widget_show( jpilot_file_selector.fileSelector );
-	gtk_grab_add( jpilot_file_selector.fileSelector );
+	if( sFile ) {
+		sUTF8File = conv_filename_to_utf8( sFile );
+		gtk_entry_set_text( GTK_ENTRY(jpilotedit.file_entry), sUTF8File );
+		g_free( sUTF8File );
+		g_free( sFile );
+		edit_jpilot_file_check();
+	}
 }
 
 static void addressbook_edit_jpilot_create( gboolean *cancelled ) {
@@ -267,18 +243,18 @@ static void addressbook_edit_jpilot_create( gboolean *cancelled ) {
 	GtkWidget *statusbar;
 	gint top, i;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_widget_set_usize(window, 450, -1);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_widget_set_size_request(window, 450, -1);
 	gtk_container_set_border_width(GTK_CONTAINER(window), 0);
 	gtk_window_set_title(GTK_WINDOW(window), _("Edit JPilot Entry"));
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);	
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(edit_jpilot_delete_event),
-			   cancelled);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(edit_jpilot_key_pressed),
-			   cancelled);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(edit_jpilot_delete_event),
+			 cancelled);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(edit_jpilot_key_pressed),
+			 cancelled);
 
 	vbox = gtk_vbox_new(FALSE, 8);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
@@ -341,8 +317,9 @@ static void addressbook_edit_jpilot_create( gboolean *cancelled ) {
 	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
 
 	/* Button panel */
-	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&hbbox, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
 	gtk_container_set_border_width( GTK_CONTAINER(hbbox), 0 );
 	gtk_widget_grab_default(ok_btn);
@@ -350,20 +327,21 @@ static void addressbook_edit_jpilot_create( gboolean *cancelled ) {
 	hsep = gtk_hseparator_new();
 	gtk_box_pack_end(GTK_BOX(vbox), hsep, FALSE, FALSE, 0);
 
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_jpilot_ok), cancelled);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_jpilot_cancel), cancelled);
-	gtk_signal_connect(GTK_OBJECT(file_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_jpilot_file_select), NULL);
-	gtk_signal_connect(GTK_OBJECT(check_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_jpilot_file_check), NULL);
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(edit_jpilot_ok), cancelled);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(edit_jpilot_cancel), cancelled);
+	g_signal_connect(G_OBJECT(file_btn), "clicked",
+			 G_CALLBACK(edit_jpilot_file_select), NULL);
+	g_signal_connect(G_OBJECT(check_btn), "clicked",
+			 G_CALLBACK(edit_jpilot_file_check), NULL);
 
 	gtk_widget_show_all(vbox);
 
 	jpilotedit.window     = window;
 	jpilotedit.name_entry = name_entry;
 	jpilotedit.file_entry = file_entry;
+	jpilotedit.hbbox      = hbbox;
 	jpilotedit.ok_btn     = ok_btn;
 	jpilotedit.cancel_btn = cancel_btn;
 	jpilotedit.statusbar  = statusbar;
@@ -378,12 +356,15 @@ AdapterDSource *addressbook_edit_jpilot( AddressIndex *addrIndex, AdapterDSource
 	static gboolean cancelled;
 	gchar *sName;
 	gchar *sFile;
+	gchar *sFSFile;
 	AddressDataSource *ds = NULL;
 	JPilotFile *jpf = NULL;
 	gboolean fin;
 
 	if( ! jpilotedit.window )
 		addressbook_edit_jpilot_create(&cancelled);
+	gtkut_box_set_reverse_order(GTK_BOX(jpilotedit.hbbox),
+				    !prefs_common.comply_gnome_hig);
 	gtk_widget_grab_focus(jpilotedit.ok_btn);
 	gtk_widget_grab_focus(jpilotedit.name_entry);
 	gtk_widget_show(jpilotedit.window);
@@ -418,8 +399,10 @@ AdapterDSource *addressbook_edit_jpilot( AddressIndex *addrIndex, AdapterDSource
 	fin = FALSE;
 	sName = gtk_editable_get_chars( GTK_EDITABLE(jpilotedit.name_entry), 0, -1 );
 	sFile = gtk_editable_get_chars( GTK_EDITABLE(jpilotedit.file_entry), 0, -1 );
+	sFSFile = conv_filename_from_utf8( sFile );
 	if( *sName == '\0' ) fin = TRUE;
 	if( *sFile == '\0' ) fin = TRUE;
+	if( ! sFSFile || *sFSFile == '\0' ) fin = TRUE;
 
 	if( ! fin ) {
 		if( ! ads ) {
@@ -429,11 +412,12 @@ AdapterDSource *addressbook_edit_jpilot( AddressIndex *addrIndex, AdapterDSource
 		}
 		addressbook_ads_set_name( ads, sName );
 		jpilot_set_name( jpf, sName );
-		jpilot_set_file( jpf, sFile );
+		jpilot_set_file( jpf, sFSFile );
 		edit_jpilot_read_check_box( jpf );
 	}
-	g_free( sName );
+	g_free( sFSFile );
 	g_free( sFile );
+	g_free( sName );
 
 	return ads;
 }
