@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,11 +24,11 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 
-#include "intl.h"
 #include "folder.h"
 #include "session.h"
 #include "imap.h"
@@ -173,6 +173,9 @@ FolderItem *folder_item_new(const gchar *name, const gchar *path)
 	item->collapsed = FALSE;
 	item->threaded = TRUE;
 	item->opened = FALSE;
+	item->updated = FALSE;
+	item->cache_dirty = FALSE;
+	item->mark_dirty = FALSE;
 	item->node = NULL;
 	item->parent = NULL;
 	item->folder = NULL;
@@ -272,6 +275,44 @@ void folder_item_destroy(FolderItem *item)
 	g_free(item);
 }
 
+gint folder_item_compare(FolderItem *item_a, FolderItem *item_b)
+{
+	gint ret;
+	gchar *str_a, *str_b;
+
+	if (!item_a || !item_b)
+		return 0;
+	if (!item_a->parent || !item_b->parent)
+		return 0;
+	if (!item_a->name || !item_b->name)
+		return 0;
+
+	/* if both a and b are special folders, sort them according to
+	 * their types (which is in-order). Note that this assumes that
+	 * there are no multiple folders of a special type. */
+	if (item_a->stype != F_NORMAL && item_b->stype != F_NORMAL)
+		return item_a->stype - item_b->stype;
+
+	 /* if b is normal folder, and a is not, b is smaller (ends up
+	  * lower in the list) */
+	if (item_a->stype != F_NORMAL && item_b->stype == F_NORMAL)
+		return item_b->stype - item_a->stype;
+
+	/* if b is special folder, and a is not, b is larger (ends up
+	 * higher in the list) */
+	if (item_a->stype == F_NORMAL && item_b->stype != F_NORMAL)
+		return item_b->stype - item_a->stype;
+
+	/* otherwise just compare the folder names */
+	str_a = g_utf8_casefold(item_a->name, -1);
+	str_b = g_utf8_casefold(item_b->name, -1);
+	ret = g_utf8_collate(str_a, str_b);
+	g_free(str_b);
+	g_free(str_a);
+
+	return ret;
+}
+
 void folder_set_ui_func(Folder *folder, FolderUIFunc func, gpointer data)
 {
 	g_return_if_fail(folder != NULL);
@@ -371,7 +412,7 @@ void folder_write_list(void)
 	if ((pfile = prefs_file_open(path)) == NULL) return;
 
 	fprintf(pfile->fp, "<?xml version=\"1.0\" encoding=\"%s\"?>\n",
-		conv_get_current_charset_str());
+		CS_INTERNAL);
 	fputs("\n<folderlist>\n", pfile->fp);
 
 	for (list = folder_list; list != NULL; list = list->next) {
@@ -595,7 +636,7 @@ static FolderType folder_get_type_from_string(const gchar *str)
 
 	for (i = 0; i < sizeof(type_str_table) / sizeof(type_str_table[0]);
 	     i++) {
-		if (g_strcasecmp(type_str_table[i].str, str) == 0)
+		if (g_ascii_strcasecmp(type_str_table[i].str, str) == 0)
 			return type_str_table[i].type;
 	}
 
@@ -618,10 +659,19 @@ gchar *folder_item_get_identifier(FolderItem *item)
 	gchar *folder_id;
 
 	g_return_val_if_fail(item != NULL, NULL);
-	g_return_val_if_fail(item->path != NULL, NULL);
+
+	if (!item->path) {
+		if (!item->parent)
+			return folder_get_identifier(item->folder);
+		else
+			return NULL;
+	}
 
 	folder_id = folder_get_identifier(item->folder);
 	id = g_strconcat(folder_id, "/", item->path, NULL);
+#ifdef G_OS_WIN32
+	subst_char(id, G_DIR_SEPARATOR, '/');
+#endif
 	g_free(folder_id);
 
 	return id;
@@ -804,9 +854,14 @@ gchar *folder_get_path(Folder *folder)
 
 	g_return_val_if_fail(folder != NULL, NULL);
 
-	if (FOLDER_TYPE(folder) == F_MH)
-		path = g_strdup(LOCAL_FOLDER(folder)->rootpath);
-	else if (FOLDER_TYPE(folder) == F_IMAP) {
+	if (FOLDER_TYPE(folder) == F_MH) {
+		path = g_filename_from_utf8(LOCAL_FOLDER(folder)->rootpath,
+					    -1, NULL, NULL, NULL);
+		if (!path) {
+			g_warning("folder_get_path: faild to convert character set\n");
+			path = g_strdup(LOCAL_FOLDER(folder)->rootpath);
+		}
+	} else if (FOLDER_TYPE(folder) == F_IMAP) {
 		g_return_val_if_fail(folder->account != NULL, NULL);
 		path = g_strconcat(get_imap_cache_dir(),
 				   G_DIR_SEPARATOR_S,
@@ -829,29 +884,43 @@ gchar *folder_get_path(Folder *folder)
 gchar *folder_item_get_path(FolderItem *item)
 {
 	gchar *folder_path;
-	gchar *path;
+	gchar *item_path = NULL, *path;
 
 	g_return_val_if_fail(item != NULL, NULL);
 
 	folder_path = folder_get_path(item->folder);
 	g_return_val_if_fail(folder_path != NULL, NULL);
 
-	if (folder_path[0] == G_DIR_SEPARATOR) {
-		if (item->path)
+	if (item->path) {
+		item_path = g_filename_from_utf8(item->path, -1,
+						 NULL, NULL, NULL);
+		if (!item_path) {
+			g_warning("folder_item_get_path: faild to convert character set\n");
+			item_path = g_strdup(item->path);
+		}
+#ifdef G_OS_WIN32
+		subst_char(item_path, '/', G_DIR_SEPARATOR);
+#endif
+	}
+
+	if (g_path_is_absolute(folder_path)) {
+		if (item_path)
 			path = g_strconcat(folder_path, G_DIR_SEPARATOR_S,
-					   item->path, NULL);
+					   item_path, NULL);
 		else
 			path = g_strdup(folder_path);
 	} else {
-		if (item->path)
-			path = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
-					   folder_path, G_DIR_SEPARATOR_S,
-					   item->path, NULL);
+		if (item_path)
+			path = g_strconcat(get_mail_base_dir(),
+					   G_DIR_SEPARATOR_S, folder_path,
+					   G_DIR_SEPARATOR_S, item_path, NULL);
 		else
-			path = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S,
-					   folder_path, NULL);
+			path = g_strconcat(get_mail_base_dir(),
+					   G_DIR_SEPARATOR_S, folder_path,
+					   NULL);
 	}
 
+	g_free(item_path);
 	g_free(folder_path);
 	return path;
 }
@@ -1178,17 +1247,17 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 
 		if (!attr || !attr->name || !attr->value) continue;
 		if (!strcmp(attr->name, "type")) {
-			if (!strcasecmp(attr->value, "normal"))
+			if (!g_ascii_strcasecmp(attr->value, "normal"))
 				stype = F_NORMAL;
-			else if (!strcasecmp(attr->value, "inbox"))
+			else if (!g_ascii_strcasecmp(attr->value, "inbox"))
 				stype = F_INBOX;
-			else if (!strcasecmp(attr->value, "outbox"))
+			else if (!g_ascii_strcasecmp(attr->value, "outbox"))
 				stype = F_OUTBOX;
-			else if (!strcasecmp(attr->value, "draft"))
+			else if (!g_ascii_strcasecmp(attr->value, "draft"))
 				stype = F_DRAFT;
-			else if (!strcasecmp(attr->value, "queue"))
+			else if (!g_ascii_strcasecmp(attr->value, "queue"))
 				stype = F_QUEUE;
-			else if (!strcasecmp(attr->value, "trash"))
+			else if (!g_ascii_strcasecmp(attr->value, "trash"))
 				stype = F_TRASH;
 		} else if (!strcmp(attr->name, "name"))
 			name = attr->value;
@@ -1331,15 +1400,15 @@ static gboolean folder_read_folder_func(GNode *node, gpointer data)
 
 		if (!attr || !attr->name || !attr->value) continue;
 		if (!strcmp(attr->name, "type")) {
-			if (!strcasecmp(attr->value, "mh"))
+			if (!g_ascii_strcasecmp(attr->value, "mh"))
 				type = F_MH;
-			else if (!strcasecmp(attr->value, "mbox"))
+			else if (!g_ascii_strcasecmp(attr->value, "mbox"))
 				type = F_MBOX;
-			else if (!strcasecmp(attr->value, "maildir"))
+			else if (!g_ascii_strcasecmp(attr->value, "maildir"))
 				type = F_MAILDIR;
-			else if (!strcasecmp(attr->value, "imap"))
+			else if (!g_ascii_strcasecmp(attr->value, "imap"))
 				type = F_IMAP;
-			else if (!strcasecmp(attr->value, "news"))
+			else if (!g_ascii_strcasecmp(attr->value, "news"))
 				type = F_NEWS;
 		} else if (!strcmp(attr->name, "name"))
 			name = attr->value;
