@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #endif
 
 #include <glib.h>
-#include <time.h>
 
 #include "mainwindow.h"
 #include "progressdialog.h"
@@ -47,6 +46,7 @@ typedef enum
 	INC_IO_ERROR,
 	INC_SOCKET_ERROR,
 	INC_EOF,
+	INC_TIMEOUT,
 	INC_CANCEL
 } IncState;
 
@@ -58,6 +58,9 @@ struct _IncProgressDialog
 
 	gboolean show_dialog;
 
+	GTimeVal progress_tv;
+	GTimeVal folder_tv;
+
 	GList *queue_list;	/* list of IncSession */
 	gint cur_row;
 };
@@ -68,6 +71,9 @@ struct _IncSession
 	IncState inc_state;
 
 	GHashTable *folder_table;	/* table of destination folders */
+	GHashTable *tmp_folder_table;	/* for progressive update */
+
+	gint cur_total_bytes;
 
 	gpointer data;
 };
@@ -75,11 +81,11 @@ struct _IncSession
 #define TIMEOUT_ITV	200
 
 void inc_mail			(MainWindow	*mainwin);
+gint inc_account_mail		(MainWindow	*mainwin,
+				 PrefsAccount	*account);
 void inc_all_account_mail	(MainWindow	*mainwin,
 				 gboolean	 autocheck);
 void inc_progress_update	(Pop3Session	*session);
-gint inc_drop_message		(const gchar	*file,
-				 Pop3Session	*session);
 
 gboolean inc_is_active		(void);
 
