@@ -32,6 +32,7 @@
 #include "addrcache.h"
 
 #include "base64.h"
+#include "utils.h"
 
 /*
 * Create new object.
@@ -192,7 +193,7 @@ void ldif_print_file( LdifFile *ldifFile, FILE *stream ) {
 static gint ldif_open_file( LdifFile* ldifFile ) {
 	/* printf( "Opening file\n" ); */
 	if( ldifFile->path ) {
-		ldifFile->file = fopen( ldifFile->path, "r" );
+		ldifFile->file = g_fopen( ldifFile->path, "rb" );
 		if( ! ldifFile->file ) {
 			/* printf( "can't open %s\n", ldifFile->path ); */
 			ldifFile->retVal = MGU_OPEN_FILE;
@@ -325,6 +326,7 @@ static gchar *ldif_get_tagvalue( gchar* line ) {
 	return value;
 }
 
+#if 0
 /*
 * Dump linked lists of character strings (for debug).
 */
@@ -342,6 +344,7 @@ static void ldif_dump_lists( GSList *listName, GSList *listAddr, GSList *listRem
 	fprintf( stdout, "------------\n" );
 	mgu_print_list( listID, stdout );
 }
+#endif
 
 /*
 * Parsed address data.
@@ -496,19 +499,19 @@ static void ldif_add_value( Ldif_ParsedRec *rec, gchar *tagName, gchar *tagValue
 		val = g_strdup( "" );
 	}
 	g_strstrip( val );
-	if( g_strcasecmp( nm, LDIF_TAG_COMMONNAME ) == 0 ) {
+	if( g_ascii_strcasecmp( nm, LDIF_TAG_COMMONNAME ) == 0 ) {
 		rec->listCName = g_slist_append( rec->listCName, val );
 	}
-	else if( g_strcasecmp( nm, LDIF_TAG_FIRSTNAME ) == 0 ) {
+	else if( g_ascii_strcasecmp( nm, LDIF_TAG_FIRSTNAME ) == 0 ) {
 		rec->listFName = g_slist_append( rec->listFName, val );
 	}
-	else if( g_strcasecmp( nm, LDIF_TAG_LASTNAME ) == 0 ) {
+	else if( g_ascii_strcasecmp( nm, LDIF_TAG_LASTNAME ) == 0 ) {
 		rec->listLName = g_slist_append( rec->listLName, val );
 	}
-	else if( g_strcasecmp( nm, LDIF_TAG_NICKNAME ) == 0 ) {
+	else if( g_ascii_strcasecmp( nm, LDIF_TAG_NICKNAME ) == 0 ) {
 		rec->listNName = g_slist_append( rec->listNName, val );
 	}
-	else if( g_strcasecmp( nm, LDIF_TAG_EMAIL ) == 0 ) {
+	else if( g_ascii_strcasecmp( nm, LDIF_TAG_EMAIL ) == 0 ) {
 		rec->listAddress = g_slist_append( rec->listAddress, val );
 	}
 	else {
@@ -551,10 +554,10 @@ static void ldif_clear_rec( Ldif_ParsedRec *rec ) {
 	rec->listID = NULL;
 }
 
+#if 0
 /*
 * Print parsed data.
 */
-#if 0
 static void ldif_print_record( Ldif_ParsedRec *rec, FILE *stream ) {
 	GSList *list;
 
@@ -586,7 +589,6 @@ static void ldif_print_record( Ldif_ParsedRec *rec, FILE *stream ) {
 	}
 	list = NULL;
 }
-#endif
 
 static void ldif_dump_b64( gchar *buf ) {
 	Base64Decoder *decoder = NULL;
@@ -606,6 +608,7 @@ static void ldif_dump_b64( gchar *buf ) {
 	base64_decoder_free( decoder );
 	decoder = NULL;
 }
+#endif
 
 /*
 * Read file data into address cache.
@@ -741,19 +744,19 @@ static void ldif_hash_add_list( GHashTable *table, GSList *list ) {
 			gchar *key = g_strdup( tag );
 
 			rec = ldif_create_fieldrec( tag );
-			if( g_strcasecmp( tag, LDIF_TAG_COMMONNAME ) == 0 ) {
+			if( g_ascii_strcasecmp( tag, LDIF_TAG_COMMONNAME ) == 0 ) {
 				rec->reserved = TRUE;
 			}
-			else if( g_strcasecmp( tag, LDIF_TAG_FIRSTNAME ) == 0 ) {
+			else if( g_ascii_strcasecmp( tag, LDIF_TAG_FIRSTNAME ) == 0 ) {
 				rec->reserved = TRUE;
 			}
-			else if( g_strcasecmp( tag, LDIF_TAG_LASTNAME ) == 0 ) {
+			else if( g_ascii_strcasecmp( tag, LDIF_TAG_LASTNAME ) == 0 ) {
 				rec->reserved = TRUE;
 			}
-			else if( g_strcasecmp( tag, LDIF_TAG_NICKNAME ) == 0 ) {
+			else if( g_ascii_strcasecmp( tag, LDIF_TAG_NICKNAME ) == 0 ) {
 				rec->reserved = TRUE;
 			}
-			else if( g_strcasecmp( tag, LDIF_TAG_EMAIL ) == 0 ) {
+			else if( g_ascii_strcasecmp( tag, LDIF_TAG_EMAIL ) == 0 ) {
 				rec->reserved = TRUE;
 			}
 			g_hash_table_insert( table, key, rec );
@@ -768,7 +771,7 @@ static void ldif_hash_add_list( GHashTable *table, GSList *list ) {
 static int ldif_field_compare( gconstpointer ptr1, gconstpointer ptr2 ) {
 	const Ldif_FieldRec *rec1 = ptr1;
 	const Ldif_FieldRec *rec2 = ptr2;
-	return g_strcasecmp( rec1->tagName, rec2->tagName );
+	return g_ascii_strcasecmp( rec1->tagName, rec2->tagName );
 }
 
 /*
@@ -840,7 +843,7 @@ static void ldif_read_tag_list( LdifFile *ldifFile ) {
 				if( tagName ) {
 					/* Add tag to list */
 					listTags = g_slist_append( listTags, tagName );
-					if( g_strcasecmp( tagName, LDIF_TAG_EMAIL ) == 0 ) {
+					if( g_ascii_strcasecmp( tagName, LDIF_TAG_EMAIL ) == 0 ) {
 						flagMail = TRUE;
 					}
 				}
