@@ -38,8 +38,9 @@
 #include <time.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <signal.h>
-#include <dirent.h>
+#ifdef G_OS_UNIX
+#  include <signal.h>
+#endif
 
 #if HAVE_LOCALE_H
 #  include <locale.h>
@@ -158,7 +159,10 @@ int main(int argc, char *argv[])
 
 	parse_cmd_opt(argc, argv);
 
+	sock_init();
+
 	/* check and create unix domain socket for remote operation */
+#ifdef G_OS_UNIX
 	lock_socket = prohibit_duplicate_launch();
 	if (lock_socket < 0) return 0;
 
@@ -167,6 +171,7 @@ int main(int argc, char *argv[])
 		lock_socket_remove();
 		return 0;
 	}
+#endif
 
 	gtk_set_locale();
 	gtk_init(&argc, &argv);
@@ -185,8 +190,6 @@ int main(int argc, char *argv[])
 	ssl_init();
 #endif
 
-	srandom((gint)time(NULL));
-
 	/* parse gtkrc files */
 	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtkrc-2.0",
 			     NULL);
@@ -206,9 +209,10 @@ int main(int argc, char *argv[])
 
 	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), 1);
 
+#ifndef G_OS_WIN32
 	/* backup if old rc file exists */
 	if (is_file_exist(RC_DIR)) {
-		if (rename(RC_DIR, RC_DIR ".bak") < 0)
+		if (rename_force(RC_DIR, RC_DIR ".bak") < 0)
 			FILE_OP_ERROR(RC_DIR, "rename");
 	}
 
@@ -237,31 +241,40 @@ int main(int argc, char *argv[])
 					return 1;
 			}
 		}
+
 		if (make_dir(RC_DIR) < 0)
 			return 1;
 		if (is_dir_exist(OLD_RC_DIR))
 			migrate_old_config();
 	}
+#else
+	if (!is_dir_exist(get_rc_dir())) {
+		if (make_dir_hier(get_rc_dir()) < 0)
+			return 1;
+	}
+
+	MAKE_DIR_IF_NOT_EXIST(get_mail_base_dir());
+#endif /* G_OS_WIN32 */
+
+	CHDIR_RETURN_VAL_IF_FAIL(get_rc_dir(), 1);
 
 	MAKE_DIR_IF_NOT_EXIST(get_imap_cache_dir());
 	MAKE_DIR_IF_NOT_EXIST(get_news_cache_dir());
 	MAKE_DIR_IF_NOT_EXIST(get_mime_tmp_dir());
 	MAKE_DIR_IF_NOT_EXIST(get_tmp_dir());
-	MAKE_DIR_IF_NOT_EXIST(RC_DIR G_DIR_SEPARATOR_S UIDL_DIR);
+	MAKE_DIR_IF_NOT_EXIST(UIDL_DIR);
 
 	/* remove temporary files */
 	remove_all_files(get_tmp_dir());
 	remove_all_files(get_mime_tmp_dir());
 
-	if (is_file_exist(RC_DIR G_DIR_SEPARATOR_S "sylpheed.log")) {
-		if (rename(RC_DIR G_DIR_SEPARATOR_S "sylpheed.log",
-			   RC_DIR G_DIR_SEPARATOR_S "sylpheed.log.bak") < 0)
+	if (is_file_exist("sylpheed.log")) {
+		if (rename_force("sylpheed.log", "sylpheed.log.bak") < 0)
 			FILE_OP_ERROR("sylpheed.log", "rename");
 	}
-	set_log_file(RC_DIR G_DIR_SEPARATOR_S "sylpheed.log");
+	set_log_file("sylpheed.log");
 
-	set_ui_update_func(gtkut_events_flush);
-	set_progress_func(main_window_progress_show);
+	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), 1);
 
 	prefs_common_read_config();
 
@@ -321,10 +334,12 @@ int main(int argc, char *argv[])
 	folderview = mainwin->folderview;
 
 	/* register the callback of unix domain socket input */
+#ifdef G_OS_UNIX
 	lock_socket_tag = gdk_input_add(lock_socket,
 					GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
 					lock_socket_input_cb,
 					mainwin);
+#endif
 
 	account_read_config_all();
 
@@ -346,7 +361,9 @@ int main(int argc, char *argv[])
 	inc_autocheck_timer_init(mainwin);
 
 	/* ignore SIGPIPE signal for preventing sudden death of program */
+#ifdef G_OS_UNIX
 	signal(SIGPIPE, SIG_IGN);
+#endif
 
 	if (cmd.receive_all)
 		inc_all_account_mail(mainwin, FALSE);
@@ -543,6 +560,8 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 	ssl_done();
 #endif
 
+	sock_cleanup();
+
 	gtk_main_quit();
 }
 
@@ -563,7 +582,11 @@ static gchar *get_socket_name(void)
 	if (filename == NULL) {
 		filename = g_strdup_printf("%s%csylpheed-%d",
 					   g_get_tmp_dir(), G_DIR_SEPARATOR,
+#if HAVE_GETUID
 					   getuid());
+#else
+					   0);
+#endif
 	}
 
 	return filename;
@@ -577,7 +600,7 @@ static gint prohibit_duplicate_launch(void)
 	path = get_socket_name();
 	uxsock = fd_connect_unix(path);
 	if (uxsock < 0) {
-		unlink(path);
+		g_unlink(path);
 		return fd_open_unix(path);
 	}
 
@@ -654,6 +677,7 @@ static gint prohibit_duplicate_launch(void)
 
 static gint lock_socket_remove(void)
 {
+#ifdef G_OS_UNIX
 	gchar *filename;
 
 	if (lock_socket < 0) return -1;
@@ -662,7 +686,8 @@ static gint lock_socket_remove(void)
 		gdk_input_remove(lock_socket_tag);
 	fd_close(lock_socket);
 	filename = get_socket_name();
-	unlink(filename);
+	g_unlink(filename);
+#endif
 
 	return 0;
 }
@@ -745,8 +770,8 @@ static void lock_socket_input_cb(gpointer data,
 
 static void migrate_old_config(void)
 {
-	DIR *dp;
-	struct dirent *d;
+	GDir *dir;
+	const gchar *dir_name;
 	GPatternSpec *pspec;
 
 	if (alertpanel(_("Migration of configuration"),
@@ -795,22 +820,22 @@ static void migrate_old_config(void)
 	if (!is_file_exist(OLD_RC_DIR G_DIR_SEPARATOR_S ADDRESSBOOK_INDEX_FILE))
 		return;
 
-	if ((dp = opendir(OLD_RC_DIR)) == NULL) {
-		FILE_OP_ERROR(OLD_RC_DIR, "opendir");
+	if ((dir = g_dir_open(OLD_RC_DIR, 0, NULL)) == NULL) {
+		g_warning("failed to open directory: %s\n", OLD_RC_DIR);
 		return;
 	}
 
 	pspec = g_pattern_spec_new("addrbook-*.xml");
 
-	while ((d = readdir(dp)) != NULL) {
-		if (g_pattern_match_string(pspec, d->d_name)) {
+	while ((dir_name = g_dir_read_name(dir)) != NULL) {
+		if (g_pattern_match_string(pspec, dir_name)) {
 			gchar *old_file;
 			gchar *new_file;
 
 			old_file = g_strconcat(OLD_RC_DIR G_DIR_SEPARATOR_S,
-					       d->d_name, NULL);
+					       dir_name, NULL);
 			new_file = g_strconcat(RC_DIR G_DIR_SEPARATOR_S,
-					       d->d_name, NULL);
+					       dir_name, NULL);
 			copy_file(old_file, new_file, FALSE);
 			g_free(new_file);
 			g_free(old_file);
@@ -818,8 +843,7 @@ static void migrate_old_config(void)
 	}
 
 	g_pattern_spec_free(pspec);
-
-	closedir(dp);
+	g_dir_close(dir);
 }
 
 static void open_compose_new(const gchar *address, GPtrArray *attach_files)
