@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -32,7 +33,6 @@
 #include <ctype.h>
 #include <time.h>
 
-#include "intl.h"
 #include "mbox.h"
 #include "procmsg.h"
 #include "folder.h"
@@ -50,7 +50,7 @@
 		g_warning(_("can't write to temporary file\n")); \
 		fclose(tmp_fp); \
 		fclose(mbox_fp); \
-		unlink(tmp_file); \
+		g_unlink(tmp_file); \
 		g_free(tmp_file); \
 		return -1; \
 	} \
@@ -68,7 +68,7 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 
 	debug_print(_("Getting messages from %s into %s...\n"), mbox, dest->path);
 
-	if ((mbox_fp = fopen(mbox, "rb")) == NULL) {
+	if ((mbox_fp = g_fopen(mbox, "rb")) == NULL) {
 		FILE_OP_ERROR(mbox, "fopen");
 		return -1;
 	}
@@ -105,7 +105,7 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 		gboolean is_next_msg = FALSE;
 		FilterInfo *fltinfo;
 
-		if ((tmp_fp = fopen(tmp_file, "wb")) == NULL) {
+		if ((tmp_fp = g_fopen(tmp_file, "wb")) == NULL) {
 			FILE_OP_ERROR(tmp_file, "fopen");
 			g_warning(_("can't open temporary file\n"));
 			g_free(tmp_file);
@@ -194,7 +194,7 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 		if (fclose(tmp_fp) == EOF) {
 			FILE_OP_ERROR(tmp_file, "fclose");
 			g_warning(_("can't write to temporary file\n"));
-			unlink(tmp_file);
+			g_unlink(tmp_file);
 			g_free(tmp_file);
 			fclose(mbox_fp);
 			return -1;
@@ -212,7 +212,7 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 			if (folder_item_add_msg(dest, tmp_file, &fltinfo->flags,
 						FALSE) < 0) {
 				filter_info_free(fltinfo);
-				unlink(tmp_file);
+				g_unlink(tmp_file);
 				g_free(tmp_file);
 				fclose(mbox_fp);
 				return -1;
@@ -243,7 +243,7 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 		}
 
 		filter_info_free(fltinfo);
-		unlink(tmp_file);
+		g_unlink(tmp_file);
 
 		msgs++;
 	} while (from_line[0] != '\0');
@@ -257,6 +257,7 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 
 gint lock_mbox(const gchar *base, LockType type)
 {
+#ifdef G_OS_UNIX
 	gint retval = 0;
 
 	if (type == LOCK_FILE) {
@@ -265,7 +266,7 @@ gint lock_mbox(const gchar *base, LockType type)
 		FILE *lockfp;
 
 		lockfile = g_strdup_printf("%s.%d", base, getpid());
-		if ((lockfp = fopen(lockfile, "wb")) == NULL) {
+		if ((lockfp = g_fopen(lockfile, "wb")) == NULL) {
 			FILE_OP_ERROR(lockfile, "fopen");
 			g_warning(_("can't create lock file %s\n"), lockfile);
 			g_warning(_("use 'flock' instead of 'file' if possible.\n"));
@@ -281,7 +282,7 @@ gint lock_mbox(const gchar *base, LockType type)
 			FILE_OP_ERROR(lockfile, "link");
 			if (retry >= 5) {
 				g_warning(_("can't create %s\n"), lockfile);
-				unlink(lockfile);
+				g_unlink(lockfile);
 				g_free(lockfile);
 				return -1;
 			}
@@ -291,7 +292,7 @@ gint lock_mbox(const gchar *base, LockType type)
 			retry++;
 			sleep(5);
 		}
-		unlink(lockfile);
+		g_unlink(lockfile);
 		g_free(lockfile);
 	} else if (type == LOCK_FLOCK) {
 		gint lockfd;
@@ -327,6 +328,9 @@ gint lock_mbox(const gchar *base, LockType type)
 	}
 
 	return retval;
+#else
+	return -1;
+#endif /* G_OS_UNIX */
 }
 
 gint unlock_mbox(const gchar *base, gint fd, LockType type)
@@ -335,7 +339,7 @@ gint unlock_mbox(const gchar *base, gint fd, LockType type)
 		gchar *lockfile;
 
 		lockfile = g_strconcat(base, ".lock", NULL);
-		if (unlink(lockfile) < 0) {
+		if (g_unlink(lockfile) < 0) {
 			FILE_OP_ERROR(lockfile, "unlink");
 			g_free(lockfile);
 			return -1;
@@ -380,17 +384,23 @@ gint copy_mbox(const gchar *src, const gchar *dest)
 
 void empty_mbox(const gchar *mbox)
 {
+#if HAVE_TRUNCATE
 	if (truncate(mbox, 0) < 0) {
+#endif
 		FILE *fp;
 
+#if HAVE_TRUNCATE
 		FILE_OP_ERROR(mbox, "truncate");
-		if ((fp = fopen(mbox, "wb")) == NULL) {
+#endif
+		if ((fp = g_fopen(mbox, "wb")) == NULL) {
 			FILE_OP_ERROR(mbox, "fopen");
 			g_warning(_("can't truncate mailbox to zero.\n"));
 			return;
 		}
 		fclose(fp);
+#if HAVE_TRUNCATE
 	}
+#endif
 }
 
 /* read all messages in SRC, and store them into one MBOX file. */
@@ -410,7 +420,7 @@ gint export_to_mbox(FolderItem *src, const gchar *mbox)
 	debug_print(_("Exporting messages from %s into %s...\n"),
 		    src->path, mbox);
 
-	if ((mbox_fp = fopen(mbox, "wb")) == NULL) {
+	if ((mbox_fp = g_fopen(mbox, "wb")) == NULL) {
 		FILE_OP_ERROR(mbox, "fopen");
 		return -1;
 	}
