@@ -27,16 +27,13 @@
 #include <glib/gi18n.h>
 #include <dirent.h>
 #include <sys/stat.h>
+#include <time.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
 
 #undef MEASURE_TIME
 
-#ifdef MEASURE_TIME
-#  include <sys/time.h>
-#endif
-
 #include "folder.h"
 #include "mh.h"
 #include "procmsg.h"
@@ -197,13 +194,15 @@ static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
 	GHashTable *msg_table;
 	time_t cur_mtime;
 #ifdef MEASURE_TIME
-	struct timeval tv_before, tv_after, tv_result;
-
-	gettimeofday(&tv_before, NULL);
+	GTimer *timer;
 #endif
 
 	g_return_val_if_fail(item != NULL, NULL);
 
+#ifdef MEASURE_TIME
+	timer = g_timer_new();
+#endif
+
 	cur_mtime = mh_get_mtime(item);
 
 	if (use_cache && item->mtime == cur_mtime) {
@@ -257,11 +256,10 @@ static GSList *mh_get_msg_list(Folder *folder, FolderItem *item,
 	mlist = procmsg_sort_msg_list(mlist, item->sort_key, item->sort_type);
 
 #ifdef MEASURE_TIME
-	gettimeofday(&tv_after, NULL);
-
-	timersub(&tv_after, &tv_before, &tv_result);
-	g_print("mh_get_msg_list: %s: elapsed time: %ld.%06ld sec\n",
-		item->path, tv_result.tv_sec, tv_result.tv_usec);
+	g_timer_stop(timer);
+	g_print("%s: %s: elapsed time: %f sec\n",
+		G_STRFUNC, item->path, g_timer_elapsed(timer, NULL));
+	g_timer_destroy(timer);
 #endif
 	debug_print("cache_dirty: %d, mark_dirty: %d\n",
 		    item->cache_dirty, item->mark_dirty);
@@ -407,14 +405,18 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 		if (first_ == 0 || first_ > dest->last_num + 1)
 			first_ = dest->last_num + 1;
 
+#ifdef G_OS_UNIX
 		if (link(fileinfo->file, destfile) < 0) {
+#endif
 			if (copy_file(fileinfo->file, destfile, TRUE) < 0) {
 				g_warning(_("can't copy message %s to %s\n"),
 					  fileinfo->file, destfile);
 				g_free(destfile);
 				return -1;
 			}
+#ifdef G_OS_UNIX
 		}
+#endif
 
 		g_free(destfile);
 		dest->last_num++;
@@ -453,7 +455,7 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 	if (remove_source) {
 		for (cur = file_list; cur != NULL; cur = cur->next) {
 			fileinfo = (MsgFileInfo *)cur->data;
-			if (unlink(fileinfo->file) < 0)
+			if (g_unlink(fileinfo->file) < 0)
 				FILE_OP_ERROR(fileinfo->file, "unlink");
 		}
 	}
@@ -657,7 +659,7 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, MsgInfo *msginfo)
 	file = mh_fetch_msg(folder, item, msginfo->msgnum);
 	g_return_val_if_fail(file != NULL, -1);
 
-	if (unlink(file) < 0) {
+	if (g_unlink(file) < 0) {
 		FILE_OP_ERROR(file, "unlink");
 		g_free(file);
 		return -1;
@@ -703,7 +705,7 @@ static gboolean mh_is_msg_changed(Folder *folder, FolderItem *item,
 {
 	struct stat s;
 
-	if (stat(itos(msginfo->msgnum), &s) < 0 ||
+	if (g_stat(itos(msginfo->msgnum), &s) < 0 ||
 	    msginfo->size  != s.st_size ||
 	    msginfo->mtime != s.st_mtime)
 		return TRUE;
@@ -836,7 +838,7 @@ static gint mh_create_tree(Folder *folder)
 
 	g_return_val_if_fail(folder != NULL, -1);
 
-	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), -1);
+	CHDIR_RETURN_VAL_IF_FAIL(get_mail_base_dir(), -1);
 	rootpath = LOCAL_FOLDER(folder)->rootpath;
 	MAKE_DIR_IF_NOT_EXIST(rootpath);
 	CHDIR_RETURN_VAL_IF_FAIL(rootpath, -1);
@@ -953,7 +955,7 @@ static gint mh_move_folder_real(Folder *folder, FolderItem *item,
 
 	debug_print("mh_move_folder: rename(%s, %s)\n", oldpath, newpath);
 
-	if (rename(oldpath, newpath) < 0) {
+	if (g_rename(oldpath, newpath) < 0) {
 		FILE_OP_ERROR(oldpath, "rename");
 		g_free(oldpath);
 		g_free(newpath);
@@ -1041,7 +1043,7 @@ static time_t mh_get_mtime(FolderItem *item)
 	struct stat s;
 
 	path = folder_item_get_path(item);
-	if (stat(path, &s) < 0) {
+	if (g_stat(path, &s) < 0) {
 		FILE_OP_ERROR(path, "stat");
 		return -1;
 	} else {
@@ -1224,8 +1226,13 @@ static void mh_remove_missing_folder_items(Folder *folder)
 static void mh_scan_tree_recursive(FolderItem *item)
 {
 	Folder *folder;
+#ifdef G_OS_WIN32
+	GDir *dir;
+#else
 	DIR *dp;
 	struct dirent *d;
+#endif
+	const gchar *dir_name;
 	struct stat s;
 	gchar *fs_path;
 	gchar *entry;
@@ -1243,12 +1250,21 @@ static void mh_scan_tree_recursive(FolderItem *item)
 		: g_strdup(".");
 	if (!fs_path)
 		fs_path = g_strdup(item->path);
+#ifdef G_OS_WIN32
+	dir = g_dir_open(fs_path, 0, NULL);
+	if (!dir) {
+		g_warning("failed to open directory: %s\n", fs_path);
+		g_free(fs_path);
+		return;
+	}
+#else
 	dp = opendir(fs_path);
 	if (!dp) {
 		FILE_OP_ERROR(fs_path, "opendir");
 		g_free(fs_path);
 		return;
 	}
+#endif
 	g_free(fs_path);
 
 	debug_print("scanning %s ...\n",
@@ -1257,12 +1273,17 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	if (folder->ui_func)
 		folder->ui_func(folder, item, folder->ui_func_data);
 
+#ifdef G_OS_WIN32
+	while ((dir_name = g_dir_read_name(dir)) != NULL) {
+#else
 	while ((d = readdir(dp)) != NULL) {
-		if (d->d_name[0] == '.') continue;
+		dir_name = d->d_name;
+#endif
+		if (dir_name[0] == '.') continue;
 
-		utf8name = g_filename_to_utf8(d->d_name, -1, NULL, NULL, NULL);
+		utf8name = g_filename_to_utf8(dir_name, -1, NULL, NULL, NULL);
 		if (!utf8name)
-			utf8name = g_strdup(d->d_name);
+			utf8name = g_strdup(dir_name);
 
 		if (item->path)
 			utf8entry = g_strconcat(item->path, G_DIR_SEPARATOR_S,
@@ -1274,12 +1295,12 @@ static void mh_scan_tree_recursive(FolderItem *item)
 			entry = g_strdup(utf8entry);
 
 		if (
-#ifdef HAVE_DIRENT_D_TYPE
+#if !defined(G_OS_WIN32) && defined(HAVE_DIRENT_D_TYPE)
 			d->d_type == DT_DIR ||
 			(d->d_type == DT_UNKNOWN &&
 #endif
-			stat(entry, &s) == 0 && S_ISDIR(s.st_mode)
-#ifdef HAVE_DIRENT_D_TYPE
+			g_stat(entry, &s) == 0 && S_ISDIR(s.st_mode)
+#if !defined(G_OS_WIN32) && defined(HAVE_DIRENT_D_TYPE)
 			)
 #endif
 		   ) {
@@ -1294,6 +1315,8 @@ static void mh_scan_tree_recursive(FolderItem *item)
 				continue;
 			}
 #endif
+
+#ifndef G_OS_WIN32
 			if (g_utf8_validate(utf8name, -1, NULL) == FALSE) {
 				g_warning(_("Directory name\n"
 					    "'%s' is not a valid UTF-8 string.\n"
@@ -1308,6 +1331,7 @@ static void mh_scan_tree_recursive(FolderItem *item)
 				g_free(utf8name);
 				continue;
 			}
+#endif /* G_OS_WIN32 */
 
 			node = item->node;
 			for (node = node->children; node != NULL; node = node->next) {
@@ -1325,37 +1349,41 @@ static void mh_scan_tree_recursive(FolderItem *item)
 
 			if (!item->path) {
 				if (!folder->inbox &&
-				    !strcmp(d->d_name, INBOX_DIR)) {
+				    !strcmp(dir_name, INBOX_DIR)) {
 					new_item->stype = F_INBOX;
 					folder->inbox = new_item;
 				} else if (!folder->outbox &&
-					   !strcmp(d->d_name, OUTBOX_DIR)) {
+					   !strcmp(dir_name, OUTBOX_DIR)) {
 					new_item->stype = F_OUTBOX;
 					folder->outbox = new_item;
 				} else if (!folder->draft &&
-					   !strcmp(d->d_name, DRAFT_DIR)) {
+					   !strcmp(dir_name, DRAFT_DIR)) {
 					new_item->stype = F_DRAFT;
 					folder->draft = new_item;
 				} else if (!folder->queue &&
-					   !strcmp(d->d_name, QUEUE_DIR)) {
+					   !strcmp(dir_name, QUEUE_DIR)) {
 					new_item->stype = F_QUEUE;
 					folder->queue = new_item;
 				} else if (!folder->trash &&
-					   !strcmp(d->d_name, TRASH_DIR)) {
+					   !strcmp(dir_name, TRASH_DIR)) {
 					new_item->stype = F_TRASH;
 					folder->trash = new_item;
 				}
 			}
 
 			mh_scan_tree_recursive(new_item);
-		} else if (to_number(d->d_name) > 0) n_msg++;
+		} else if (to_number(dir_name) > 0) n_msg++;
 
 		g_free(entry);
 		g_free(utf8entry);
 		g_free(utf8name);
 	}
 
+#ifdef G_OS_WIN32
+	g_dir_close(dir);
+#else
 	closedir(dp);
+#endif
 
 	if (item->path) {
 		gint new, unread, total, min, max;
