@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -31,7 +32,6 @@
 #include <unistd.h>
 #include <time.h>
 
-#include "intl.h"
 #include "news.h"
 #include "nntp.h"
 #include "socket.h"
@@ -151,6 +151,7 @@ static FolderClass news_class =
 	news_close,
 	news_scan_group,
 
+	NULL,
 	NULL,
 	NULL,
 	NULL
@@ -268,8 +269,8 @@ static NNTPSession *news_session_get(Folder *folder)
 		return NNTP_SESSION(rfolder->session);
 	}
 
-	if (time(NULL) - rfolder->session->last_access_time < SESSION_TIMEOUT_INTERVAL) {
-		rfolder->session->last_access_time = time(NULL);
+	if (time(NULL) - rfolder->session->last_access_time <
+		SESSION_TIMEOUT_INTERVAL) {
 		return NNTP_SESSION(rfolder->session);
 	}
 
@@ -285,7 +286,8 @@ static NNTPSession *news_session_get(Folder *folder)
 	}
 
 	if (rfolder->session)
-		rfolder->session->last_access_time = time(NULL);
+		session_set_access_time(rfolder->session);
+
 	return NNTP_SESSION(rfolder->session);
 }
 
@@ -314,10 +316,13 @@ static GSList *news_get_article_list(Folder *folder, FolderItem *item,
 		cache_last = procmsg_get_last_num_in_msg_list(alist);
 		newlist = news_get_uncached_articles
 			(session, item, cache_last, &first, &last);
+		if (newlist)
+			item->cache_dirty = TRUE;
 		if (first == 0 && last == 0) {
 			news_delete_all_articles(item);
 			procmsg_msg_list_free(alist);
 			alist = NULL;
+			item->cache_dirty = TRUE;
 		} else {
 			alist = news_delete_old_articles(alist, item, first);
 			news_delete_expired_caches(alist, item);
@@ -333,12 +338,16 @@ static GSList *news_get_article_list(Folder *folder, FolderItem *item,
 			(session, item, 0, NULL, &last);
 		news_delete_all_articles(item);
 		item->last_num = last;
+		item->cache_dirty = TRUE;
 	}
 
 	procmsg_set_flags(alist, item);
 
 	alist = procmsg_sort_msg_list(alist, item->sort_key, item->sort_type);
 
+	debug_print("cache_dirty: %d, mark_dirty: %d\n",
+		    item->cache_dirty, item->mark_dirty);
+
 	return alist;
 }
 
@@ -496,7 +505,7 @@ static void news_group_info_free(NewsGroupInfo *ginfo)
 static gint news_group_info_compare(NewsGroupInfo *ginfo1,
 				    NewsGroupInfo *ginfo2)
 {
-	return g_strcasecmp(ginfo1->name, ginfo2->name);
+	return g_ascii_strcasecmp(ginfo1->name, ginfo2->name);
 }
 
 GSList *news_get_group_list(Folder *folder)
@@ -516,7 +525,7 @@ GSList *news_get_group_list(Folder *folder)
 	filename = g_strconcat(path, G_DIR_SEPARATOR_S, NEWSGROUP_LIST, NULL);
 	g_free(path);
 
-	if ((fp = fopen(filename, "rb")) == NULL) {
+	if ((fp = g_fopen(filename, "rb")) == NULL) {
 		NNTPSession *session;
 		gint ok;
 
@@ -543,7 +552,7 @@ GSList *news_get_group_list(Folder *folder)
 			return NULL;
 		}
 
-		if ((fp = fopen(filename, "rb")) == NULL) {
+		if ((fp = g_fopen(filename, "rb")) == NULL) {
 			FILE_OP_ERROR(filename, "fopen");
 			g_free(filename);
 			return NULL;
@@ -623,7 +632,7 @@ gint news_post(Folder *folder, const gchar *file)
 	g_return_val_if_fail(FOLDER_TYPE(folder) == F_NEWS, -1);
 	g_return_val_if_fail(file != NULL, -1);
 
-	if ((fp = fopen(file, "rb")) == NULL) {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return -1;
 	}
@@ -717,7 +726,8 @@ static gint news_select_group(NNTPSession *session, const gchar *group,
 	gint num_, first_, last_;
 
 	if (!num || !first || !last) {
-		if (session->group && g_strcasecmp(session->group, group) == 0)
+		if (session->group &&
+		    g_ascii_strcasecmp(session->group, group) == 0)
 			return NN_SUCCESS;
 		num = &num_;
 		first = &first_;
@@ -746,6 +756,7 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 	GSList *newlist = NULL;
 	GSList *llast = NULL;
 	MsgInfo *msginfo;
+	gint max_articles;
 
 	if (rfirst) *rfirst = -1;
 	if (rlast)  *rlast  = -1;
@@ -753,6 +764,7 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 	g_return_val_if_fail(session != NULL, NULL);
 	g_return_val_if_fail(item != NULL, NULL);
 	g_return_val_if_fail(item->folder != NULL, NULL);
+	g_return_val_if_fail(item->folder->account != NULL, NULL);
 	g_return_val_if_fail(FOLDER_TYPE(item->folder) == F_NEWS, NULL);
 
 	ok = news_select_group(session, item->path, &num, &first, &last);
@@ -785,9 +797,9 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 		begin = cache_last + 1;
 	end = last;
 
-	if (prefs_common.max_articles > 0 &&
-	    end - begin + 1 > prefs_common.max_articles)
-		begin = end - prefs_common.max_articles + 1;
+	max_articles = item->folder->account->max_nntp_articles;
+	if (max_articles > 0 && end - begin + 1 > max_articles)
+		begin = end - max_articles + 1;
 
 	log_message(_("getting xover %d - %d in %s...\n"),
 		    begin, end, item->path);
@@ -894,6 +906,8 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 		llast = llast->next;
 	}
 
+	session_set_access_time(SESSION(session));
+
 	return newlist;
 }
 
@@ -908,7 +922,6 @@ static GSList *news_get_uncached_articles(NNTPSession *session,
 static MsgInfo *news_parse_xover(const gchar *xover_str)
 {
 	MsgInfo *msginfo;
-	gchar buf[NNTPBUFSIZE];
 	gchar *subject, *sender, *size, *line, *date, *msgid, *ref, *tmp;
 	gchar *p;
 	gint num, size_int, line_int;
@@ -941,12 +954,10 @@ static MsgInfo *news_parse_xover(const gchar *xover_str)
 	msginfo->date = g_strdup(date);
 	msginfo->date_t = procheader_date_parse(NULL, date, 0);
 
-	conv_unmime_header(buf, sizeof(buf), sender, NULL);
-	msginfo->from = g_strdup(buf);
-	msginfo->fromname = procheader_get_fromname(buf);
+	msginfo->from = conv_unmime_header(sender, NULL);
+	msginfo->fromname = procheader_get_fromname(msginfo->from);
 
-	conv_unmime_header(buf, sizeof(buf), subject, NULL);
-	msginfo->subject = g_strdup(buf);
+	msginfo->subject = conv_unmime_header(subject, NULL);
 
 	extract_parenthesis(msgid, '<', '>');
 	remove_space(msgid);
@@ -1014,6 +1025,7 @@ static GSList *news_delete_old_articles(GSList *alist, FolderItem *item,
 		if (msginfo && msginfo->msgnum < first) {
 			procmsg_msginfo_free(msginfo);
 			alist = g_slist_remove(alist, msginfo);
+			item->cache_dirty = TRUE;
 		}
 
 		cur = next;
