@@ -21,7 +21,10 @@
 #  include "config.h"
 #endif
 
+#include "defs.h"
+
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
@@ -30,7 +33,6 @@
 #include <time.h>
 #include <errno.h>
 
-#include "intl.h"
 #include "pop.h"
 #include "md5.h"
 #include "prefs_account.h"
@@ -334,7 +336,7 @@ static gint pop3_retr_recv(Pop3Session *session, const gchar *data, guint len)
 	}
 
 	drop_ok = session->drop_message(session, file);
-	unlink(file);
+	g_unlink(file);
 	g_free(file);
 	if (drop_ok < 0) {
 		session->error_val = PS_IOERR;
@@ -383,7 +385,7 @@ static void pop3_gen_send(Pop3Session *session, const gchar *format, ...)
 	g_vsnprintf(buf, sizeof(buf) - 2, format, args);
 	va_end(args);
 
-	if (!strncasecmp(buf, "PASS ", 5))
+	if (!g_ascii_strncasecmp(buf, "PASS ", 5))
 		log_print("POP3> PASS ********\n");
 	else
 		log_print("POP3> %s\n", buf);
@@ -454,19 +456,12 @@ GHashTable *pop3_get_uidl_table(PrefsAccount *ac_prefs)
 	table = g_hash_table_new(g_str_hash, g_str_equal);
 
 	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
-			   "uidl", G_DIR_SEPARATOR_S, ac_prefs->recv_server,
+			   UIDL_DIR, G_DIR_SEPARATOR_S, ac_prefs->recv_server,
 			   "-", ac_prefs->userid, NULL);
-	if ((fp = fopen(path, "rb")) == NULL) {
+	if ((fp = g_fopen(path, "rb")) == NULL) {
 		if (ENOENT != errno) FILE_OP_ERROR(path, "fopen");
 		g_free(path);
-		path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
-				   "uidl-", ac_prefs->recv_server,
-				   "-", ac_prefs->userid, NULL);
-		if ((fp = fopen(path, "rb")) == NULL) {
-			if (ENOENT != errno) FILE_OP_ERROR(path, "fopen");
-			g_free(path);
-			return table;
-		}
+		return table;
 	}
 	g_free(path);
 
@@ -504,7 +499,7 @@ gint pop3_write_uidl_list(Pop3Session *session)
 			   "uidl", G_DIR_SEPARATOR_S,
 			   session->ac_prefs->recv_server,
 			   "-", session->ac_prefs->userid, NULL);
-	if ((fp = fopen(path, "wb")) == NULL) {
+	if ((fp = g_fopen(path, "wb")) == NULL) {
 		FILE_OP_ERROR(path, "fopen");
 		g_free(path);
 		return -1;
@@ -512,8 +507,7 @@ gint pop3_write_uidl_list(Pop3Session *session)
 
 	for (n = 1; n <= session->count; n++) {
 		msg = &session->msg[n];
-		if (msg->uidl && msg->received &&
-		    (!msg->deleted || session->state != POP3_DONE))
+		if (msg->uidl && msg->received && !msg->deleted)
 			fprintf(fp, "%s\t%ld\n", msg->uidl, msg->recv_time);
 	}
 
@@ -531,7 +525,7 @@ static gint pop3_write_msg_to_file(const gchar *file, const gchar *data,
 
 	g_return_val_if_fail(file != NULL, -1);
 
-	if ((fp = fopen(file, "wb")) == NULL) {
+	if ((fp = g_fopen(file, "wb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return -1;
 	}
@@ -550,7 +544,7 @@ static gint pop3_write_msg_to_file(const gchar *file, const gchar *data,
 			FILE_OP_ERROR(file, "fwrite");
 			g_warning("can't write to file: %s\n", file);
 			fclose(fp);
-			unlink(file);
+			g_unlink(file);
 			return -1;
 		}
 
@@ -576,7 +570,7 @@ static gint pop3_write_msg_to_file(const gchar *file, const gchar *data,
 		FILE_OP_ERROR(file, "fwrite");
 		g_warning("can't write to file: %s\n", file);
 		fclose(fp);
-		unlink(file);
+		g_unlink(file);
 		return -1;
 	}
 	if (data[len - 1] != '\r' && data[len - 1] != '\n') {
@@ -584,14 +578,14 @@ static gint pop3_write_msg_to_file(const gchar *file, const gchar *data,
 			FILE_OP_ERROR(file, "fputc");
 			g_warning("can't write to file: %s\n", file);
 			fclose(fp);
-			unlink(file);
+			g_unlink(file);
 			return -1;
 		}
 	}
 
 	if (fclose(fp) == EOF) {
 		FILE_OP_ERROR(file, "fclose");
-		unlink(file);
+		g_unlink(file);
 		return -1;
 	}
 
@@ -702,7 +696,7 @@ static gint pop3_session_recv_msg(Session *session, const gchar *msg)
 {
 	Pop3Session *pop3_session = POP3_SESSION(session);
 	Pop3ErrorValue val = PS_SUCCESS;
-	const guchar *body;
+	const gchar *body;
 
 	body = msg;
 	if (pop3_session->state != POP3_GETRANGE_UIDL_RECV &&
@@ -717,9 +711,9 @@ static gint pop3_session_recv_msg(Session *session, const gchar *msg)
 
 		if (*body == '+' || *body == '-')
 			body++;
-		while (isalpha(*body))
+		while (g_ascii_isalpha(*body))
 			body++;
-		while (isspace(*body))
+		while (g_ascii_isspace(*body))
 			body++;
 	}
 
@@ -800,7 +794,6 @@ static gint pop3_session_recv_msg(Session *session, const gchar *msg)
 		}
 		break;
 	case POP3_LOGOUT:
-		pop3_session->state = POP3_DONE;
 		session_disconnect(session);
 		break;
 	case POP3_ERROR:
