@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #  include "config.h"
 #endif
 
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,9 +30,9 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include "intl.h"
 #include "main.h"
 #include "prefs.h"
+#include "codeconv.h"
 #include "utils.h"
 #include "gtkutils.h"
 
@@ -40,28 +42,24 @@ typedef enum
 } DummyEnum;
 
 void prefs_read_config(PrefParam *param, const gchar *label,
-		       const gchar *rcfile)
+		       const gchar *rcfile, const gchar *encoding)
 {
 	FILE *fp;
 	gchar buf[PREFSBUFSIZE];
-	gchar *rcpath;
 	gchar *block_label;
 
 	g_return_if_fail(param != NULL);
 	g_return_if_fail(label != NULL);
 	g_return_if_fail(rcfile != NULL);
 
-	debug_print(_("Reading configuration...\n"));
+	debug_print("Reading configuration...\n");
 
 	prefs_set_default(param);
 
-	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, rcfile, NULL);
-	if ((fp = fopen(rcpath, "rb")) == NULL) {
-		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
-		g_free(rcpath);
+	if ((fp = g_fopen(rcfile, "rb")) == NULL) {
+		if (ENOENT != errno) FILE_OP_ERROR(rcfile, "fopen");
 		return;
 	}
-	g_free(rcpath);
 
 	block_label = g_strdup_printf("[%s]", label);
 
@@ -69,9 +67,20 @@ void prefs_read_config(PrefParam *param, const gchar *label,
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		gint val;
 
-		val = strncmp(buf, block_label, strlen(block_label));
+		if (encoding) {
+			gchar *conv_str;
+
+			conv_str = conv_codeset_strdup
+				(buf, encoding, CS_INTERNAL);
+			if (!conv_str)
+				conv_str = g_strdup(buf);
+			val = strncmp
+				(conv_str, block_label, strlen(block_label));
+			g_free(conv_str);
+		} else
+			val = strncmp(buf, block_label, strlen(block_label));
 		if (val == 0) {
-			debug_print(_("Found %s\n"), block_label);
+			debug_print("Found %s\n", block_label);
 			break;
 		}
 	}
@@ -82,10 +91,20 @@ void prefs_read_config(PrefParam *param, const gchar *label,
 		/* reached next block */
 		if (buf[0] == '[') break;
 
-		prefs_config_parse_one_line(param, buf);
+		if (encoding) {
+			gchar *conv_str;
+
+			conv_str = conv_codeset_strdup
+				(buf, encoding, CS_INTERNAL);
+			if (!conv_str)
+				conv_str = g_strdup(buf);
+			prefs_config_parse_one_line(param, conv_str);
+			g_free(conv_str);
+		} else
+			prefs_config_parse_one_line(param, buf);
 	}
 
-	debug_print(_("Finished reading configuration.\n"));
+	debug_print("Finished reading configuration.\n");
 	fclose(fp);
 }
 
@@ -97,7 +116,7 @@ void prefs_config_parse_one_line(PrefParam *param, const gchar *buf)
 
 	for (i = 0; param[i].name != NULL; i++) {
 		name_len = strlen(param[i].name);
-		if (strncasecmp(buf, param[i].name, name_len))
+		if (g_ascii_strncasecmp(buf, param[i].name, name_len))
 			continue;
 		if (buf[name_len] != '=')
 			continue;
@@ -159,7 +178,7 @@ void prefs_write_config(PrefParam *param, const gchar *label,
 	g_return_if_fail(rcfile != NULL);
 
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, rcfile, NULL);
-	if ((orig_fp = fopen(rcpath, "rb")) == NULL) {
+	if ((orig_fp = g_fopen(rcpath, "rb")) == NULL) {
 		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
 	}
 
@@ -267,7 +286,7 @@ PrefFile *prefs_file_open(const gchar *path)
 	g_return_val_if_fail(path != NULL, NULL);
 
 	tmppath = g_strconcat(path, ".tmp", NULL);
-	if ((fp = fopen(tmppath, "wb")) == NULL) {
+	if ((fp = g_fopen(tmppath, "wb")) == NULL) {
 		FILE_OP_ERROR(tmppath, "fopen");
 		g_free(tmppath);
 		return NULL;
@@ -301,7 +320,7 @@ gint prefs_file_close(PrefFile *pfile)
 	tmppath = g_strconcat(path, ".tmp", NULL);
 	if (fclose(fp) == EOF) {
 		FILE_OP_ERROR(tmppath, "fclose");
-		unlink(tmppath);
+		g_unlink(tmppath);
 		g_free(path);
 		g_free(tmppath);
 		return -1;
@@ -309,9 +328,9 @@ gint prefs_file_close(PrefFile *pfile)
 
 	if (is_file_exist(path)) {
 		bakpath = g_strconcat(path, ".bak", NULL);
-		if (rename(path, bakpath) < 0) {
+		if (rename_force(path, bakpath) < 0) {
 			FILE_OP_ERROR(path, "rename");
-			unlink(tmppath);
+			g_unlink(tmppath);
 			g_free(path);
 			g_free(tmppath);
 			g_free(bakpath);
@@ -319,9 +338,9 @@ gint prefs_file_close(PrefFile *pfile)
 		}
 	}
 
-	if (rename(tmppath, path) < 0) {
+	if (rename_force(tmppath, path) < 0) {
 		FILE_OP_ERROR(tmppath, "rename");
-		unlink(tmppath);
+		g_unlink(tmppath);
 		g_free(path);
 		g_free(tmppath);
 		g_free(bakpath);
@@ -342,7 +361,8 @@ gint prefs_file_close_revert(PrefFile *pfile)
 
 	tmppath = g_strconcat(pfile->path, ".tmp", NULL);
 	fclose(pfile->fp);
-	if (unlink(tmppath) < 0) FILE_OP_ERROR(tmppath, "unlink");
+	if (g_unlink(tmppath) < 0)
+		FILE_OP_ERROR(tmppath, "unlink");
 	g_free(tmppath);
 	g_free(pfile->path);
 	g_free(pfile);
@@ -362,10 +382,23 @@ void prefs_set_default(PrefParam *param)
 		switch (param[i].type) {
 		case P_STRING:
 			if (param[i].defval != NULL) {
-				if (!strncasecmp(param[i].defval, "ENV_", 4))
-					*((gchar **)param[i].data) =
-						g_strdup(g_getenv(param[i].defval + 4));
-				else if (param[i].defval[0] == '~')
+				if (!g_ascii_strncasecmp(param[i].defval, "ENV_", 4)) {
+					const gchar *envstr;
+					gchar *tmp = NULL;
+
+					envstr = g_getenv(param[i].defval + 4);
+					if (envstr) {
+						tmp = conv_codeset_strdup
+							(envstr,
+							 conv_get_locale_charset_str(),
+							 CS_UTF_8);
+						if (!tmp) {
+							g_warning("failed to convert character set.");
+							tmp = g_strdup(envstr);
+						}
+					}
+					*((gchar **)param[i].data) = tmp;
+				} else if (param[i].defval[0] == '~')
 					*((gchar **)param[i].data) =
 						g_strconcat(get_home_dir(),
 							    param[i].defval + 1,
@@ -387,7 +420,7 @@ void prefs_set_default(PrefParam *param)
 			break;
 		case P_BOOL:
 			if (param[i].defval != NULL) {
-				if (!strcasecmp(param[i].defval, "TRUE"))
+				if (!g_ascii_strcasecmp(param[i].defval, "TRUE"))
 					*((gboolean *)param[i].data) = TRUE;
 				else
 					*((gboolean *)param[i].data) =
@@ -447,9 +480,9 @@ void prefs_dialog_create(PrefsDialog *dialog)
 
 	g_return_if_fail(dialog != NULL);
 
-	window = gtk_window_new (GTK_WINDOW_DIALOG);
+	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
-	gtk_window_position (GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+	gtk_window_set_position (GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
 	gtk_window_set_policy (GTK_WINDOW(window), FALSE, TRUE, FALSE);
 
@@ -464,19 +497,20 @@ void prefs_dialog_create(PrefsDialog *dialog)
 	/* GTK_WIDGET_UNSET_FLAGS (notebook, GTK_CAN_FOCUS); */
 	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
 
-	gtkut_button_set_create(&confirm_area,
-				&ok_btn,	_("OK"),
-				&cancel_btn,	_("Cancel"),
-				&apply_btn,	_("Apply"));
+	gtkut_stock_button_set_create(&confirm_area,
+				      &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      &apply_btn, GTK_STOCK_APPLY);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end (GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
-	dialog->window     = window;
-	dialog->notebook   = notebook;
-	dialog->ok_btn     = ok_btn;
-	dialog->cancel_btn = cancel_btn;
-	dialog->apply_btn  = apply_btn;
+	dialog->window       = window;
+	dialog->notebook     = notebook;
+	dialog->confirm_area = confirm_area;
+	dialog->ok_btn       = ok_btn;
+	dialog->cancel_btn   = cancel_btn;
+	dialog->apply_btn    = apply_btn;
 }
 
 void prefs_dialog_destroy(PrefsDialog *dialog)
@@ -535,8 +569,10 @@ void prefs_set_dialog_to_default(PrefParam *param)
 		switch (tmpparam.type) {
 		case P_STRING:
 			if (tmpparam.defval) {
-				if (!strncasecmp(tmpparam.defval, "ENV_", 4)) {
-					str_data = g_strdup(g_getenv(param[i].defval + 4));
+				if (!g_ascii_strncasecmp
+					(tmpparam.defval, "ENV_", 4)) {
+					str_data = g_strdup
+						(g_getenv(param[i].defval + 4));
 					tmpparam.data = &str_data;
 					break;
 				} else if (tmpparam.defval[0] == '~') {
@@ -566,7 +602,7 @@ void prefs_set_dialog_to_default(PrefParam *param)
 			break;
 		case P_BOOL:
 			if (tmpparam.defval) {
-				if (!strcasecmp(tmpparam.defval, "TRUE"))
+				if (!g_ascii_strcasecmp(tmpparam.defval, "TRUE"))
 					bool_data = TRUE;
 				else
 					bool_data = atoi(tmpparam.defval)
@@ -593,7 +629,8 @@ void prefs_set_dialog_to_default(PrefParam *param)
 
 void prefs_set_data_from_entry(PrefParam *pparam)
 {
-	gchar **str, *entry_str;
+	gchar **str;
+	const gchar *entry_str;
 
 	g_return_if_fail(*pparam->widget != NULL);
 
@@ -646,17 +683,34 @@ void prefs_set_entry(PrefParam *pparam)
 void prefs_set_data_from_text(PrefParam *pparam)
 {
 	gchar **str;
-	gchar *text, *tp;
+	gchar *text = NULL, *tp = NULL;
 	gchar *tmp, *tmpp;
 
 	g_return_if_fail(*pparam->widget != NULL);
+	g_return_if_fail(GTK_IS_EDITABLE(*pparam->widget) ||
+			 GTK_IS_TEXT_VIEW(*pparam->widget));
 
 	switch (pparam->type) {
 	case P_STRING:
 		str = (gchar **)pparam->data;
 		g_free(*str);
-		tp = text = gtk_editable_get_chars
-			(GTK_EDITABLE(*pparam->widget), 0, -1);
+		if (GTK_IS_EDITABLE(*pparam->widget)) {
+			tp = text = gtk_editable_get_chars
+				(GTK_EDITABLE(*pparam->widget), 0, -1);
+		} else if (GTK_IS_TEXT_VIEW(*pparam->widget)) {
+			GtkTextView *textview = GTK_TEXT_VIEW(*pparam->widget);
+			GtkTextBuffer *buffer;
+			GtkTextIter start, end;
+
+			buffer = gtk_text_view_get_buffer(textview);
+			gtk_text_buffer_get_start_iter(buffer, &start);
+			gtk_text_buffer_get_iter_at_offset(buffer, &end, -1);
+			tp = text = gtk_text_buffer_get_text
+				(buffer, &start, &end, FALSE);
+		}
+
+		g_return_if_fail(tp != NULL && text != NULL);
+
 		if (text[0] == '\0') {
 			*str = NULL;
 			g_free(text);
@@ -678,7 +732,7 @@ void prefs_set_data_from_text(PrefParam *pparam)
 		g_free(text);
 		break;
 	default:
-		g_warning("Invalid PrefType for GtkText widget: %d\n",
+		g_warning("Invalid PrefType for GtkTextView widget: %d\n",
 			  pparam->type);
 	}
 }
@@ -687,7 +741,9 @@ void prefs_set_text(PrefParam *pparam)
 {
 	gchar *buf, *sp, *bufp;
 	gchar **str;
-	GtkText *text;
+	GtkTextView *text;
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
 
 	g_return_if_fail(*pparam->widget != NULL);
 
@@ -711,14 +767,14 @@ void prefs_set_text(PrefParam *pparam)
 		} else
 			buf = "";
 
-		text = GTK_TEXT(*pparam->widget);
-		gtk_text_set_point(text, 0);
-		gtk_text_forward_delete(text, gtk_text_get_length(text));
-		gtk_text_set_point(text, 0);
-		gtk_text_insert(text, NULL, NULL, NULL, buf, -1);
+		text = GTK_TEXT_VIEW(*pparam->widget);
+		buffer = gtk_text_view_get_buffer(text);
+		gtk_text_buffer_set_text(buffer, "", 0);
+		gtk_text_buffer_get_start_iter(buffer, &iter);
+		gtk_text_buffer_insert(buffer, &iter, buf, -1);
 		break;
 	default:
-		g_warning("Invalid PrefType for GtkText widget: %d\n",
+		g_warning("Invalid PrefType for GtkTextView widget: %d\n",
 			  pparam->type);
 	}
 }
