@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -75,6 +75,7 @@ struct _PrefsDialog
 	GtkWidget *window;
 	GtkWidget *notebook;
 
+	GtkWidget *confirm_area;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
 	GtkWidget *apply_btn;
@@ -115,6 +116,25 @@ struct _PrefsDialog
 	gtk_frame_set_label_align(GTK_FRAME(frame), 0.01, 0.5); \
 }
 
+#define PACK_FRAME_WITH_CHECK_BUTTON(box, frame, chkbtn, label) \
+{ \
+	chkbtn = gtk_check_button_new_with_label(label); \
+	gtk_widget_show(chkbtn); \
+	PACK_FRAME(box, frame, NULL); \
+	gtk_frame_set_label_widget(GTK_FRAME(frame), chkbtn); \
+}
+
+#define PACK_SMALL_LABEL(box, label, str) \
+{ \
+	label = gtk_label_new(str); \
+	gtk_widget_show(label); \
+	gtk_box_pack_start(GTK_BOX(box), label, FALSE, TRUE, 0); \
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5); \
+	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT); \
+	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE); \
+	gtkut_widget_set_small_font_size(label); \
+}
+
 #define PACK_VSPACER(box, vbox, spacing) \
 { \
 	vbox = gtk_vbox_new(FALSE, 0); \
@@ -125,16 +145,17 @@ struct _PrefsDialog
 #define SET_TOGGLE_SENSITIVITY(togglewid, targetwid) \
 { \
 	gtk_widget_set_sensitive(targetwid, FALSE); \
-	gtk_signal_connect(GTK_OBJECT(togglewid), "toggled", \
-			   GTK_SIGNAL_FUNC(prefs_button_toggled), targetwid); \
+	g_signal_connect(G_OBJECT(togglewid), "toggled", \
+			 G_CALLBACK(prefs_button_toggled), targetwid); \
 }
 
 void prefs_read_config		(PrefParam	*param,
 				 const gchar	*label,
-				 const gchar	*rcfile);
+				 const gchar	*rcfile,
+				 const gchar	*encoding);
 void prefs_config_parse_one_line(PrefParam	*param,
 				 const gchar	*buf);
-void prefs_save_config		(PrefParam	*param,
+void prefs_write_config		(PrefParam	*param,
 				 const gchar	*label,
 				 const gchar	*rcfile);
 
