@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 
 #include "defs.h"
 
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
@@ -32,11 +34,11 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "intl.h"
 #include "main.h"
 #include "prefs.h"
 #include "prefs_account.h"
 #include "prefs_customheader.h"
+#include "prefs_common.h"
 #include "account.h"
 #include "mainwindow.h"
 #include "manage_window.h"
@@ -81,6 +83,7 @@ static struct Basic {
 
 static struct Receive {
 	GtkWidget *pop3_frame;
+	GtkWidget *use_apop_chkbtn;
 	GtkWidget *rmmail_chkbtn;
 	GtkWidget *leave_time_entry;
 	GtkWidget *getall_chkbtn;
@@ -94,6 +97,10 @@ static struct Receive {
 	GtkWidget *imap_frame;
 	GtkWidget *imap_auth_type_optmenu;
 
+	GtkWidget *nntp_frame;
+	GtkWidget *maxarticle_spinbtn;
+	GtkObject *maxarticle_spinbtn_adj;
+
 	GtkWidget *recvatgetall_chkbtn;
 } receive;
 
@@ -124,8 +131,9 @@ static struct Compose {
 
 #if USE_GPGME
 static struct Privacy {
-	GtkWidget *default_encrypt_chkbtn;
 	GtkWidget *default_sign_chkbtn;
+	GtkWidget *default_encrypt_chkbtn;
+	GtkWidget *encrypt_reply_chkbtn;
 	GtkWidget *ascii_armored_chkbtn;
 	GtkWidget *clearsign_chkbtn;
 	GtkWidget *defaultkey_radiobtn;
@@ -250,6 +258,10 @@ static PrefParam param[] = {
 	 &receive.inbox_entry, prefs_set_data_from_entry, prefs_set_entry},
 
 	/* Receive */
+	{"use_apop_auth", "FALSE", &tmp_ac_prefs.use_apop_auth, P_BOOL,
+	 &receive.use_apop_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+
 	{"remove_mail", "TRUE", &tmp_ac_prefs.rmmail, P_BOOL,
 	 &receive.rmmail_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
@@ -278,6 +290,10 @@ static PrefParam param[] = {
 	 prefs_account_imap_auth_type_set_data_from_optmenu,
 	 prefs_account_imap_auth_type_set_optmenu},
 
+	{"max_nntp_articles", "300", &tmp_ac_prefs.max_nntp_articles, P_INT,
+	 &receive.maxarticle_spinbtn,
+	 prefs_set_data_from_spinbtn, prefs_set_spinbtn},
+
 	{"receive_at_get_all", "TRUE", &tmp_ac_prefs.recv_at_getall, P_BOOL,
 	 &receive.recvatgetall_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
@@ -321,7 +337,8 @@ static PrefParam param[] = {
 	 &compose.sigfile_radiobtn,
 	 prefs_account_enum_set_data_from_radiobtn,
 	 prefs_account_enum_set_radiobtn},
-	{"signature_path", "~/"DEFAULT_SIGNATURE, &tmp_ac_prefs.sig_path, P_STRING,
+	{"signature_path", "~" G_DIR_SEPARATOR_S DEFAULT_SIGNATURE,
+	 &tmp_ac_prefs.sig_path, P_STRING,
 	 &compose.sigpath_entry,
 	 prefs_set_data_from_entry, prefs_set_entry},
 
@@ -351,11 +368,14 @@ static PrefParam param[] = {
 
 #if USE_GPGME
 	/* Privacy */
+	{"default_sign", "FALSE", &tmp_ac_prefs.default_sign, P_BOOL,
+	 &privacy.default_sign_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"default_encrypt", "FALSE", &tmp_ac_prefs.default_encrypt, P_BOOL,
 	 &privacy.default_encrypt_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
-	{"default_sign", "FALSE", &tmp_ac_prefs.default_sign, P_BOOL,
-	 &privacy.default_sign_chkbtn,
+	{"encrypt_reply", "TRUE", &tmp_ac_prefs.encrypt_reply, P_BOOL,
+	 &privacy.encrypt_reply_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"ascii_armored", "FALSE", &tmp_ac_prefs.ascii_armored, P_BOOL,
 	 &privacy.ascii_armored_chkbtn,
@@ -486,7 +506,7 @@ static void prefs_account_edit_custom_header	(void);
 static gint prefs_account_deleted		(GtkWidget	*widget,
 						 GdkEventAny	*event,
 						 gpointer	 data);
-static void prefs_account_key_pressed		(GtkWidget	*widget,
+static gboolean prefs_account_key_pressed	(GtkWidget	*widget,
 						 GdkEventKey	*event,
 						 gpointer	 data);
 static void prefs_account_ok			(void);
@@ -509,23 +529,34 @@ PrefsAccount *prefs_account_new(void)
 void prefs_account_read_config(PrefsAccount *ac_prefs, const gchar *label)
 {
 	const gchar *p = label;
+	gchar *rcpath;
 	gint id;
 
 	g_return_if_fail(ac_prefs != NULL);
 	g_return_if_fail(label != NULL);
 
 	memset(&tmp_ac_prefs, 0, sizeof(PrefsAccount));
-	prefs_read_config(param, label, ACCOUNT_RC);
+
+	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, ACCOUNT_RC, NULL);
+	prefs_read_config(param, label, rcpath, NULL);
+	g_free(rcpath);
+
 	*ac_prefs = tmp_ac_prefs;
-	while (*p && !isdigit(*p)) p++;
+	while (*p && !g_ascii_isdigit(*p)) p++;
 	id = atoi(p);
 	if (id < 0) g_warning("wrong account id: %d\n", id);
 	ac_prefs->account_id = id;
 
+	if (ac_prefs->protocol == A_APOP) {
+		debug_print("converting protocol A_APOP to new prefs.\n");
+		ac_prefs->protocol = A_POP3;
+		ac_prefs->use_apop_auth = TRUE;
+	}
+
 	prefs_custom_header_read_config(ac_prefs);
 }
 
-void prefs_account_save_config_all(GList *account_list)
+void prefs_account_write_config_all(GList *account_list)
 {
 	GList *cur;
 	gchar *rcpath;
@@ -603,8 +634,10 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 		prefs_account_create();
 	}
 
+	gtkut_box_set_reverse_order(GTK_BOX(dialog.confirm_area),
+				    !prefs_common.comply_gnome_hig);
 	manage_window_set_transient(GTK_WINDOW(dialog.window));
-	gtk_notebook_set_page(GTK_NOTEBOOK(dialog.notebook), 0);
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(dialog.notebook), 0);
 	gtk_widget_grab_focus(dialog.ok_btn);
 
 	tmp_ac_prefs = *ac_prefs;
@@ -665,18 +698,18 @@ static void prefs_account_create(void)
 
 	/* create dialog */
 	prefs_dialog_create(&dialog);
-	gtk_signal_connect(GTK_OBJECT(dialog.window), "delete_event",
-			   GTK_SIGNAL_FUNC(prefs_account_deleted), NULL);
-	gtk_signal_connect(GTK_OBJECT(dialog.window), "key_press_event",
-			   GTK_SIGNAL_FUNC(prefs_account_key_pressed), NULL);
+	g_signal_connect(G_OBJECT(dialog.window), "delete_event",
+			 G_CALLBACK(prefs_account_deleted), NULL);
+	g_signal_connect(G_OBJECT(dialog.window), "key_press_event",
+			 G_CALLBACK(prefs_account_key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(dialog.window);
 
-	gtk_signal_connect(GTK_OBJECT(dialog.ok_btn), "clicked",
-	 		   GTK_SIGNAL_FUNC(prefs_account_ok), NULL);
-	gtk_signal_connect(GTK_OBJECT(dialog.apply_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_account_apply), NULL);
-	gtk_signal_connect(GTK_OBJECT(dialog.cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_account_cancel), NULL);
+	g_signal_connect(G_OBJECT(dialog.ok_btn), "clicked",
+	 		 G_CALLBACK(prefs_account_ok), NULL);
+	g_signal_connect(G_OBJECT(dialog.apply_btn), "clicked",
+			 G_CALLBACK(prefs_account_apply), NULL);
+	g_signal_connect(G_OBJECT(dialog.cancel_btn), "clicked",
+			 G_CALLBACK(prefs_account_cancel), NULL);
 
 	prefs_account_basic_create();
 	SET_NOTEBOOK_LABEL(dialog.notebook, _("Basic"), page++);
@@ -700,9 +733,8 @@ static void prefs_account_create(void)
 
 #define SET_ACTIVATE(menuitem) \
 { \
-	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", \
-			   GTK_SIGNAL_FUNC(prefs_account_protocol_activated), \
-			   NULL); \
+	g_signal_connect(G_OBJECT(menuitem), "activate", \
+			 G_CALLBACK(prefs_account_protocol_activated), NULL); \
 }
 
 static void prefs_account_basic_create(void)
@@ -751,7 +783,7 @@ static void prefs_account_basic_create(void)
 
 	acname_entry = gtk_entry_new ();
 	gtk_widget_show (acname_entry);
-	gtk_widget_set_usize (acname_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (acname_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), acname_entry, TRUE, TRUE, 0);
 
 	default_chkbtn = gtk_check_button_new_with_label (_("Set as default"));
@@ -824,9 +856,7 @@ static void prefs_account_basic_create(void)
 
 	optmenu_menu = gtk_menu_new ();
 
-	MENUITEM_ADD (optmenu_menu, menuitem, _("POP3 (normal)"),  A_POP3);
-	SET_ACTIVATE (menuitem);
-	MENUITEM_ADD (optmenu_menu, menuitem, _("POP3 (APOP auth)"),  A_APOP);
+	MENUITEM_ADD (optmenu_menu, menuitem, _("POP3"),  A_POP3);
 	SET_ACTIVATE (menuitem);
 	MENUITEM_ADD (optmenu_menu, menuitem, _("IMAP4"), A_IMAP4);
 	SET_ACTIVATE (menuitem);
@@ -871,14 +901,14 @@ static void prefs_account_basic_create(void)
 
 	uid_entry = gtk_entry_new ();
 	gtk_widget_show (uid_entry);
-	gtk_widget_set_usize (uid_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (uid_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_table_attach (GTK_TABLE (serv_table), uid_entry, 1, 2, 5, 6,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
 	pass_entry = gtk_entry_new ();
 	gtk_widget_show (pass_entry);
-	gtk_widget_set_usize (pass_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (pass_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_table_attach (GTK_TABLE (serv_table), pass_entry, 3, 4, 5, 6,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
@@ -947,6 +977,7 @@ static void prefs_account_receive_create(void)
 	GtkWidget *vbox1;
 	GtkWidget *frame1;
 	GtkWidget *vbox2;
+	GtkWidget *use_apop_chkbtn;
 	GtkWidget *rmmail_chkbtn;
 	GtkWidget *hbox_spc;
 	GtkWidget *leave_time_label;
@@ -961,10 +992,18 @@ static void prefs_account_receive_create(void)
 	GtkWidget *inbox_label;
 	GtkWidget *inbox_entry;
 	GtkWidget *inbox_btn;
+
 	GtkWidget *imap_frame;
 	GtkWidget *optmenu;
 	GtkWidget *optmenu_menu;
 	GtkWidget *menuitem;
+
+	GtkWidget *nntp_frame;
+	GtkWidget *maxarticle_label;
+	GtkWidget *maxarticle_spinbtn;
+	GtkObject *maxarticle_spinbtn_adj;
+	GtkWidget *maxarticle_desc_label;
+
 	GtkWidget *recvatgetall_chkbtn;
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
@@ -979,6 +1018,9 @@ static void prefs_account_receive_create(void)
 	gtk_container_add (GTK_CONTAINER (frame1), vbox2);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
 
+	PACK_CHECK_BUTTON (vbox2, use_apop_chkbtn,
+			   _("Use secure authentication (APOP)"));
+
 	PACK_CHECK_BUTTON (vbox2, rmmail_chkbtn,
 			   _("Remove messages on server when received"));
 
@@ -989,7 +1031,7 @@ static void prefs_account_receive_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox1), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 12, -1);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 
 	leave_time_label = gtk_label_new (_("Remove after"));
 	gtk_widget_show (leave_time_label);
@@ -997,7 +1039,7 @@ static void prefs_account_receive_create(void)
 
 	leave_time_entry = gtk_entry_new ();
 	gtk_widget_show (leave_time_entry);
-	gtk_widget_set_usize (leave_time_entry, 64, -1);
+	gtk_widget_set_size_request (leave_time_entry, 64, -1);
 	gtk_box_pack_start (GTK_BOX (hbox1), leave_time_entry, FALSE, FALSE, 0);
 
 	leave_time_label = gtk_label_new (_("days"));
@@ -1015,7 +1057,7 @@ static void prefs_account_receive_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox1), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 12, -1);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 
 	leave_time_label = gtk_label_new (_("(0 days: remove immediately)"));
 	gtk_widget_show (leave_time_label);
@@ -1034,7 +1076,7 @@ static void prefs_account_receive_create(void)
 
 	size_limit_entry = gtk_entry_new ();
 	gtk_widget_show (size_limit_entry);
-	gtk_widget_set_usize (size_limit_entry, 64, -1);
+	gtk_widget_set_size_request (size_limit_entry, 64, -1);
 	gtk_box_pack_start (GTK_BOX (hbox1), size_limit_entry, FALSE, FALSE, 0);
 
 	label = gtk_label_new (_("KB"));
@@ -1058,27 +1100,21 @@ static void prefs_account_receive_create(void)
 
 	inbox_entry = gtk_entry_new ();
 	gtk_widget_show (inbox_entry);
-	gtk_widget_set_usize (inbox_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (inbox_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox1), inbox_entry, TRUE, TRUE, 0);
 
 	inbox_btn = gtk_button_new_with_label (_(" Select... "));
 	gtk_widget_show (inbox_btn);
 	gtk_box_pack_start (GTK_BOX (hbox1), inbox_btn, FALSE, FALSE, 0);
-	gtk_signal_connect (GTK_OBJECT (inbox_btn), "clicked",
-			    GTK_SIGNAL_FUNC (prefs_account_select_folder_cb),
-			    inbox_entry);
+	g_signal_connect (G_OBJECT (inbox_btn), "clicked",
+			  G_CALLBACK (prefs_account_select_folder_cb),
+			  inbox_entry);
 
 	PACK_VSPACER(vbox2, vbox3, VSPACING_NARROW_2);
 
-	hbox1 = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox1);
-	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);
-
-	label = gtk_label_new
-		(_("(Unfiltered messages will be stored in this folder)"));
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
-	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	PACK_SMALL_LABEL
+		(vbox2, label, 
+		 _("Unfiltered messages will be stored in this folder."));
 
 	PACK_FRAME (vbox1, imap_frame, _("IMAP4"));
 
@@ -1107,11 +1143,43 @@ static void prefs_account_receive_create(void)
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu), optmenu_menu);
 
+	PACK_FRAME (vbox1, nntp_frame, _("News"));
+
+	vbox2 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox2);
+	gtk_container_add (GTK_CONTAINER (nntp_frame), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
+
+	hbox1 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox1);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);
+
+	maxarticle_label = gtk_label_new
+		(_("Maximum number of articles to download"));
+	gtk_widget_show (maxarticle_label);
+	gtk_box_pack_start (GTK_BOX (hbox1), maxarticle_label, FALSE, FALSE, 0);
+	gtk_label_set_justify (GTK_LABEL (maxarticle_label), GTK_JUSTIFY_LEFT);
+
+	maxarticle_spinbtn_adj =
+		gtk_adjustment_new (300, 0, 10000, 10, 100, 100);
+	maxarticle_spinbtn = gtk_spin_button_new
+		(GTK_ADJUSTMENT (maxarticle_spinbtn_adj), 10, 0);
+	gtk_widget_show (maxarticle_spinbtn);
+	gtk_box_pack_start (GTK_BOX (hbox1), maxarticle_spinbtn,
+			    FALSE, FALSE, 0);
+	gtk_widget_set_size_request (maxarticle_spinbtn, 64, -1);
+	gtk_spin_button_set_numeric
+		(GTK_SPIN_BUTTON (maxarticle_spinbtn), TRUE);
+
+	PACK_SMALL_LABEL (vbox2, maxarticle_desc_label,
+			  _("No limit if 0 is specified."));
+
 	PACK_CHECK_BUTTON
 		(vbox1, recvatgetall_chkbtn,
 		 _("`Get all' checks for new messages on this account"));
 
 	receive.pop3_frame            = frame1;
+	receive.use_apop_chkbtn       = use_apop_chkbtn;
 	receive.rmmail_chkbtn         = rmmail_chkbtn;
 	receive.leave_time_entry      = leave_time_entry;
 	receive.getall_chkbtn         = getall_chkbtn;
@@ -1125,6 +1193,10 @@ static void prefs_account_receive_create(void)
 	receive.imap_frame             = imap_frame;
 	receive.imap_auth_type_optmenu = optmenu;
 
+	receive.nntp_frame             = nntp_frame;
+	receive.maxarticle_spinbtn     = maxarticle_spinbtn;
+	receive.maxarticle_spinbtn_adj = maxarticle_spinbtn_adj;
+
 	receive.recvatgetall_chkbtn = recvatgetall_chkbtn;
 }
 
@@ -1177,9 +1249,9 @@ static void prefs_account_send_create(void)
 	gtk_widget_show (customhdr_edit_btn);
 	gtk_box_pack_start (GTK_BOX (hbox), customhdr_edit_btn,
 			    FALSE, FALSE, 0);
-	gtk_signal_connect (GTK_OBJECT (customhdr_edit_btn), "clicked",
-			    GTK_SIGNAL_FUNC (prefs_account_edit_custom_header),
-			    NULL);
+	g_signal_connect (G_OBJECT (customhdr_edit_btn), "clicked",
+			  G_CALLBACK (prefs_account_edit_custom_header),
+			  NULL);
 
 	SET_TOGGLE_SENSITIVITY (customhdr_chkbtn, customhdr_edit_btn);
 
@@ -1204,7 +1276,7 @@ static void prefs_account_send_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 12, -1);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 
 	label = gtk_label_new (_("Authentication method"));
 	gtk_widget_show (label);
@@ -1217,6 +1289,7 @@ static void prefs_account_send_create(void)
 	optmenu_menu = gtk_menu_new ();
 
 	MENUITEM_ADD (optmenu_menu, menuitem, _("Automatic"), 0);
+	MENUITEM_ADD (optmenu_menu, menuitem, "PLAIN", SMTPAUTH_PLAIN);
 	MENUITEM_ADD (optmenu_menu, menuitem, "LOGIN", SMTPAUTH_LOGIN);
 	MENUITEM_ADD (optmenu_menu, menuitem, "CRAM-MD5", SMTPAUTH_CRAM_MD5);
 	MENUITEM_ADD (optmenu_menu, menuitem, "DIGEST-MD5", SMTPAUTH_DIGEST_MD5);
@@ -1233,7 +1306,7 @@ static void prefs_account_send_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 12, -1);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 
 	label = gtk_label_new (_("User ID"));
 	gtk_widget_show (label);
@@ -1241,7 +1314,7 @@ static void prefs_account_send_create(void)
 
 	smtp_uid_entry = gtk_entry_new ();
 	gtk_widget_show (smtp_uid_entry);
-	gtk_widget_set_usize (smtp_uid_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (smtp_uid_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), smtp_uid_entry, TRUE, TRUE, 0);
 
 	label = gtk_label_new (_("Password"));
@@ -1250,7 +1323,7 @@ static void prefs_account_send_create(void)
 
 	smtp_pass_entry = gtk_entry_new ();
 	gtk_widget_show (smtp_pass_entry);
-	gtk_widget_set_usize (smtp_pass_entry, DEFAULT_ENTRY_WIDTH, -1);
+	gtk_widget_set_size_request (smtp_pass_entry, DEFAULT_ENTRY_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), smtp_pass_entry, TRUE, TRUE, 0);
 	gtk_entry_set_visibility (GTK_ENTRY (smtp_pass_entry), FALSE);
 
@@ -1263,14 +1336,16 @@ static void prefs_account_send_create(void)
 	hbox_spc = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox_spc);
 	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 12, -1);
+	gtk_widget_set_size_request (hbox_spc, 12, -1);
 
 	label = gtk_label_new
-		(_("If you leave these entries empty, the same\n"
+		(_("If you leave these entries empty, the same "
 		   "user ID and password as receiving will be used."));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtkut_widget_set_small_font_size (label);
 
 	SET_TOGGLE_SENSITIVITY (smtp_auth_chkbtn, vbox4);
 
@@ -1328,16 +1403,16 @@ static void prefs_account_compose_create(void)
 	gtk_widget_show (sigfile_radiobtn);
 	gtk_box_pack_start (GTK_BOX (sig_hbox), sigfile_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (sigfile_radiobtn),
-				  GINT_TO_POINTER (SIG_FILE));
+	g_object_set_data (G_OBJECT (sigfile_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIG_FILE));
 
 	sigcmd_radiobtn = gtk_radio_button_new_with_label_from_widget
 		(GTK_RADIO_BUTTON(sigfile_radiobtn), _("Command output"));
 	gtk_widget_show (sigcmd_radiobtn);
 	gtk_box_pack_start (GTK_BOX (sig_hbox), sigcmd_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (sigcmd_radiobtn),
-				  GINT_TO_POINTER (SIG_COMMAND));
+	g_object_set_data (G_OBJECT (sigcmd_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIG_COMMAND));
 
 	sigpath_entry = gtk_entry_new ();
 	gtk_widget_show (sigpath_entry);
@@ -1411,8 +1486,9 @@ static void prefs_account_privacy_create(void)
 	GtkWidget *vbox2;
 	GtkWidget *hbox1;
 	GtkWidget *label;
-	GtkWidget *default_encrypt_chkbtn;
 	GtkWidget *default_sign_chkbtn;
+	GtkWidget *default_encrypt_chkbtn;
+	GtkWidget *encrypt_reply_chkbtn;
 	GtkWidget *ascii_armored_chkbtn;
 	GtkWidget *clearsign_chkbtn;
 	GtkWidget *defaultkey_radiobtn;
@@ -1429,16 +1505,19 @@ static void prefs_account_privacy_create(void)
 	gtk_widget_show (vbox2);
 	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);
 
-	PACK_CHECK_BUTTON (vbox2, default_encrypt_chkbtn,
-			   _("Encrypt message by default"));
 	PACK_CHECK_BUTTON (vbox2, default_sign_chkbtn,
 			   _("Sign message by default"));
+	PACK_CHECK_BUTTON (vbox2, default_encrypt_chkbtn,
+			   _("Encrypt message by default"));
+	PACK_CHECK_BUTTON (vbox2, encrypt_reply_chkbtn,
+			   _("Encrypt when replying to encrypted message"));
 	PACK_CHECK_BUTTON (vbox2, ascii_armored_chkbtn,
 			   _("Use ASCII-armored format for encryption"));
 	PACK_CHECK_BUTTON (vbox2, clearsign_chkbtn,
 			   _("Use clear text signature"));
-	gtk_signal_connect (GTK_OBJECT (ascii_armored_chkbtn), "toggled",
-			    prefs_account_ascii_armored_warning, NULL);
+	g_signal_connect (G_OBJECT (ascii_armored_chkbtn), "toggled",
+			  G_CALLBACK (prefs_account_ascii_armored_warning),
+			  NULL);
 
 	PACK_FRAME (vbox1, frame1, _("Sign key"));
 
@@ -1452,8 +1531,8 @@ static void prefs_account_privacy_create(void)
 	gtk_widget_show (defaultkey_radiobtn);
 	gtk_box_pack_start (GTK_BOX (vbox2), defaultkey_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (defaultkey_radiobtn),
-				  GINT_TO_POINTER (SIGN_KEY_DEFAULT));
+	g_object_set_data (G_OBJECT (defaultkey_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIGN_KEY_DEFAULT));
 
 	emailkey_radiobtn = gtk_radio_button_new_with_label_from_widget
 		(GTK_RADIO_BUTTON (defaultkey_radiobtn),
@@ -1461,8 +1540,8 @@ static void prefs_account_privacy_create(void)
 	gtk_widget_show (emailkey_radiobtn);
 	gtk_box_pack_start (GTK_BOX (vbox2), emailkey_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (emailkey_radiobtn),
-				  GINT_TO_POINTER (SIGN_KEY_BY_FROM));
+	g_object_set_data (G_OBJECT (emailkey_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIGN_KEY_BY_FROM));
 
 	customkey_radiobtn = gtk_radio_button_new_with_label_from_widget
 		(GTK_RADIO_BUTTON (defaultkey_radiobtn),
@@ -1470,8 +1549,8 @@ static void prefs_account_privacy_create(void)
 	gtk_widget_show (customkey_radiobtn);
 	gtk_box_pack_start (GTK_BOX (vbox2), customkey_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (customkey_radiobtn),
-				  GINT_TO_POINTER (SIGN_KEY_CUSTOM));
+	g_object_set_data (G_OBJECT (customkey_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SIGN_KEY_CUSTOM));
 
 	hbox1 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox1);
@@ -1480,7 +1559,7 @@ static void prefs_account_privacy_create(void)
 	label = gtk_label_new ("");
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
-	gtk_widget_set_usize (label, 16, -1);
+	gtk_widget_set_size_request (label, 16, -1);
 
 	label = gtk_label_new (_("User or key ID:"));
 	gtk_widget_show (label);
@@ -1493,8 +1572,9 @@ static void prefs_account_privacy_create(void)
 
 	SET_TOGGLE_SENSITIVITY (customkey_radiobtn, customkey_entry);
 
-	privacy.default_encrypt_chkbtn = default_encrypt_chkbtn;
 	privacy.default_sign_chkbtn    = default_sign_chkbtn;
+	privacy.default_encrypt_chkbtn = default_encrypt_chkbtn;
+	privacy.encrypt_reply_chkbtn   = encrypt_reply_chkbtn;
 	privacy.ascii_armored_chkbtn   = ascii_armored_chkbtn;
 	privacy.clearsign_chkbtn       = clearsign_chkbtn;
 	privacy.defaultkey_radiobtn    = defaultkey_radiobtn;
@@ -1512,8 +1592,8 @@ static void prefs_account_privacy_create(void)
 		(GTK_RADIO_BUTTON (btn_p), label);			\
 	gtk_widget_show (btn);						\
 	gtk_box_pack_start (GTK_BOX (box), btn, FALSE, FALSE, 0);	\
-	gtk_object_set_user_data (GTK_OBJECT (btn),			\
-				  GINT_TO_POINTER (data));		\
+	g_object_set_data (G_OBJECT (btn), MENU_VAL_ID,			\
+			   GINT_TO_POINTER (data));			\
 }
 
 #define CREATE_RADIO_BUTTONS(box,					\
@@ -1524,8 +1604,8 @@ static void prefs_account_privacy_create(void)
 	btn1 = gtk_radio_button_new_with_label(NULL, btn1_label);	\
 	gtk_widget_show (btn1);						\
 	gtk_box_pack_start (GTK_BOX (box), btn1, FALSE, FALSE, 0);	\
-	gtk_object_set_user_data (GTK_OBJECT (btn1),			\
-				  GINT_TO_POINTER (btn1_data));		\
+	g_object_set_data (G_OBJECT (btn1), MENU_VAL_ID,		\
+			   GINT_TO_POINTER (btn1_data));		\
 									\
 	CREATE_RADIO_BUTTON(box, btn2, btn1, btn2_label, btn2_data);	\
 	CREATE_RADIO_BUTTON(box, btn3, btn1, btn3_label, btn3_data);	\
@@ -1560,8 +1640,6 @@ static void prefs_account_ssl_create(void)
 
 	GtkWidget *vbox6;
 	GtkWidget *use_nonblocking_ssl_chkbtn;
-	GtkWidget *hbox;
-	GtkWidget *hbox_spc;
 	GtkWidget *label;
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
@@ -1614,8 +1692,8 @@ static void prefs_account_ssl_create(void)
 	gtk_widget_show (nntp_nossl_radiobtn);
 	gtk_box_pack_start (GTK_BOX (vbox4), nntp_nossl_radiobtn,
 			    FALSE, FALSE, 0);
-	gtk_object_set_user_data (GTK_OBJECT (nntp_nossl_radiobtn),
-				  GINT_TO_POINTER (SSL_NONE));
+	g_object_set_data (G_OBJECT (nntp_nossl_radiobtn), MENU_VAL_ID,
+			   GINT_TO_POINTER (SSL_NONE));
 
 	CREATE_RADIO_BUTTON(vbox4, nntp_ssltunnel_radiobtn, nntp_nossl_radiobtn,
 			    _("Use SSL for NNTP connection"), SSL_TUNNEL);
@@ -1643,20 +1721,9 @@ static void prefs_account_ssl_create(void)
 
 	PACK_CHECK_BUTTON(vbox6, use_nonblocking_ssl_chkbtn,
 			  _("Use non-blocking SSL"));
-
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox6), hbox, FALSE, FALSE, 0);
-
-	hbox_spc = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox_spc);
-	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_usize (hbox_spc, 16, -1);
-
-	label = gtk_label_new
-		(_("(Turn this off if you have problems in SSL connection)"));
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	PACK_SMALL_LABEL
+		(vbox6, label,
+		 _("Turn this off if you have problems in SSL connection."));
 
 	ssl.pop_frame               = pop_frame;
 	ssl.pop_nossl_radiobtn      = pop_nossl_radiobtn;
@@ -1725,10 +1792,11 @@ static void prefs_account_advanced_create(void)
 
 #define PACK_PORT_ENTRY(box, entry) \
 { \
-	entry = gtk_entry_new_with_max_length (5); \
+	entry = gtk_entry_new (); \
+	gtk_entry_set_max_length (GTK_ENTRY(entry), 5); \
 	gtk_widget_show (entry); \
 	gtk_box_pack_start (GTK_BOX (box), entry, FALSE, FALSE, 0); \
-	gtk_widget_set_usize (entry, 64, -1); \
+	gtk_widget_set_size_request (entry, 64, -1); \
 }
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
@@ -1827,9 +1895,9 @@ static void prefs_account_advanced_create(void)
 	gtk_widget_show (button);					\
 	gtk_table_attach (GTK_TABLE (table), button,			\
 			  2, 3, n, n + 1, GTK_FILL, 0, 0, 0);		\
-	gtk_signal_connect						\
-		(GTK_OBJECT (button), "clicked",			\
-		 GTK_SIGNAL_FUNC (prefs_account_select_folder_cb),	\
+	g_signal_connect						\
+		(G_OBJECT (button), "clicked",				\
+		 G_CALLBACK (prefs_account_select_folder_cb),		\
 		 entry);						\
 									\
 	SET_TOGGLE_SENSITIVITY (chkbtn, entry);				\
@@ -1875,11 +1943,12 @@ static gint prefs_account_deleted(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
-static void prefs_account_key_pressed(GtkWidget *widget, GdkEventKey *event,
-				      gpointer data)
+static gboolean prefs_account_key_pressed(GtkWidget *widget, GdkEventKey *event,
+					  gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		prefs_account_cancel();
+	return FALSE;
 }
 
 static void prefs_account_ok(void)
@@ -1897,7 +1966,7 @@ static gint prefs_account_apply(void)
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(basic.protocol_optmenu));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 	protocol = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 
 	if (*gtk_entry_get_text(GTK_ENTRY(basic.acname_entry)) == '\0') {
 		alertpanel_error(_("Account name is not entered."));
@@ -1907,17 +1976,17 @@ static gint prefs_account_apply(void)
 		alertpanel_error(_("Mail address is not entered."));
 		return -1;
 	}
-	if ((protocol == A_POP3 || protocol == A_APOP || protocol == A_LOCAL) &&
+	if ((protocol == A_POP3 || protocol == A_LOCAL) &&
 	    *gtk_entry_get_text(GTK_ENTRY(basic.smtpserv_entry)) == '\0') {
 		alertpanel_error(_("SMTP server is not entered."));
 		return -1;
 	}
-	if ((protocol == A_POP3 || protocol == A_APOP || protocol == A_IMAP4) &&
+	if ((protocol == A_POP3 || protocol == A_IMAP4) &&
 	    *gtk_entry_get_text(GTK_ENTRY(basic.uid_entry)) == '\0') {
 		alertpanel_error(_("User ID is not entered."));
 		return -1;
 	}
-	if ((protocol == A_POP3 || protocol == A_APOP) &&
+	if (protocol == A_POP3 &&
 	    *gtk_entry_get_text(GTK_ENTRY(basic.recvserv_entry)) == '\0') {
 		alertpanel_error(_("POP3 server is not entered."));
 		return -1;
@@ -1969,12 +2038,13 @@ static void prefs_account_enum_set_data_from_radiobtn(PrefParam *pparam)
 	GSList *group;
 
 	radiobtn = GTK_RADIO_BUTTON (*pparam->widget);
-	group = gtk_radio_button_group (radiobtn);
+	group = gtk_radio_button_get_group (radiobtn);
 	while (group != NULL) {
 		GtkToggleButton *btn = GTK_TOGGLE_BUTTON (group->data);
 		if (gtk_toggle_button_get_active (btn)) {
 			*((gint *)pparam->data) = GPOINTER_TO_INT
-				(gtk_object_get_user_data (GTK_OBJECT (btn)));
+				(g_object_get_data (G_OBJECT (btn),
+						    MENU_VAL_ID));
 			break;
 		}
 		group = group->next;
@@ -1989,10 +2059,12 @@ static void prefs_account_enum_set_radiobtn(PrefParam *pparam)
 
 	data = GINT_TO_POINTER (*((gint *)pparam->data));
 	radiobtn = GTK_RADIO_BUTTON (*pparam->widget);
-	group = gtk_radio_button_group (radiobtn);
+	group = gtk_radio_button_get_group (radiobtn);
 	while (group != NULL) {
 		GtkToggleButton *btn = GTK_TOGGLE_BUTTON (group->data);
-		gpointer data1 = gtk_object_get_user_data (GTK_OBJECT (btn));
+		gpointer data1;
+
+		data1 = g_object_get_data (G_OBJECT (btn), MENU_VAL_ID);
 		if (data1 == data) {
 			gtk_toggle_button_set_active (btn, TRUE);
 			break;
@@ -2022,7 +2094,7 @@ static void prefs_account_protocol_set_data_from_optmenu(PrefParam *pparam)
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(*pparam->widget));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 	*((RecvProtocol *)pparam->data) = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 }
 
 static void prefs_account_protocol_set_optmenu(PrefParam *pparam)
@@ -2031,42 +2103,19 @@ static void prefs_account_protocol_set_optmenu(PrefParam *pparam)
 	GtkOptionMenu *optmenu = GTK_OPTION_MENU(*pparam->widget);
 	GtkWidget *menu;
 	GtkWidget *menuitem;
-	GList *children;
-	gint list_order[] = {
-		0,  /* A_POP3  */
-		1,  /* A_APOP  */
-		-1, /* A_RPOP  */
-		2,  /* A_IMAP4 */
-		3,  /* A_NNTP  */
-		4   /* A_LOCAL */
-	};
+	gint index;
 
 	protocol = *((RecvProtocol *)pparam->data);
-	if (protocol < 0 || protocol > A_LOCAL) return;
-	if (list_order[protocol] < 0) return;
-	gtk_option_menu_set_history(optmenu, list_order[protocol]);
+	index = menu_find_option_menu_index
+		(optmenu, GINT_TO_POINTER(protocol), NULL);
+	if (index < 0) return;
+	gtk_option_menu_set_history(optmenu, index);
 
 	menu = gtk_option_menu_get_menu(optmenu);
 	menu_set_insensitive_all(GTK_MENU_SHELL(menu));
 
-#define SET_NTH_SENSITIVE(proto) \
-{ \
-	menuitem = g_list_nth_data(children, list_order[proto]); \
-	if (menuitem) \
-		gtk_widget_set_sensitive(menuitem, TRUE); \
-}
-
-	children = GTK_MENU_SHELL(menu)->children;
-	SET_NTH_SENSITIVE(protocol);
-	if (protocol == A_POP3) {
-		SET_NTH_SENSITIVE(A_APOP);
-	} else if (protocol == A_APOP) {
-		SET_NTH_SENSITIVE(A_POP3);
-	}
-
-#undef SET_NTH_SENSITIVE
-
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
+	gtk_widget_set_sensitive(menuitem, TRUE);
 	gtk_menu_item_activate(GTK_MENU_ITEM(menuitem));
 }
 
@@ -2078,7 +2127,7 @@ static void prefs_account_imap_auth_type_set_data_from_optmenu(PrefParam *pparam
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(*pparam->widget));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 	*((RecvProtocol *)pparam->data) = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 }
 
 static void prefs_account_imap_auth_type_set_optmenu(PrefParam *pparam)
@@ -2113,7 +2162,7 @@ static void prefs_account_smtp_auth_type_set_data_from_optmenu(PrefParam *pparam
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(*pparam->widget));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
 	*((RecvProtocol *)pparam->data) = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 }
 
 static void prefs_account_smtp_auth_type_set_optmenu(PrefParam *pparam)
@@ -2124,15 +2173,18 @@ static void prefs_account_smtp_auth_type_set_optmenu(PrefParam *pparam)
 	GtkWidget *menuitem;
 
 	switch (type) {
-	case SMTPAUTH_LOGIN:
+	case SMTPAUTH_PLAIN:
 		gtk_option_menu_set_history(optmenu, 1);
 		break;
-	case SMTPAUTH_CRAM_MD5:
+	case SMTPAUTH_LOGIN:
 		gtk_option_menu_set_history(optmenu, 2);
 		break;
-	case SMTPAUTH_DIGEST_MD5:
+	case SMTPAUTH_CRAM_MD5:
 		gtk_option_menu_set_history(optmenu, 3);
 		break;
+	case SMTPAUTH_DIGEST_MD5:
+		gtk_option_menu_set_history(optmenu, 4);
+		break;
 	case 0:
 	default:
 		gtk_option_menu_set_history(optmenu, 0);
@@ -2149,7 +2201,7 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 	gboolean active;
 
 	protocol = GPOINTER_TO_INT
-		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 
 	switch(protocol) {
 	case A_NNTP:
@@ -2168,6 +2220,7 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_entry, active);
 		gtk_widget_hide(receive.pop3_frame);
 		gtk_widget_hide(receive.imap_frame);
+		gtk_widget_show(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
 		if (!tmp_ac_prefs.account_name) {
@@ -2203,6 +2256,7 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_entry, FALSE);
 		gtk_widget_hide(receive.pop3_frame);
 		gtk_widget_hide(receive.imap_frame);
+		gtk_widget_hide(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, FALSE);
 
 		if (!tmp_ac_prefs.account_name) {
@@ -2238,6 +2292,7 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_entry, TRUE);
 		gtk_widget_hide(receive.pop3_frame);
 		gtk_widget_show(receive.imap_frame);
+		gtk_widget_hide(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
 		if (!tmp_ac_prefs.account_name) {
@@ -2274,6 +2329,7 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_entry, TRUE);
 		gtk_widget_show(receive.pop3_frame);
 		gtk_widget_hide(receive.imap_frame);
+		gtk_widget_hide(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
 		if (!tmp_ac_prefs.account_name) {
