@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto & The Sylpheed Claws Team
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto & The Sylpheed Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
@@ -31,7 +32,6 @@
 #include <string.h>
 #include <errno.h>
 
-#include "intl.h"
 #include "prefs.h"
 #include "inc.h"
 #include "utils.h"
@@ -47,6 +47,7 @@ static struct Actions
 {
 	GtkWidget *window;
 
+	GtkWidget *confirm_area;
 	GtkWidget *ok_btn;
 
 	GtkWidget *name_entry;
@@ -83,9 +84,9 @@ static void prefs_actions_row_move	(GtkCList	*clist,
 static gint prefs_actions_deleted	(GtkWidget	*widget,
 					 GdkEventAny	*event,
 					 gpointer	*data);
-static void prefs_actions_key_pressed	(GtkWidget	*widget,
-					 GdkEventKey	*event,
-					 gpointer	 data);
+static gboolean prefs_actions_key_pressed	(GtkWidget	*widget,
+						 GdkEventKey	*event,
+						 gpointer	 data);
 static void prefs_actions_cancel	(GtkWidget	*w,
 					 gpointer	 data);
 static void prefs_actions_ok		(GtkWidget	*w,
@@ -99,6 +100,8 @@ void prefs_actions_open(MainWindow *mainwin)
 	if (!actions.window)
 		prefs_actions_create(mainwin);
 
+	gtkut_box_set_reverse_order(GTK_BOX(actions.confirm_area),
+				    !prefs_common.comply_gnome_hig);
 	manage_window_set_transient(GTK_WINDOW(actions.window));
 	gtk_widget_grab_focus(actions.ok_btn);
 
@@ -147,10 +150,10 @@ static void prefs_actions_create(MainWindow *mainwin)
 
 	debug_print("Creating actions configuration window...\n");
 
-	window = gtk_window_new (GTK_WINDOW_DIALOG);
+	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
 	gtk_container_set_border_width(GTK_CONTAINER (window), 8);
-	gtk_window_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, TRUE);
 	gtk_window_set_default_size(GTK_WINDOW(window), 400, -1);
@@ -159,22 +162,23 @@ static void prefs_actions_create(MainWindow *mainwin)
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
-	gtkut_button_set_create(&confirm_area, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
 	gtk_window_set_title(GTK_WINDOW(window), _("Actions configuration"));
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(prefs_actions_deleted), NULL);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(prefs_actions_key_pressed), NULL);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(prefs_actions_deleted), NULL);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(prefs_actions_key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_ok), mainwin);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_cancel), NULL);
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(prefs_actions_ok), mainwin);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(prefs_actions_cancel), NULL);
 
 	vbox1 = gtk_vbox_new(FALSE, 8);
 	gtk_widget_show(vbox1);
@@ -241,7 +245,7 @@ static void prefs_actions_create(MainWindow *mainwin)
 	arrow = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
 	gtk_widget_show(arrow);
 	gtk_box_pack_start(GTK_BOX(reg_hbox), arrow, FALSE, FALSE, 0);
-	gtk_widget_set_usize(arrow, -1, 16);
+	gtk_widget_set_size_request(arrow, -1, 16);
 
 	btn_hbox = gtk_hbox_new(TRUE, 4);
 	gtk_widget_show(btn_hbox);
@@ -250,27 +254,26 @@ static void prefs_actions_create(MainWindow *mainwin)
 	reg_btn = gtk_button_new_with_label(_("Add"));
 	gtk_widget_show(reg_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), reg_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(reg_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_register_cb), NULL);
+	g_signal_connect(G_OBJECT(reg_btn), "clicked",
+			 G_CALLBACK(prefs_actions_register_cb), NULL);
 
 	subst_btn = gtk_button_new_with_label(_(" Replace "));
 	gtk_widget_show(subst_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), subst_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(subst_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_substitute_cb),
-			   NULL);
+	g_signal_connect(G_OBJECT(subst_btn), "clicked",
+			 G_CALLBACK(prefs_actions_substitute_cb), NULL);
 
 	del_btn = gtk_button_new_with_label(_("Delete"));
 	gtk_widget_show(del_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), del_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(del_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_delete_cb), NULL);
+	g_signal_connect(G_OBJECT(del_btn), "clicked",
+			 G_CALLBACK(prefs_actions_delete_cb), NULL);
 
 	help_toggle = gtk_toggle_button_new_with_label(_(" Syntax help "));
 	gtk_widget_show(help_toggle);
 	gtk_box_pack_end(GTK_BOX(reg_hbox), help_toggle, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(help_toggle), "toggled",
-			   GTK_SIGNAL_FUNC(prefs_actions_help_cb), help_vbox);
+	g_signal_connect(G_OBJECT(help_toggle), "toggled",
+			 G_CALLBACK(prefs_actions_help_cb), help_vbox);
 
 	cond_hbox = gtk_hbox_new(FALSE, 8);
 	gtk_widget_show(cond_hbox);
@@ -278,7 +281,7 @@ static void prefs_actions_create(MainWindow *mainwin)
 
 	cond_scrolledwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(cond_scrolledwin);
-	gtk_widget_set_usize(cond_scrolledwin, -1, 150);
+	gtk_widget_set_size_request(cond_scrolledwin, -1, 150);
 	gtk_box_pack_start(GTK_BOX(cond_hbox), cond_scrolledwin,
 			   TRUE, TRUE, 0);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (cond_scrolledwin),
@@ -294,11 +297,10 @@ static void prefs_actions_create(MainWindow *mainwin)
 				     GTK_SELECTION_BROWSE);
 	GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(cond_clist)->column[0].button,
 			       GTK_CAN_FOCUS);
-	gtk_signal_connect(GTK_OBJECT(cond_clist), "select_row",
-			   GTK_SIGNAL_FUNC(prefs_actions_select), NULL);
-	gtk_signal_connect_after(GTK_OBJECT(cond_clist), "row_move",
-				 GTK_SIGNAL_FUNC(prefs_actions_row_move),
-				 NULL);
+	g_signal_connect(G_OBJECT(cond_clist), "select_row",
+			 G_CALLBACK(prefs_actions_select), NULL);
+	g_signal_connect_after(G_OBJECT(cond_clist), "row_move",
+			       G_CALLBACK(prefs_actions_row_move), NULL);
 
 	btn_vbox = gtk_vbox_new(FALSE, 8);
 	gtk_widget_show(btn_vbox);
@@ -307,18 +309,20 @@ static void prefs_actions_create(MainWindow *mainwin)
 	up_btn = gtk_button_new_with_label(_("Up"));
 	gtk_widget_show(up_btn);
 	gtk_box_pack_start(GTK_BOX(btn_vbox), up_btn, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(up_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_up), NULL);
+	g_signal_connect(G_OBJECT(up_btn), "clicked",
+			 G_CALLBACK(prefs_actions_up), NULL);
 
 	down_btn = gtk_button_new_with_label(_("Down"));
 	gtk_widget_show(down_btn);
 	gtk_box_pack_start(GTK_BOX(btn_vbox), down_btn, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(down_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_down), NULL);
+	g_signal_connect(G_OBJECT(down_btn), "clicked",
+			 G_CALLBACK(prefs_actions_down), NULL);
 
 	gtk_widget_show(window);
 
 	actions.window = window;
+
+	actions.confirm_area = confirm_area;
 	actions.ok_btn = ok_btn;
 
 	actions.name_entry = name_entry;
@@ -345,7 +349,7 @@ void prefs_actions_read_config(void)
 	debug_print("Reading actions configurations...\n");
 
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, ACTIONS_RC, NULL);
-	if ((fp = fopen(rcpath, "rb")) == NULL) {
+	if ((fp = g_fopen(rcpath, "rb")) == NULL) {
 		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
 		g_free(rcpath);
 		return;
@@ -452,7 +456,7 @@ static void prefs_actions_set_list(void)
 static gint prefs_actions_clist_set_row(gint row)
 {
 	GtkCList *clist = GTK_CLIST(actions.actions_clist);
-	gchar *entry_text;
+	const gchar *entry_text;
 	gint len;
 	gchar action[PREFSBUFSIZE];
 	gchar *buf[1];
@@ -558,7 +562,7 @@ static void prefs_actions_delete_cb(GtkWidget *w, gpointer data)
 
 	if (alertpanel(_("Delete action"),
 		       _("Do you really want to delete this action?"),
-		       _("Yes"), _("No"), NULL) == G_ALERTALTERNATE)
+		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
 		return;
 
 	action = gtk_clist_get_row_data(clist, row);
@@ -639,11 +643,12 @@ static gint prefs_actions_deleted(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
-static void prefs_actions_key_pressed(GtkWidget *widget, GdkEventKey *event,
-				      gpointer data)
+static gboolean prefs_actions_key_pressed(GtkWidget *widget, GdkEventKey *event,
+					  gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		prefs_actions_cancel(widget, data);
+	return FALSE;
 }
 
 static void prefs_actions_cancel(GtkWidget *w, gpointer data)
