@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,12 +58,14 @@ struct _PrefsCommon
 	gboolean autochk_newmail;
 	gint autochk_itv;
 	gboolean chk_on_startup;
-	gint max_articles;
+	gboolean enable_newmsg_notify;
+	gchar *newmsg_notify_cmd;
 
 	/* Send */
 	gboolean use_extsend;
 	gchar *extsend_cmd;
 	gboolean savemsg;
+	gboolean filter_sent;
 	gchar *outgoing_charset;
 	TransferEncodingMethod encoding_method;
 
@@ -90,11 +92,7 @@ struct _PrefsCommon
 	gchar *fw_quotefmt;
 
 	/* Display */
-	gchar *widgetfont;
 	gchar *textfont;
-	gchar *normalfont;
-	gchar *boldfont;
-	gchar *smallfont;
 
 	gboolean trans_hdr;
 	gboolean display_folder_unread;
@@ -104,9 +102,8 @@ struct _PrefsCommon
 	gboolean expand_thread;
 	gchar *date_format;
 
-	gboolean enable_hscrollbar;
+	gboolean enable_rules_hint;
 	gboolean bold_unread;
-	gboolean enable_thread;
 
 	ToolbarStyle toolbar_style;
 	gboolean show_statusbar;
@@ -168,13 +165,15 @@ struct _PrefsCommon
 	gboolean conv_mb_alnum;
 	gboolean display_header_pane;
 	gboolean display_header;
-	gboolean head_space;
 	gint line_space;
+	gboolean render_html;
+	gboolean textview_cursor_visible;
 	gboolean enable_smooth_scroll;
 	gint scroll_step;
 	gboolean scroll_halfpage;
 
 	gboolean resize_image;
+	gboolean inline_image;
 
 	gchar *force_charset;
 
@@ -188,6 +187,14 @@ struct _PrefsCommon
 
 	GList *mime_open_cmd_history;
 
+	/* Junk Mail */
+	gboolean enable_junk;
+	gchar *junk_learncmd;
+	gchar *nojunk_learncmd;
+	gchar *junk_classify_cmd;
+	gchar *junk_folder;
+	gboolean filter_junk_on_recv;
+
 #if USE_GPGME
 	/* Privacy */
 	gboolean auto_check_signatures;
@@ -201,31 +208,37 @@ struct _PrefsCommon
 	/* Interface */
 	gboolean sep_folder;
 	gboolean sep_msg;
-	gboolean emulate_emacs;
 	gboolean always_show_msg;
 	gboolean open_unread_on_enter;
 	gboolean mark_as_read_on_new_window;
 	gboolean open_inbox_on_inc;
 	gboolean immediate_exec;
 	RecvDialogMode recv_dialog_mode;
-	gboolean close_recv_dialog;
 	gboolean no_recv_err_panel;
-	gboolean add_address_by_click;
+	gboolean close_recv_dialog;
+	gboolean comply_gnome_hig;
 
 	/* Other */
 	gchar *uri_cmd;
 	gchar *print_cmd;
 	gchar *ext_editor_cmd;
 
+	gboolean add_address_by_click;
+
 	gboolean confirm_on_exit;
 	gboolean clean_on_exit;
 	gboolean ask_on_clean;
 	gboolean warn_queued_on_exit;
 
+	gint logwin_line_limit;
+
+	/* Advanced */
+	gboolean strict_cache_check;
 	gint io_timeout_secs;
 
 	/* Filtering */
 	GSList *fltlist;
+	GSList *junk_fltlist;
 
 	/* Actions */
 	GSList *actions_list;
