@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
@@ -31,7 +32,6 @@
 #include <string.h>
 #include <errno.h>
 
-#include "intl.h"
 #include "prefs.h"
 #include "prefs_display_header.h"
 #include "prefs_common.h"
@@ -44,6 +44,7 @@
 static struct DisplayHeader {
 	GtkWidget *window;
 
+	GtkWidget *confirm_area;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
 
@@ -76,7 +77,7 @@ static void prefs_display_header_row_moved	(GtkCList	*clist,
 						 gint		 dest_row,
 						 gpointer	 data);
 
-static void prefs_display_header_key_pressed	(GtkWidget	*widget,
+static gboolean prefs_display_header_key_pressed(GtkWidget	*widget,
 						 GdkEventKey	*event,
 						 gpointer	 data);
 static void prefs_display_header_ok		(void);
@@ -132,6 +133,8 @@ void prefs_display_header_open(void)
 		prefs_display_header_create();
 	}
 
+	gtkut_box_set_reverse_order(GTK_BOX(dispheader.confirm_area),
+				    !prefs_common.comply_gnome_hig);
 	manage_window_set_transient(GTK_WINDOW(dispheader.window));
 	gtk_widget_grab_focus(dispheader.ok_btn);
 
@@ -145,9 +148,9 @@ static void prefs_display_header_create(void)
 	GtkWidget *window;
 	GtkWidget *vbox;
 	GtkWidget *btn_hbox;
+	GtkWidget *confirm_area;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
-	GtkWidget *confirm_area;
 
 	GtkWidget *vbox1;
 
@@ -174,9 +177,9 @@ static void prefs_display_header_create(void)
 
 	debug_print(_("Creating display header setting window...\n"));
 
-	window = gtk_window_new (GTK_WINDOW_DIALOG);
+	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
-	gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
+	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
 	gtk_window_set_policy (GTK_WINDOW (window), FALSE, TRUE, FALSE);
 
@@ -188,8 +191,9 @@ static void prefs_display_header_create(void)
 	gtk_widget_show (btn_hbox);
 	gtk_box_pack_end (GTK_BOX (vbox), btn_hbox, FALSE, FALSE, 0);
 
-	gtkut_button_set_create(&confirm_area, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_widget_show (confirm_area);
 	gtk_box_pack_end (GTK_BOX(btn_hbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default (ok_btn);
@@ -197,18 +201,14 @@ static void prefs_display_header_create(void)
 	gtk_window_set_title (GTK_WINDOW(window),
 			      _("Display header setting"));
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
-	gtk_signal_connect (GTK_OBJECT(window), "delete_event",
-			    GTK_SIGNAL_FUNC(prefs_display_header_deleted),
-			    NULL);
-	gtk_signal_connect (GTK_OBJECT(window), "key_press_event",
-			    GTK_SIGNAL_FUNC(prefs_display_header_key_pressed),
-			    NULL);
-	gtk_signal_connect (GTK_OBJECT(ok_btn), "clicked",
-			    GTK_SIGNAL_FUNC(prefs_display_header_ok),
-			    NULL);
-	gtk_signal_connect (GTK_OBJECT(cancel_btn), "clicked",
-			    GTK_SIGNAL_FUNC(prefs_display_header_cancel),
-			    NULL);
+	g_signal_connect (G_OBJECT(window), "delete_event",
+			  G_CALLBACK(prefs_display_header_deleted), NULL);
+	g_signal_connect (G_OBJECT(window), "key_press_event",
+			  G_CALLBACK(prefs_display_header_key_pressed), NULL);
+	g_signal_connect (G_OBJECT(ok_btn), "clicked",
+			  G_CALLBACK(prefs_display_header_ok), NULL);
+	g_signal_connect (G_OBJECT(cancel_btn), "clicked",
+			  G_CALLBACK(prefs_display_header_cancel), NULL);
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
@@ -226,7 +226,7 @@ static void prefs_display_header_create(void)
 	hdr_combo = gtk_combo_new ();
 	gtk_widget_show (hdr_combo);
 	gtk_box_pack_start (GTK_BOX (hbox1), hdr_combo, TRUE, TRUE, 0);
-	gtk_widget_set_usize (hdr_combo, 150, -1);
+	gtk_widget_set_size_request (hdr_combo, 150, -1);
 	gtkut_combo_set_items (GTK_COMBO (hdr_combo),
 			       "From", "To", "Cc", "Subject", "Date",
 			       "Reply-To", "Sender", "User-Agent", "X-Mailer",
@@ -243,7 +243,7 @@ static void prefs_display_header_create(void)
 	gtk_box_pack_start (GTK_BOX (clist_hbox), clist_hbox1, TRUE, TRUE, 0);
 
 	clist_scrolledwin = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_set_usize (clist_scrolledwin, 200, 210);
+	gtk_widget_set_size_request (clist_scrolledwin, 200, 210);
 	gtk_widget_show (clist_scrolledwin);
 	gtk_box_pack_start (GTK_BOX (clist_hbox1), clist_scrolledwin,
 			    TRUE, TRUE, 0);
@@ -261,9 +261,9 @@ static void prefs_display_header_create(void)
 	gtk_clist_set_use_drag_icons (GTK_CLIST (headers_clist), FALSE);
 	GTK_WIDGET_UNSET_FLAGS (GTK_CLIST (headers_clist)->column[0].button,
 				GTK_CAN_FOCUS);
-	gtk_signal_connect_after
-		(GTK_OBJECT (headers_clist), "row_move",
-		 GTK_SIGNAL_FUNC (prefs_display_header_row_moved), NULL);
+	g_signal_connect_after
+		(G_OBJECT (headers_clist), "row_move",
+		 G_CALLBACK (prefs_display_header_row_moved), NULL);
 
 	btn_vbox = gtk_vbox_new (FALSE, 8);
 	gtk_widget_show (btn_vbox);
@@ -272,27 +272,27 @@ static void prefs_display_header_create(void)
 	reg_btn = gtk_button_new_with_label (_("Add"));
 	gtk_widget_show (reg_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), reg_btn, FALSE, TRUE, 0);
-	gtk_signal_connect (GTK_OBJECT (reg_btn), "clicked",
-			    GTK_SIGNAL_FUNC (prefs_display_header_register_cb),
-			    GINT_TO_POINTER(FALSE));
+	g_signal_connect (G_OBJECT (reg_btn), "clicked",
+			  G_CALLBACK (prefs_display_header_register_cb),
+			  GINT_TO_POINTER(FALSE));
 	del_btn = gtk_button_new_with_label (_("Delete"));
 	gtk_widget_show (del_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), del_btn, FALSE, TRUE, 0);
-	gtk_signal_connect (GTK_OBJECT (del_btn), "clicked",
-			    GTK_SIGNAL_FUNC (prefs_display_header_delete_cb),
-			    headers_clist);
+	g_signal_connect (G_OBJECT (del_btn), "clicked",
+			  G_CALLBACK (prefs_display_header_delete_cb),
+			  headers_clist);
 
 	up_btn = gtk_button_new_with_label (_("Up"));
 	gtk_widget_show (up_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), up_btn, FALSE, FALSE, 0);
-	gtk_signal_connect (GTK_OBJECT (up_btn), "clicked",
-			    GTK_SIGNAL_FUNC (prefs_display_header_up), NULL);
+	g_signal_connect (G_OBJECT (up_btn), "clicked",
+			  G_CALLBACK (prefs_display_header_up), NULL);
 
 	down_btn = gtk_button_new_with_label (_("Down"));
 	gtk_widget_show (down_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), down_btn, FALSE, FALSE, 0);
-	gtk_signal_connect (GTK_OBJECT (down_btn), "clicked",
-			    GTK_SIGNAL_FUNC (prefs_display_header_down), NULL);
+	g_signal_connect (G_OBJECT (down_btn), "clicked",
+			  G_CALLBACK (prefs_display_header_down), NULL);
 
 	/* hidden headers list */
 
@@ -301,7 +301,7 @@ static void prefs_display_header_create(void)
 	gtk_box_pack_start (GTK_BOX (clist_hbox), clist_hbox2, TRUE, TRUE, 0);
 
 	clist_scrolledwin = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_set_usize (clist_scrolledwin, 200, 210);
+	gtk_widget_set_size_request (clist_scrolledwin, 200, 210);
 	gtk_widget_show (clist_scrolledwin);
 	gtk_box_pack_start (GTK_BOX (clist_hbox2), clist_scrolledwin,
 			    TRUE, TRUE, 0);
@@ -327,16 +327,15 @@ static void prefs_display_header_create(void)
 	reg_btn = gtk_button_new_with_label (_("Add"));
 	gtk_widget_show (reg_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), reg_btn, FALSE, TRUE, 0);
-	gtk_signal_connect (GTK_OBJECT (reg_btn), "clicked",
-			    GTK_SIGNAL_FUNC
-			    (prefs_display_header_register_cb),
-			    GINT_TO_POINTER(TRUE));
+	g_signal_connect (G_OBJECT (reg_btn), "clicked",
+			  G_CALLBACK (prefs_display_header_register_cb),
+			  GINT_TO_POINTER (TRUE));
 	del_btn = gtk_button_new_with_label (_("Delete"));
 	gtk_widget_show (del_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), del_btn, FALSE, TRUE, 0);
-	gtk_signal_connect (GTK_OBJECT (del_btn), "clicked",
-			    GTK_SIGNAL_FUNC (prefs_display_header_delete_cb),
-			    hidden_headers_clist);
+	g_signal_connect (G_OBJECT (del_btn), "clicked",
+			  G_CALLBACK (prefs_display_header_delete_cb),
+			  hidden_headers_clist);
 
 	PACK_CHECK_BUTTON (btn_hbox, checkbtn_other_headers,
 			   _("Show all unspecified headers"));
@@ -345,6 +344,8 @@ static void prefs_display_header_create(void)
 	gtk_widget_show_all(window);
 
 	dispheader.window        = window;
+
+	dispheader.confirm_area  = confirm_area;
 	dispheader.ok_btn        = ok_btn;
 	dispheader.cancel_btn    = cancel_btn;
 
@@ -368,7 +369,7 @@ void prefs_display_header_read_config(void)
 
 	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
 			     DISPLAY_HEADER_RC, NULL);
-	if ((fp = fopen(rcpath, "rb")) == NULL) {
+	if ((fp = g_fopen(rcpath, "rb")) == NULL) {
 		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
 		g_free(rcpath);
 		prefs_common.disphdr_list = NULL;
@@ -506,7 +507,7 @@ static gint prefs_display_header_find_header(GtkCList *clist,
 	DisplayHeaderProp *dp;
 
 	while ((dp = gtk_clist_get_row_data(clist, row)) != NULL) {
-		if (g_strcasecmp(dp->name, header) == 0)
+		if (g_ascii_strcasecmp(dp->name, header) == 0)
 			return row;
 		row++;
 	}
@@ -518,7 +519,7 @@ static gint prefs_display_header_clist_set_row(gboolean hidden)
 {
 	GtkCList *clist;
 	DisplayHeaderProp *dp;
-	gchar *entry_text;
+	const gchar *entry_text;
 	gchar *dp_str[1];
 	gint row;
 
@@ -604,12 +605,13 @@ static void prefs_display_header_row_moved(GtkCList *clist, gint source_row,
 	prefs_display_header_set_list();
 }
 
-static void prefs_display_header_key_pressed(GtkWidget *widget,
+static gboolean prefs_display_header_key_pressed(GtkWidget *widget,
 					     GdkEventKey *event,
 					     gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		prefs_display_header_cancel();
+	return FALSE;
 }
 
 static void prefs_display_header_ok(void)
