@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,18 +24,17 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <locale.h>
 #include <ctype.h>
 
-#include "intl.h"
 #include "procmime.h"
 #include "procheader.h"
 #include "base64.h"
 #include "quoted-printable.h"
 #include "uuencode.h"
-#include "unmime.h"
 #include "html.h"
 #include "codeconv.h"
 #include "utils.h"
@@ -198,7 +197,10 @@ MimeInfo *procmime_scan_message(MsgInfo *msginfo)
 #endif
 
 	if (mimeinfo) {
-		mimeinfo->size = get_left_file_size(fp);
+		mimeinfo->size = msginfo->size;
+		mimeinfo->content_size = get_left_file_size(fp);
+		if (mimeinfo->encoding_type == ENC_BASE64)
+			mimeinfo->content_size = mimeinfo->content_size / 4 * 3;
 		if (mimeinfo->mime_type == MIME_MULTIPART ||
 		    mimeinfo->mime_type == MIME_MESSAGE_RFC822)
 			procmime_scan_multipart_message(mimeinfo, fp);
@@ -249,17 +251,23 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 	for (;;) {
 		MimeInfo *partinfo;
 		gboolean eom = FALSE;
+		glong content_pos;
+		gboolean is_base64;
 		gint len;
+		guint b64_content_len = 0;
+		gint b64_pad_len = 0;
 
 		prev_fpos = fpos;
 		debug_print("prev_fpos: %ld\n", fpos);
 
+		/* scan part header */
 		if (mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
 			MimeInfo *sub;
 
 			mimeinfo->sub = sub = procmime_scan_mime_header(fp);
 			if (!sub) break;
 
+			debug_print("message/rfc822 part found\n");
 			sub->level = mimeinfo->level + 1;
 			sub->parent = mimeinfo->parent;
 			sub->main = mimeinfo;
@@ -273,6 +281,10 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 				    partinfo->content_type);
 		}
 
+		/* begin content */
+		content_pos = ftell(fp);
+		debug_print("content_pos: %ld\n", content_pos);
+
 		if (partinfo->mime_type == MIME_MULTIPART ||
 		    partinfo->mime_type == MIME_MESSAGE_RFC822) {
 			if (partinfo->level < 8)
@@ -281,12 +293,20 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 
 		/* look for next boundary */
 		buf[0] = '\0';
+		is_base64 = partinfo->encoding_type == ENC_BASE64;
 		while ((p = fgets(buf, sizeof(buf), fp)) != NULL) {
 			if (IS_BOUNDARY(buf, boundary, boundary_len)) {
 				if (buf[2 + boundary_len]     == '-' &&
 				    buf[2 + boundary_len + 1] == '-')
 					eom = TRUE;
 				break;
+			} else if (is_base64) {
+				const gchar *s;
+				for (s = buf; *s && *s != '\r' && *s != '\n';
+				     ++s)
+					if (*s == '=')
+						++b64_pad_len;
+				b64_content_len += s - buf;
 			}
 		}
 		if (p == NULL) {
@@ -301,7 +321,14 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 
 		len = strlen(buf);
 		partinfo->size = fpos - prev_fpos - len;
+		if (is_base64)
+			partinfo->content_size =
+				b64_content_len / 4 * 3 - b64_pad_len;
+		else
+			partinfo->content_size = fpos - content_pos - len;
 		debug_print("partinfo->size: %d\n", partinfo->size);
+		debug_print("partinfo->content_size: %d\n",
+			    partinfo->content_size);
 		if (partinfo->sub && !partinfo->sub->sub &&
 		    !partinfo->sub->children) {
 			partinfo->sub->size =
@@ -329,15 +356,15 @@ void procmime_scan_encoding(MimeInfo *mimeinfo, const gchar *encoding)
 	g_free(mimeinfo->encoding);
 
 	mimeinfo->encoding = g_strdup(g_strstrip(buf));
-	if (!strcasecmp(buf, "7bit"))
+	if (!g_ascii_strcasecmp(buf, "7bit"))
 		mimeinfo->encoding_type = ENC_7BIT;
-	else if (!strcasecmp(buf, "8bit"))
+	else if (!g_ascii_strcasecmp(buf, "8bit"))
 		mimeinfo->encoding_type = ENC_8BIT;
-	else if (!strcasecmp(buf, "quoted-printable"))
+	else if (!g_ascii_strcasecmp(buf, "quoted-printable"))
 		mimeinfo->encoding_type = ENC_QUOTED_PRINTABLE;
-	else if (!strcasecmp(buf, "base64"))
+	else if (!g_ascii_strcasecmp(buf, "base64"))
 		mimeinfo->encoding_type = ENC_BASE64;
-	else if (!strcasecmp(buf, "x-uuencode"))
+	else if (!g_ascii_strcasecmp(buf, "x-uuencode"))
 		mimeinfo->encoding_type = ENC_X_UUENCODE;
 	else
 		mimeinfo->encoding_type = ENC_UNKNOWN;
@@ -346,29 +373,36 @@ void procmime_scan_encoding(MimeInfo *mimeinfo, const gchar *encoding)
 
 void procmime_scan_content_type(MimeInfo *mimeinfo, const gchar *content_type)
 {
-	gchar *delim, *p, *cnttype;
-	gchar *buf;
-
-	if (conv_get_current_charset() == C_EUC_JP &&
-	    strchr(content_type, '\033')) {
-		gint len;
-		len = strlen(content_type) * 2 + 1;
-		Xalloca(buf, len, return);
-		conv_jistoeuc(buf, len, content_type);
-	} else
-		Xstrdup_a(buf, content_type, return);
-
 	g_free(mimeinfo->content_type);
 	g_free(mimeinfo->charset);
 	g_free(mimeinfo->name);
+	g_free(mimeinfo->boundary);
 	mimeinfo->content_type = NULL;
 	mimeinfo->charset      = NULL;
 	mimeinfo->name         = NULL;
+	mimeinfo->boundary     = NULL;
+
+	procmime_scan_content_type_str(content_type, &mimeinfo->content_type,
+				       &mimeinfo->charset, &mimeinfo->name,
+				       &mimeinfo->boundary);
+
+	mimeinfo->mime_type = procmime_scan_mime_type(mimeinfo->content_type);
+	if (mimeinfo->mime_type == MIME_MULTIPART && !mimeinfo->boundary)
+		mimeinfo->mime_type = MIME_TEXT;
+}
+
+void procmime_scan_content_type_str(const gchar *content_type,
+				    gchar **mime_type, gchar **charset,
+				    gchar **name, gchar **boundary)
+{
+	gchar *delim, *p;
+	gchar *buf;
+
+	Xstrdup_a(buf, content_type, return);
 
 	if ((delim = strchr(buf, ';'))) *delim = '\0';
-	mimeinfo->content_type = cnttype = g_strdup(g_strstrip(buf));
-
-	mimeinfo->mime_type = procmime_scan_mime_type(cnttype);
+	if (mime_type)
+		*mime_type = g_strdup(g_strstrip(buf));
 
 	if (!delim) return;
 	p = delim + 1;
@@ -395,26 +429,18 @@ void procmime_scan_content_type(MimeInfo *mimeinfo, const gchar *content_type)
 		}
 
 		if (*value) {
-			if (!strcasecmp(attr, "charset"))
-				mimeinfo->charset = g_strdup(value);
-			else if (!strcasecmp(attr, "name")) {
-				gchar *tmp;
-				size_t len;
-
-				len = strlen(value) + 1;
-				Xalloca(tmp, len, return);
-				conv_unmime_header(tmp, len, value, NULL);
-				mimeinfo->name = g_strdup(tmp);
-			} else if (!strcasecmp(attr, "boundary"))
-				mimeinfo->boundary = g_strdup(value);
+			if (charset && !g_ascii_strcasecmp(attr, "charset"))
+				*charset = g_strdup(value);
+			else if (name && !g_ascii_strcasecmp(attr, "name"))
+				*name = conv_unmime_header(value, NULL);
+			else if (boundary &&
+				 !g_ascii_strcasecmp(attr, "boundary"))
+				*boundary = g_strdup(value);
 		}
 
 		if (!delim) break;
 		p = delim + 1;
 	}
-
-	if (mimeinfo->mime_type == MIME_MULTIPART && !mimeinfo->boundary)
-		mimeinfo->mime_type = MIME_TEXT;
 }
 
 void procmime_scan_content_disposition(MimeInfo *mimeinfo,
@@ -423,14 +449,7 @@ void procmime_scan_content_disposition(MimeInfo *mimeinfo,
 	gchar *delim, *p, *dispos;
 	gchar *buf;
 
-	if (conv_get_current_charset() == C_EUC_JP &&
-	    strchr(content_disposition, '\033')) {
-		gint len;
-		len = strlen(content_disposition) * 2 + 1;
-		Xalloca(buf, len, return);
-		conv_jistoeuc(buf, len, content_disposition);
-	} else
-		Xstrdup_a(buf, content_disposition, return);
+	Xstrdup_a(buf, content_disposition, return);
 
 	if ((delim = strchr(buf, ';'))) *delim = '\0';
 	mimeinfo->content_disposition = dispos = g_strdup(g_strstrip(buf));
@@ -460,15 +479,10 @@ void procmime_scan_content_disposition(MimeInfo *mimeinfo,
 		}
 
 		if (*value) {
-			if (!strcasecmp(attr, "filename")) {
-				gchar *tmp;
-				size_t len;
-
-				len = strlen(value) + 1;
-				Xalloca(tmp, len, return);
-				conv_unmime_header(tmp, len, value, NULL);
+			if (!g_ascii_strcasecmp(attr, "filename")) {
 				g_free(mimeinfo->filename);
-				mimeinfo->filename = g_strdup(tmp);
+				mimeinfo->filename =
+					conv_unmime_header(value, NULL);
 				break;
 			}
 		}
@@ -573,10 +587,12 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 		Base64Decoder *decoder;
 		gboolean uncanonicalize = FALSE;
 		FILE *tmpfp = outfp;
+		ContentType content_type;
 
-		if (mimeinfo->mime_type == MIME_TEXT ||
-		    mimeinfo->mime_type == MIME_TEXT_HTML ||
-		    mimeinfo->mime_type == MIME_MESSAGE_RFC822) {
+		content_type = procmime_scan_mime_type(mimeinfo->content_type);
+		if (content_type == MIME_TEXT ||
+		    content_type == MIME_TEXT_HTML ||
+		    content_type == MIME_MESSAGE_RFC822) {
 			uncanonicalize = TRUE;
 			tmpfp = my_tmpfile();
 			if (!tmpfp) {
@@ -643,25 +659,38 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 gint procmime_get_part(const gchar *outfile, const gchar *infile,
 		       MimeInfo *mimeinfo)
 {
-	FILE *infp, *outfp;
-	gchar buf[BUFFSIZE];
+	FILE *infp;
+	gint ret;
 
 	g_return_val_if_fail(outfile != NULL, -1);
 	g_return_val_if_fail(infile != NULL, -1);
 	g_return_val_if_fail(mimeinfo != NULL, -1);
 
-	if ((infp = fopen(infile, "rb")) == NULL) {
+	if ((infp = g_fopen(infile, "rb")) == NULL) {
 		FILE_OP_ERROR(infile, "fopen");
 		return -1;
 	}
+	ret = procmime_get_part_fp(outfile, infp, mimeinfo);
+	fclose(infp);
+
+	return ret;
+}
+
+gint procmime_get_part_fp(const gchar *outfile, FILE *infp, MimeInfo *mimeinfo)
+{
+	FILE *outfp;
+	gchar buf[BUFFSIZE];
+
+	g_return_val_if_fail(outfile != NULL, -1);
+	g_return_val_if_fail(infp != NULL, -1);
+	g_return_val_if_fail(mimeinfo != NULL, -1);
+
 	if (fseek(infp, mimeinfo->fpos, SEEK_SET) < 0) {
-		FILE_OP_ERROR(infile, "fseek");
-		fclose(infp);
+		FILE_OP_ERROR("procmime_get_part_fp()", "fseek");
 		return -1;
 	}
-	if ((outfp = fopen(outfile, "wb")) == NULL) {
+	if ((outfp = g_fopen(outfile, "wb")) == NULL) {
 		FILE_OP_ERROR(outfile, "fopen");
-		fclose(infp);
 		return -1;
 	}
 
@@ -670,23 +699,22 @@ gint procmime_get_part(const gchar *outfile, const gchar *infile,
 
 	procmime_decode_content(outfp, infp, mimeinfo);
 
-	fclose(infp);
 	if (fclose(outfp) == EOF) {
 		FILE_OP_ERROR(outfile, "fclose");
-		unlink(outfile);
+		g_unlink(outfile);
 		return -1;
 	}
 
 	return 0;
 }
 
-FILE *procmime_get_text_content(MimeInfo *mimeinfo, FILE *infp)
+FILE *procmime_get_text_content(MimeInfo *mimeinfo, FILE *infp,
+				const gchar *encoding)
 {
 	FILE *tmpfp, *outfp;
-	gchar *src_codeset;
+	const gchar *src_encoding;
 	gboolean conv_fail = FALSE;
 	gchar buf[BUFFSIZE];
-	gchar *str;
 
 	g_return_val_if_fail(mimeinfo != NULL, NULL);
 	g_return_val_if_fail(infp != NULL, NULL);
@@ -711,12 +739,14 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo, FILE *infp)
 		return NULL;
 	}
 
-	src_codeset = prefs_common.force_charset
+	src_encoding = prefs_common.force_charset
 		? prefs_common.force_charset : mimeinfo->charset;
 
 	if (mimeinfo->mime_type == MIME_TEXT) {
 		while (fgets(buf, sizeof(buf), tmpfp) != NULL) {
-			str = conv_codeset_strdup(buf, src_codeset, NULL);
+			gchar *str;
+
+			str = conv_codeset_strdup(buf, src_encoding, encoding);
 			if (str) {
 				fputs(str, outfp);
 				g_free(str);
@@ -728,8 +758,9 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo, FILE *infp)
 	} else if (mimeinfo->mime_type == MIME_TEXT_HTML) {
 		HTMLParser *parser;
 		CodeConverter *conv;
+		const gchar *str;
 
-		conv = conv_code_converter_new(src_codeset);
+		conv = conv_code_converter_new(src_encoding, encoding);
 		parser = html_parser_new(tmpfp, conv);
 		while ((str = html_parse(parser)) != NULL) {
 			fputs(str, outfp);
@@ -749,7 +780,7 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo, FILE *infp)
 
 /* search the first text part of (multipart) MIME message,
    decode, convert it and output to outfp. */
-FILE *procmime_get_first_text_content(MsgInfo *msginfo)
+FILE *procmime_get_first_text_content(MsgInfo *msginfo, const gchar *encoding)
 {
 	FILE *infp, *outfp = NULL;
 	MimeInfo *mimeinfo, *partinfo;
@@ -774,7 +805,7 @@ FILE *procmime_get_first_text_content(MsgInfo *msginfo)
 	}
 
 	if (partinfo)
-		outfp = procmime_get_text_content(partinfo, infp);
+		outfp = procmime_get_text_content(partinfo, infp, encoding);
 
 	fclose(infp);
 	procmime_mimeinfo_free_all(mimeinfo);
@@ -795,12 +826,12 @@ gboolean procmime_find_string_part(MimeInfo *mimeinfo, const gchar *filename,
 	g_return_val_if_fail(str != NULL, FALSE);
 	g_return_val_if_fail(find_func != NULL, FALSE);
 
-	if ((infp = fopen(filename, "rb")) == NULL) {
+	if ((infp = g_fopen(filename, "rb")) == NULL) {
 		FILE_OP_ERROR(filename, "fopen");
 		return FALSE;
 	}
 
-	outfp = procmime_get_text_content(mimeinfo, infp);
+	outfp = procmime_get_text_content(mimeinfo, infp, NULL);
 	fclose(infp);
 
 	if (!outfp)
@@ -853,6 +884,21 @@ gboolean procmime_find_string(MsgInfo *msginfo, const gchar *str,
 	return found;
 }
 
+gchar *procmime_get_part_file_name(MimeInfo *mimeinfo)
+{
+	gchar *base;
+	const gchar *base_;
+
+	base_ = mimeinfo->filename ? mimeinfo->filename
+		: mimeinfo->name ? mimeinfo->name : "mimetmp";
+	base_ = g_basename(base_);
+	if (*base_ == '\0') base_ = "mimetmp";
+	base = conv_filename_from_utf8(base_);
+	subst_for_filename(base);
+
+	return base;
+}
+
 gchar *procmime_get_tmp_file_name(MimeInfo *mimeinfo)
 {
 	static guint32 id = 0;
@@ -865,19 +911,15 @@ gchar *procmime_get_tmp_file_name(MimeInfo *mimeinfo)
 	g_snprintf(f_prefix, sizeof(f_prefix), "%08x.", id++);
 
 	if (MIME_TEXT_HTML == mimeinfo->mime_type)
-		base = "mimetmp.html";
-	else {
-		base = mimeinfo->filename ? mimeinfo->filename
-			: mimeinfo->name ? mimeinfo->name : "mimetmp";
-		base = g_basename(base);
-		if (*base == '\0') base = "mimetmp";
-		Xstrdup_a(base, base, return NULL);
-		subst_for_filename(base);
-	}
+		base = g_strdup("mimetmp.html");
+	else
+		base = procmime_get_part_file_name(mimeinfo);
 
 	filename = g_strconcat(get_mime_tmp_dir(), G_DIR_SEPARATOR_S,
 			       f_prefix, base, NULL);
 
+	g_free(base);
+
 	return filename;
 }
 
@@ -885,25 +927,26 @@ ContentType procmime_scan_mime_type(const gchar *mime_type)
 {
 	ContentType type;
 
-	if (!strncasecmp(mime_type, "text/html", 9))
+	if (!g_ascii_strncasecmp(mime_type, "text/html", 9))
 		type = MIME_TEXT_HTML;
-	else if (!strncasecmp(mime_type, "text/", 5))
+	else if (!g_ascii_strncasecmp(mime_type, "text/", 5))
 		type = MIME_TEXT;
-	else if (!strncasecmp(mime_type, "message/rfc822", 14))
+	else if (!g_ascii_strncasecmp(mime_type, "message/rfc822", 14))
 		type = MIME_MESSAGE_RFC822;
-	else if (!strncasecmp(mime_type, "message/", 8))
+	else if (!g_ascii_strncasecmp(mime_type, "message/", 8))
 		type = MIME_TEXT;
-	else if (!strncasecmp(mime_type, "application/octet-stream", 24))
+	else if (!g_ascii_strncasecmp(mime_type, "application/octet-stream",
+				      24))
 		type = MIME_APPLICATION_OCTET_STREAM;
-	else if (!strncasecmp(mime_type, "application/", 12))
+	else if (!g_ascii_strncasecmp(mime_type, "application/", 12))
 		type = MIME_APPLICATION;
-	else if (!strncasecmp(mime_type, "multipart/", 10))
+	else if (!g_ascii_strncasecmp(mime_type, "multipart/", 10))
 		type = MIME_MULTIPART;
-	else if (!strncasecmp(mime_type, "image/", 6))
+	else if (!g_ascii_strncasecmp(mime_type, "image/", 6))
 		type = MIME_IMAGE;
-	else if (!strncasecmp(mime_type, "audio/", 6))
+	else if (!g_ascii_strncasecmp(mime_type, "audio/", 6))
 		type = MIME_AUDIO;
-	else if (!strcasecmp(mime_type, "text"))
+	else if (!g_ascii_strcasecmp(mime_type, "text"))
 		type = MIME_TEXT;
 	else
 		type = MIME_UNKNOWN;
@@ -960,8 +1003,8 @@ static GHashTable *procmime_get_mime_type_table(void)
 			list = procmime_get_mime_type_list("/etc/mime.types");
 			mime_type_list = g_list_concat(mime_type_list, list);
 		}
-		dir = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, RC_DIR,
-				  G_DIR_SEPARATOR_S, "mime.types", NULL);
+		dir = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
+				  "mime.types", NULL);
 		list = procmime_get_mime_type_list(dir);
 		g_free(dir);
 		mime_type_list = g_list_concat(mime_type_list, list);
@@ -1004,11 +1047,11 @@ static GList *procmime_get_mime_type_list(const gchar *file)
 	GList *list = NULL;
 	FILE *fp;
 	gchar buf[BUFFSIZE];
-	guchar *p;
+	gchar *p;
 	gchar *delim;
 	MimeType *mime_type;
 
-	if ((fp = fopen(file, "rb")) == NULL) return NULL;
+	if ((fp = g_fopen(file, "rb")) == NULL) return NULL;
 
 	debug_print("Reading %s ...\n", file);
 
@@ -1018,7 +1061,7 @@ static GList *procmime_get_mime_type_list(const gchar *file)
 		g_strstrip(buf);
 
 		p = buf;
-		while (*p && !isspace(*p)) p++;
+		while (*p && !g_ascii_isspace(*p)) p++;
 		if (*p) {
 			*p = '\0';
 			p++;
@@ -1031,7 +1074,7 @@ static GList *procmime_get_mime_type_list(const gchar *file)
 		mime_type->type = g_strdup(buf);
 		mime_type->sub_type = g_strdup(delim + 1);
 
-		while (*p && isspace(*p)) p++;
+		while (*p && g_ascii_isspace(*p)) p++;
 		if (*p)
 			mime_type->extension = g_strdup(p);
 		else
@@ -1052,14 +1095,14 @@ EncodingType procmime_get_encoding_for_charset(const gchar *charset)
 {
 	if (!charset)
 		return ENC_8BIT;
-	else if (!strncasecmp(charset, "ISO-2022-", 9) ||
-		 !strcasecmp(charset, "US-ASCII"))
+	else if (!g_ascii_strncasecmp(charset, "ISO-2022-", 9) ||
+		 !g_ascii_strcasecmp(charset, "US-ASCII"))
 		return ENC_7BIT;
-	else if (!strcasecmp(charset, "ISO-8859-5") ||
-		 !strncasecmp(charset, "KOI8-", 5) ||
-		 !strcasecmp(charset, "Windows-1251"))
+	else if (!g_ascii_strcasecmp(charset, "ISO-8859-5") ||
+		 !g_ascii_strncasecmp(charset, "KOI8-", 5) ||
+		 !g_ascii_strcasecmp(charset, "Windows-1251"))
 		return ENC_8BIT;
-	else if (!strncasecmp(charset, "ISO-8859-", 9))
+	else if (!g_ascii_strncasecmp(charset, "ISO-8859-", 9))
 		return ENC_QUOTED_PRINTABLE;
 	else
 		return ENC_8BIT;
@@ -1074,7 +1117,7 @@ EncodingType procmime_get_encoding_for_text_file(const gchar *file)
 	size_t total_len = 0;
 	gfloat octet_percentage;
 
-	if ((fp = fopen(file, "rb")) == NULL) {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		return ENC_UNKNOWN;
 	}
