@@ -20,11 +20,11 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "intl.h"
 #include "main.h"
 #include "utils.h"
 #include "procmsg.h"
@@ -33,9 +33,11 @@
 #include "send_message.h"
 #include "procmime.h"
 #include "statusbar.h"
+#include "prefs_common.h"
 #include "prefs_filter.h"
 #include "filter.h"
 #include "folder.h"
+#include "codeconv.h"
 #if USE_GPGME
 #  include "rfc2015.h"
 #endif
@@ -201,6 +203,7 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 	MsgFlags default_flags;
 	gchar file_buf[BUFFSIZE];
 	guint32 num;
+	guint refnum;
 	FolderType type;
 
 	g_return_val_if_fail(item != NULL, NULL);
@@ -234,8 +237,10 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 	}
 
 	if ((fp = procmsg_open_cache_file_with_buffer
-		(item, DATA_READ, file_buf, sizeof(file_buf))) == NULL)
+		(item, DATA_READ, file_buf, sizeof(file_buf))) == NULL) {
+		item->cache_dirty = TRUE;
 		return NULL;
+	}
 
 	debug_print("Reading summary cache...");
 
@@ -257,15 +262,28 @@ GSList *procmsg_read_cache(FolderItem *item, gboolean scan_file)
 		READ_CACHE_DATA(msginfo->msgid, fp);
 		READ_CACHE_DATA(msginfo->inreplyto, fp);
 
+		READ_CACHE_DATA_INT(refnum, fp);
+		for (; refnum != 0; refnum--) {
+			gchar *ref;
+
+			READ_CACHE_DATA(ref, fp);
+			msginfo->references =
+				g_slist_prepend(msginfo->references, ref);
+		}
+		if (msginfo->references)
+			msginfo->references =
+				g_slist_reverse(msginfo->references);
+
 		MSG_SET_PERM_FLAGS(msginfo->flags, default_flags.perm_flags);
 		MSG_SET_TMP_FLAGS(msginfo->flags, default_flags.tmp_flags);
 
 		/* if the message file doesn't exist or is changed,
 		   don't add the data */
 		if ((type == F_MH && scan_file &&
-		     folder_item_is_msg_changed(item, msginfo)) || num == 0)
+		     folder_item_is_msg_changed(item, msginfo)) || num == 0) {
 			procmsg_msginfo_free(msginfo);
-		else {
+			item->cache_dirty = TRUE;
+		} else {
 			msginfo->folder = item;
 
 			if (!mlist)
@@ -313,6 +331,7 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 	if (!mark_table) {
 		item->new = item->unread = item->total = g_slist_length(mlist);
 		item->updated = TRUE;
+		item->mark_dirty = TRUE;
 		return;
 	}
 
@@ -325,6 +344,7 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 			if (!flags) {
 				g_hash_table_foreach(mark_table,
 						     mark_unset_new_func, NULL);
+				item->mark_dirty = TRUE;
 				break;
 			}
 		}
@@ -367,6 +387,9 @@ void procmsg_set_flags(GSList *mlist, FolderItem *item)
 	item->last_num = lastnum;
 	item->updated = TRUE;
 
+	if (unflagged > 0)
+		item->mark_dirty = TRUE;
+
 	debug_print("new: %d unread: %d unflagged: %d total: %d\n",
 		    new, unread, unflagged, total);
 
@@ -443,6 +466,7 @@ void procmsg_msg_list_free(GSList *mlist)
 void procmsg_write_cache(MsgInfo *msginfo, FILE *fp)
 {
 	MsgTmpFlags flags = msginfo->flags.tmp_flags & MSG_CACHED_FLAG_MASK;
+	GSList *cur;
 
 	WRITE_CACHE_DATA_INT(msginfo->msgnum, fp);
 	WRITE_CACHE_DATA_INT(msginfo->size, fp);
@@ -459,6 +483,11 @@ void procmsg_write_cache(MsgInfo *msginfo, FILE *fp)
 	WRITE_CACHE_DATA(msginfo->subject, fp);
 	WRITE_CACHE_DATA(msginfo->msgid, fp);
 	WRITE_CACHE_DATA(msginfo->inreplyto, fp);
+
+	WRITE_CACHE_DATA_INT(g_slist_length(msginfo->references), fp);
+	for (cur = msginfo->references; cur != NULL; cur = cur->next) {
+		WRITE_CACHE_DATA((gchar *)cur->data, fp);
+	}
 }
 
 void procmsg_write_flags(MsgInfo *msginfo, FILE *fp)
@@ -607,6 +636,7 @@ static GHashTable *procmsg_read_mark_file(FolderItem *item)
 
 	if (item->mark_queue) {
 		g_hash_table_foreach(mark_table, mark_unset_new_func, NULL);
+		item->mark_dirty = TRUE;
 	}
 
 	for (cur = item->mark_queue; cur != NULL; cur = cur->next) {
@@ -629,6 +659,7 @@ static GHashTable *procmsg_read_mark_file(FolderItem *item)
 		procmsg_write_mark_file(item, mark_table);
 		procmsg_msg_list_free(item->mark_queue);
 		item->mark_queue = NULL;
+		item->mark_dirty = FALSE;
 	}
 
 	return mark_table;
@@ -662,7 +693,7 @@ static FILE *procmsg_open_data_file(const gchar *file, guint version,
 	g_return_val_if_fail(file != NULL, NULL);
 
 	if (mode == DATA_WRITE) {
-		if ((fp = fopen(file, "wb")) == NULL) {
+		if ((fp = g_fopen(file, "wb")) == NULL) {
 			FILE_OP_ERROR(file, "fopen");
 			return NULL;
 		}
@@ -674,15 +705,15 @@ static FILE *procmsg_open_data_file(const gchar *file, guint version,
 	}
 
 	/* check version */
-	if ((fp = fopen(file, "rb")) == NULL)
-		debug_print("Mark/Cache file not found\n");
+	if ((fp = g_fopen(file, "rb")) == NULL)
+		debug_print("Mark/Cache file '%s' not found\n", file);
 	else {
 		if (buf && buf_size > 0)
 			setvbuf(fp, buf, _IOFBF, buf_size);
 		if (fread(&data_ver, sizeof(data_ver), 1, fp) != 1 ||
 		    version != data_ver) {
-			debug_print("Mark/Cache version is different (%u != %u). "
-				    "Discarding it.\n", data_ver, version);
+			g_message("%s: Mark/Cache version is different (%u != %u). Discarding it.\n",
+				  file, data_ver, version);
 			fclose(fp);
 			fp = NULL;
 		}
@@ -694,7 +725,7 @@ static FILE *procmsg_open_data_file(const gchar *file, guint version,
 	if (fp) {
 		/* reopen with append mode */
 		fclose(fp);
-		if ((fp = fopen(file, "ab")) == NULL)
+		if ((fp = g_fopen(file, "ab")) == NULL)
 			FILE_OP_ERROR(file, "fopen");
 	} else {
 		/* open with overwrite mode if mark file doesn't exist or
@@ -745,6 +776,24 @@ FILE *procmsg_open_mark_file(FolderItem *item, DataOpenMode mode)
 	return fp;
 }
 
+void procmsg_clear_cache(FolderItem *item)
+{
+	FILE *fp;
+
+	fp = procmsg_open_cache_file(item, DATA_WRITE);
+	if (fp)
+		fclose(fp);
+}
+
+void procmsg_clear_mark(FolderItem *item)
+{
+	FILE *fp;
+
+	fp = procmsg_open_mark_file(item, DATA_WRITE);
+	if (fp)
+		fclose(fp);
+}
+
 /* return the reversed thread tree */
 GNode *procmsg_get_thread_tree(GSList *mlist)
 {
@@ -752,6 +801,7 @@ GNode *procmsg_get_thread_tree(GSList *mlist)
 	GHashTable *table;
 	MsgInfo *msginfo;
 	const gchar *msgid;
+	GSList *reflist;
 
 	root = g_node_new(NULL);
 	table = g_hash_table_new(g_str_hash, g_str_equal);
@@ -760,11 +810,13 @@ GNode *procmsg_get_thread_tree(GSList *mlist)
 		msginfo = (MsgInfo *)mlist->data;
 		parent = root;
 
+		/* only look for the real parent first */
 		if (msginfo->inreplyto) {
 			parent = g_hash_table_lookup(table, msginfo->inreplyto);
 			if (parent == NULL)
 				parent = root;
 		}
+
 		node = g_node_insert_data_before
 			(parent, parent == root ? parent->children : NULL,
 			 msginfo);
@@ -777,16 +829,27 @@ GNode *procmsg_get_thread_tree(GSList *mlist)
 	for (node = root->children; node != NULL; ) {
 		next = node->next;
 		msginfo = (MsgInfo *)node->data;
-		if (msginfo->inreplyto) {
+		parent = NULL;
+
+		if (msginfo->inreplyto)
 			parent = g_hash_table_lookup(table, msginfo->inreplyto);
-			/* node should not be the parent, and node should not
-			   be an ancestor of parent (circular reference) */
-			if (parent && parent != node &&
-			    !g_node_is_ancestor(node, parent)) {
-				g_node_unlink(node);
-				g_node_insert_before
-					(parent, parent->children, node);
-			}
+
+		/* try looking for the indirect parent */
+		if (!parent && msginfo->references) {
+			for (reflist = msginfo->references;
+			     reflist != NULL; reflist = reflist->next)
+				if ((parent = g_hash_table_lookup
+					(table, reflist->data)) != NULL)
+					break;
+		}
+
+		/* node should not be the parent, and node should not
+		   be an ancestor of parent (circular reference) */
+		if (parent && parent != node &&
+		    !g_node_is_ancestor(node, parent)) {
+			g_node_unlink(node);
+			g_node_insert_before
+				(parent, parent->children, node);
 		}
 		node = next;
 	}
@@ -971,7 +1034,7 @@ FILE *procmsg_open_message(MsgInfo *msginfo)
 			return NULL;
 	}
 
-	if ((fp = fopen(file, "rb")) == NULL) {
+	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
 		g_free(file);
 		return NULL;
@@ -1108,14 +1171,14 @@ void procmsg_get_filter_keyword(MsgInfo *msginfo, gchar **header, gchar **key,
 		} else if (hentry[H_X_MAILING_LIST].body != NULL) {
 			SET_FILTER_KEY("X-Mailing-list", H_X_MAILING_LIST);
 		} else if (hentry[H_X_SEQUENCE].body != NULL) {
-			guchar *p;
+			gchar *p;
 
 			SET_FILTER_KEY("X-Sequence", H_X_SEQUENCE);
 			p = *key;
 			while (*p != '\0') {
-				while (*p != '\0' && !isspace(*p)) p++;
-				while (isspace(*p)) p++;
-				if (isdigit(*p)) {
+				while (*p != '\0' && !g_ascii_isspace(*p)) p++;
+				while (g_ascii_isspace(*p)) p++;
+				if (g_ascii_isdigit(*p)) {
 					*p = '\0';
 					break;
 				}
@@ -1157,16 +1220,14 @@ void procmsg_get_filter_keyword(MsgInfo *msginfo, gchar **header, gchar **key,
 
 void procmsg_empty_trash(FolderItem *trash)
 {
-	FILE *fp;
-
 	if (trash && trash->total > 0) {
 		debug_print("Emptying messages in %s ...\n", trash->path);
 
 		folder_item_remove_all_msg(trash);
-		fp = procmsg_open_cache_file(trash, DATA_WRITE);
-		if (fp) fclose(fp);
-		fp = procmsg_open_mark_file(trash, DATA_WRITE);
-		if (fp) fclose(fp);
+		procmsg_clear_cache(trash);
+		procmsg_clear_mark(trash);
+		trash->cache_dirty = FALSE;
+		trash->mark_dirty = FALSE;
 	}
 }
 
@@ -1181,7 +1242,8 @@ void procmsg_empty_all_trash(void)
 	}
 }
 
-gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
+gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs,
+			gboolean filter_msgs)
 {
 	gint ret = 0;
 	GSList *mlist = NULL;
@@ -1197,45 +1259,67 @@ gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 	for (cur = mlist; cur != NULL; cur = cur->next) {
 		gchar *file;
 		MsgInfo *msginfo = (MsgInfo *)cur->data;
+		QueueInfo *qinfo;
+		gchar tmp[MAXPATHLEN + 1];
 
 		file = procmsg_get_message_file(msginfo);
-		if (file) {
-			QueueInfo *qinfo;
+		if (!file)
+			continue;
 
-			qinfo = send_get_queue_info(file);
-			if (!qinfo || send_message_queue(qinfo) < 0) {
-				g_warning(_("Sending queued message %d failed.\n"),
-					  msginfo->msgnum);
-			} else {
-				ret++;
-				if (save_msgs) {
-					FolderItem *outbox;
-					outbox = account_get_special_folder
-						(qinfo->ac, F_OUTBOX);
-					procmsg_save_to_outbox(outbox, file,
-							       TRUE);
-				}
-				folder_item_remove_msg(queue, msginfo);
-			}
+		qinfo = send_get_queue_info(file);
+		if (!qinfo || send_message_queue(qinfo) < 0) {
+			g_warning("Sending queued message %d failed.\n",
+				  msginfo->msgnum);
 			send_queue_info_free(qinfo);
 			g_free(file);
+			continue;
 		}
 
-		procmsg_msginfo_free(msginfo);
+		g_snprintf(tmp, sizeof(tmp), "%s%ctmpmsg.out.%08x",
+			   get_rc_dir(), G_DIR_SEPARATOR, g_random_int());
+
+		if (send_get_queue_contents(qinfo, tmp) == 0) {
+			if (save_msgs) {
+				FolderItem *outbox;
+				outbox = account_get_special_folder
+					(qinfo->ac, F_OUTBOX);
+				procmsg_save_to_outbox(outbox, tmp);
+			}
+			if (filter_msgs) {
+				FilterInfo *fltinfo;
+
+				fltinfo = filter_info_new();
+				fltinfo->account = qinfo->ac;
+				fltinfo->flags.perm_flags = 0;
+				fltinfo->flags.tmp_flags = MSG_RECEIVED;
+
+				filter_apply(prefs_common.fltlist, tmp,
+					     fltinfo);
+
+				filter_info_free(fltinfo);
+			}
+			g_unlink(tmp);
+		}
+
+		send_queue_info_free(qinfo);
+		g_free(file);
+
+		folder_item_remove_msg(queue, msginfo);
+		ret++;
 	}
 
-	g_slist_free(mlist);
+	procmsg_msg_list_free(mlist);
 
+	procmsg_clear_cache(queue);
+	queue->cache_dirty = FALSE;
 	queue->mtime = 0;
 
 	return ret;
 }
 
-gint procmsg_save_to_outbox(FolderItem *outbox, const gchar *file,
-			    gboolean is_queued)
+gint procmsg_save_to_outbox(FolderItem *outbox, const gchar *file)
 {
 	gint num;
-	FILE *fp;
 	MsgFlags flag = {0, 0};
 
 	debug_print("saving sent message...\n");
@@ -1244,43 +1328,10 @@ gint procmsg_save_to_outbox(FolderItem *outbox, const gchar *file,
 		outbox = folder_get_default_outbox();
 	g_return_val_if_fail(outbox != NULL, -1);
 
-	/* remove queueing headers */
-	if (is_queued) {
-		gchar tmp[MAXPATHLEN + 1];
-		gchar buf[BUFFSIZE];
-		FILE *outfp;
-
-		g_snprintf(tmp, sizeof(tmp), "%s%ctmpmsg.out.%08x",
-			   get_rc_dir(), G_DIR_SEPARATOR, (guint)random());
-		if ((fp = fopen(file, "rb")) == NULL) {
-			FILE_OP_ERROR(file, "fopen");
-			return -1;
-		}
-		if ((outfp = fopen(tmp, "wb")) == NULL) {
-			FILE_OP_ERROR(tmp, "fopen");
-			fclose(fp);
-			return -1;
-		}
-		while (fgets(buf, sizeof(buf), fp) != NULL)
-			if (buf[0] == '\r' || buf[0] == '\n') break;
-		while (fgets(buf, sizeof(buf), fp) != NULL)
-			fputs(buf, outfp);
-		fclose(outfp);
-		fclose(fp);
-
-		folder_item_scan(outbox);
-		if ((num = folder_item_add_msg(outbox, tmp, &flag, TRUE)) < 0) {
-			g_warning("can't save message\n");
-			unlink(tmp);
-			return -1;
-		}
-	} else {
-		folder_item_scan(outbox);
-		if ((num = folder_item_add_msg
-			(outbox, file, &flag, FALSE)) < 0) {
-			g_warning("can't save message\n");
-			return -1;
-		}
+	folder_item_scan(outbox);
+	if ((num = folder_item_add_msg(outbox, file, &flag, FALSE)) < 0) {
+		g_warning("can't save message\n");
+		return -1;
 	}
 
 	return 0;
@@ -1297,7 +1348,8 @@ void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline)
 
 	g_return_if_fail(msginfo);
 
-	if ((tmpfp = procmime_get_first_text_content(msginfo)) == NULL) {
+	if ((tmpfp = procmime_get_first_text_content
+		(msginfo, conv_get_locale_charset_str())) == NULL) {
 		g_warning(_("Can't get text part\n"));
 		return;
 	}
@@ -1305,21 +1357,31 @@ void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline)
 	prtmp = g_strdup_printf("%s%cprinttmp.%08x",
 				get_mime_tmp_dir(), G_DIR_SEPARATOR, id++);
 
-	if ((prfp = fopen(prtmp, "wb")) == NULL) {
+	if ((prfp = g_fopen(prtmp, "wb")) == NULL) {
 		FILE_OP_ERROR(prtmp, "fopen");
 		g_free(prtmp);
 		fclose(tmpfp);
 		return;
 	}
 
-	if (msginfo->date) fprintf(prfp, "Date: %s\n", msginfo->date);
-	if (msginfo->from) fprintf(prfp, "From: %s\n", msginfo->from);
-	if (msginfo->to)   fprintf(prfp, "To: %s\n", msginfo->to);
-	if (msginfo->newsgroups)
-		fprintf(prfp, "Newsgroups: %s\n", msginfo->newsgroups);
-	if (msginfo->subject) fprintf(prfp, "Subject: %s\n", msginfo->subject);
+#define OUTPUT_HEADER(s, fmt)						 \
+	if (s) {							 \
+		gchar *locale_str;					 \
+		locale_str = conv_codeset_strdup			 \
+			(s, CS_INTERNAL, conv_get_locale_charset_str()); \
+		fprintf(prfp, fmt, locale_str ? locale_str : s);	 \
+		g_free(locale_str);					 \
+	}
+
+	OUTPUT_HEADER(msginfo->date, "Date: %s\n");
+	OUTPUT_HEADER(msginfo->from, "From: %s\n");
+	OUTPUT_HEADER(msginfo->to, "To: %s\n");
+	OUTPUT_HEADER(msginfo->newsgroups, "Newsgroups: %s\n");
+	OUTPUT_HEADER(msginfo->subject, "Subject: %s\n");
 	fputc('\n', prfp);
 
+#undef OUTPUT_HEADER
+
 	while (fgets(buf, sizeof(buf), tmpfp) != NULL)
 		fputs(buf, prfp);
 
@@ -1419,6 +1481,23 @@ MsgInfo *procmsg_msginfo_get_full_info(MsgInfo *msginfo)
 	return full_msginfo;
 }
 
+gboolean procmsg_msginfo_equal(MsgInfo *msginfo_a, MsgInfo *msginfo_b)
+{
+	if (!msginfo_a || !msginfo_b)
+		return FALSE;
+
+	if (msginfo_a == msginfo_b)
+		return TRUE;
+
+	if (msginfo_a->folder == msginfo_b->folder &&
+	    msginfo_a->msgnum == msginfo_b->msgnum &&
+	    msginfo_a->size   == msginfo_b->size   &&
+	    msginfo_a->mtime  == msginfo_b->mtime)
+		return TRUE;
+
+	return FALSE;
+}
+
 void procmsg_msginfo_free(MsgInfo *msginfo)
 {
 	if (msginfo == NULL) return;
@@ -1436,6 +1515,9 @@ void procmsg_msginfo_free(MsgInfo *msginfo)
 	g_free(msginfo->msgid);
 	g_free(msginfo->inreplyto);
 
+	slist_free_strings(msginfo->references);
+	g_slist_free(msginfo->references);
+
 	g_free(msginfo->file_path);
 
 	g_free(msginfo->plaintext_file);
@@ -1448,10 +1530,8 @@ gint procmsg_cmp_msgnum_for_sort(gconstpointer a, gconstpointer b)
 	const MsgInfo *msginfo1 = a;
 	const MsgInfo *msginfo2 = b;
 
-	if (!msginfo1)
-		return -1;
-	if (!msginfo2)
-		return -1;
+	if (!msginfo1 || !msginfo2)
+		return 0;
 
 	return msginfo1->msgnum - msginfo2->msgnum;
 }
@@ -1461,11 +1541,16 @@ static gint func_name(gconstpointer a, gconstpointer b)			\
 {									\
 	const MsgInfo *msginfo1 = a;					\
 	const MsgInfo *msginfo2 = b;					\
+	gint ret;							\
 									\
 	if (!msginfo1 || !msginfo2)					\
-		return -1;						\
+		return 0;						\
 									\
-	return (val) * (cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);	\
+	ret = (val);							\
+	if (ret == 0)							\
+		ret = msginfo1->date_t - msginfo2->date_t;		\
+									\
+	return ret * (cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);	\
 }
 
 CMP_FUNC_DEF(procmsg_cmp_by_mark,
@@ -1477,9 +1562,22 @@ CMP_FUNC_DEF(procmsg_cmp_by_mime,
 CMP_FUNC_DEF(procmsg_cmp_by_label,
 	     MSG_GET_COLORLABEL(msginfo1->flags) -
 	     MSG_GET_COLORLABEL(msginfo2->flags))
+CMP_FUNC_DEF(procmsg_cmp_by_size, msginfo1->size - msginfo2->size)
+
+#undef CMP_FUNC_DEF
+#define CMP_FUNC_DEF(func_name, val)					\
+static gint func_name(gconstpointer a, gconstpointer b)			\
+{									\
+	const MsgInfo *msginfo1 = a;					\
+	const MsgInfo *msginfo2 = b;					\
+									\
+	if (!msginfo1 || !msginfo2)					\
+		return 0;						\
+									\
+	return (val) * (cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);	\
+}
 
 CMP_FUNC_DEF(procmsg_cmp_by_number, msginfo1->msgnum - msginfo2->msgnum)
-CMP_FUNC_DEF(procmsg_cmp_by_size, msginfo1->size - msginfo2->size)
 CMP_FUNC_DEF(procmsg_cmp_by_date, msginfo1->date_t - msginfo2->date_t)
 
 #undef CMP_FUNC_DEF
@@ -1488,14 +1586,20 @@ static gint func_name(gconstpointer a, gconstpointer b)			\
 {									\
 	const MsgInfo *msginfo1 = a;					\
 	const MsgInfo *msginfo2 = b;					\
+	gint ret;							\
 									\
 	if (!msginfo1->var_name)					\
-		return (msginfo2->var_name != NULL);			\
+		return (msginfo2->var_name != NULL) *			\
+			(cmp_func_sort_type == SORT_ASCENDING ? -1 : 1);\
 	if (!msginfo2->var_name)					\
-		return -1;						\
+		return (cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);	\
 									\
-	return strcasecmp(msginfo1->var_name, msginfo2->var_name) *	\
-		(cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);	\
+	ret = g_ascii_strcasecmp					\
+		(msginfo1->var_name, msginfo2->var_name);		\
+	if (ret == 0)							\
+		ret = msginfo1->date_t - msginfo2->date_t;		\
+									\
+	return ret * (cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);	\
 }
 
 CMP_FUNC_DEF(procmsg_cmp_by_from, fromname)
@@ -1503,17 +1607,21 @@ CMP_FUNC_DEF(procmsg_cmp_by_to, to)
 
 #undef CMP_FUNC_DEF
 
-static gint procmsg_cmp_by_subject(gconstpointer a, gconstpointer b)	\
-{									\
-	const MsgInfo *msginfo1 = a;					\
-	const MsgInfo *msginfo2 = b;					\
-									\
-	if (!msginfo1->subject)						\
-		return (msginfo2->subject != NULL);			\
-	if (!msginfo2->subject)						\
-		return -1;						\
-									\
-	return subject_compare_for_sort					\
-		(msginfo1->subject, msginfo2->subject) *		\
-		(cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);	\
+static gint procmsg_cmp_by_subject(gconstpointer a, gconstpointer b)
+{
+	const MsgInfo *msginfo1 = a;
+	const MsgInfo *msginfo2 = b;
+	gint ret;
+
+	if (!msginfo1->subject)
+		return (msginfo2->subject != NULL) *
+			(cmp_func_sort_type == SORT_ASCENDING ? -1 : 1);
+	if (!msginfo2->subject)
+		return (cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);
+
+	ret = subject_compare_for_sort(msginfo1->subject, msginfo2->subject);
+	if (ret == 0)
+		ret = msginfo1->date_t - msginfo2->date_t;
+
+	return ret * (cmp_func_sort_type == SORT_ASCENDING ? 1 : -1);
 }
