@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,12 +24,18 @@
 #include <glib.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <sys/wait.h>
-#include <sys/socket.h>
-#include <sys/un.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
+#ifdef G_OS_WIN32
+#  include <winsock2.h>
+#else
+#  if HAVE_SYS_WAIT_H
+#    include <sys/wait.h>
+#  endif
+#  include <sys/socket.h>
+#  include <sys/un.h>
+#  include <netinet/in.h>
+#  include <arpa/inet.h>
+#  include <netdb.h>
+#endif /* G_OS_WIN32 */
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
@@ -47,10 +53,6 @@
 #  include "ssl.h"
 #endif
 
-#if USE_GIO
-#error USE_GIO is currently not supported
-#endif
-
 #define BUFFSIZE	8192
 
 typedef gint (*SockAddrFunc)	(GList		*addr_list,
@@ -59,6 +61,7 @@ typedef gint (*SockAddrFunc)	(GList		*addr_list,
 typedef struct _SockConnectData	SockConnectData;
 typedef struct _SockLookupData	SockLookupData;
 typedef struct _SockAddrData	SockAddrData;
+typedef struct _SockSource	SockSource;
 
 struct _SockConnectData {
 	gint id;
@@ -90,19 +93,20 @@ struct _SockAddrData {
 	struct sockaddr *addr;
 };
 
+struct _SockSource {
+	GSource parent;
+	SockInfo *sock;
+};
+
 static guint io_timeout = 60;
 
 static GList *sock_connect_data_list = NULL;
 
-static gboolean sock_prepare		(gpointer	 source_data,
-					 GTimeVal	*current_time,
-					 gint		*timeout,
-					 gpointer	 data);
-static gboolean sock_check		(gpointer	 source_data,
-					 GTimeVal	*current_time,
-					 gpointer	 user_data);
-static gboolean sock_dispatch		(gpointer	 source_data,
-					 GTimeVal	*current_time,
+static gboolean sock_prepare		(GSource	*source,
+					 gint		*timeout);
+static gboolean sock_check		(GSource	*source);
+static gboolean sock_dispatch		(GSource	*source,
+					 GSourceFunc	 callback,
 					 gpointer	 user_data);
 
 GSourceFuncs sock_watch_funcs = {
@@ -126,6 +130,7 @@ static gint sock_connect_by_getaddrinfo	(const gchar	*hostname,
 					 gushort	 port);
 #endif
 
+#ifdef G_OS_UNIX
 static void sock_address_list_free		(GList		*addr_list);
 
 static gboolean sock_connect_async_cb		(GIOChannel	*source,
@@ -146,8 +151,32 @@ static SockLookupData *sock_get_address_info_async
 						 SockAddrFunc	 func,
 						 gpointer	 data);
 static gint sock_get_address_info_async_cancel	(SockLookupData	*lookup_data);
+#endif /* G_OS_UNIX */
 
 
+gint sock_init(void)
+{
+#ifdef G_OS_WIN32
+	WSADATA wsadata;
+	gint result;
+
+	result = WSAStartup(MAKEWORD(2, 2), &wsadata);
+	if (result != NO_ERROR) {
+		g_warning("WSAStartup() failed\n");
+		return -1;
+	}
+#endif
+	return 0;
+}
+
+gint sock_cleanup(void)
+{
+#ifdef G_OS_WIN32
+	WSACleanup();
+#endif
+	return 0;
+}
+
 gint sock_set_io_timeout(guint sec)
 {
 	io_timeout = sec;
@@ -156,6 +185,7 @@ gint sock_set_io_timeout(guint sec)
 
 gint fd_connect_unix(const gchar *path)
 {
+#ifdef G_OS_UNIX
 	gint sock;
 	struct sockaddr_un addr;
 
@@ -175,10 +205,14 @@ gint fd_connect_unix(const gchar *path)
 	}
 
 	return sock;
+#else
+	return -1;
+#endif
 }
 
 gint fd_open_unix(const gchar *path)
 {
+#ifdef G_OS_UNIX
 	gint sock;
 	struct sockaddr_un addr;
 
@@ -206,20 +240,28 @@ gint fd_open_unix(const gchar *path)
 	}
 
 	return sock;
+#else
+	return -1;
+#endif
 }
 
 gint fd_accept(gint sock)
 {
+#ifdef G_OS_UNIX
 	struct sockaddr_in caddr;
-	gint caddr_len;
+	guint caddr_len;
 
 	caddr_len = sizeof(caddr);
 	return accept(sock, (struct sockaddr *)&caddr, &caddr_len);
+#else
+	return -1;
+#endif
 }
 
 
 static gint set_nonblocking_mode(gint fd, gboolean nonblock)
 {
+#ifdef G_OS_UNIX
 	gint flags;
 
 	flags = fcntl(fd, F_GETFL, 0);
@@ -234,6 +276,9 @@ static gint set_nonblocking_mode(gint fd, gboolean nonblock)
 		flags &= ~O_NONBLOCK;
 
 	return fcntl(fd, F_SETFL, flags);
+#else
+	return -1;
+#endif
 }
 
 gint sock_set_nonblocking_mode(SockInfo *sock, gboolean nonblock)
@@ -245,6 +290,7 @@ gint sock_set_nonblocking_mode(SockInfo *sock, gboolean nonblock)
 
 static gboolean is_nonblocking_mode(gint fd)
 {
+#ifdef G_OS_UNIX
 	gint flags;
 
 	flags = fcntl(fd, F_GETFL, 0);
@@ -254,6 +300,9 @@ static gboolean is_nonblocking_mode(gint fd)
 	}
 
 	return ((flags & O_NONBLOCK) != 0);
+#else
+	return FALSE;
+#endif
 }
 
 gboolean sock_is_nonblocking_mode(SockInfo *sock)
@@ -264,17 +313,15 @@ gboolean sock_is_nonblocking_mode(SockInfo *sock)
 }
 
 
-static gboolean sock_prepare(gpointer source_data, GTimeVal *current_time,
-			     gint *timeout, gpointer data)
+static gboolean sock_prepare(GSource *source, gint *timeout)
 {
 	*timeout = 1;
 	return FALSE;
 }
 
-static gboolean sock_check(gpointer source_data, GTimeVal *current_time,
-			   gpointer user_data)
+static gboolean sock_check(GSource *source)
 {
-	SockInfo *sock = (SockInfo *)source_data;
+	SockInfo *sock = ((SockSource *)source)->sock;
 	struct timeval timeout = {0, 0};
 	fd_set fds;
 	GIOCondition condition = sock->condition;
@@ -306,12 +353,12 @@ static gboolean sock_check(gpointer source_data, GTimeVal *current_time,
 	return FD_ISSET(sock->sock, &fds) != 0;
 }
 
-static gboolean sock_dispatch(gpointer source_data, GTimeVal *current_time,
+static gboolean sock_dispatch(GSource *source, GSourceFunc callback,
 			      gpointer user_data)
 {
-	SockInfo *sock = (SockInfo *)source_data;
+	SockInfo *sock = ((SockSource *)source)->sock;
 
-	return sock->callback(sock, sock->condition, user_data);
+	return sock->callback(sock, sock->condition, sock->data);
 }
 
 static gboolean sock_watch_cb(GIOChannel *source, GIOCondition condition,
@@ -319,7 +366,10 @@ static gboolean sock_watch_cb(GIOChannel *source, GIOCondition condition,
 {
 	SockInfo *sock = (SockInfo *)data;
 
-	return sock->callback(sock, condition, sock->data);
+	if ((condition & sock->condition) == 0)
+		return TRUE;
+
+	return sock->callback(sock, sock->condition, sock->data);
 }
 
 guint sock_add_watch(SockInfo *sock, GIOCondition condition, SockFunc func,
@@ -330,9 +380,15 @@ guint sock_add_watch(SockInfo *sock, GIOCondition condition, SockFunc func,
 	sock->data = data;
 
 #if USE_SSL
-	if (sock->ssl)
-		return g_source_add(G_PRIORITY_DEFAULT, FALSE,
-				    &sock_watch_funcs, sock, data, NULL);
+	if (sock->ssl) {
+		GSource *source;
+
+		source = g_source_new(&sock_watch_funcs, sizeof(SockSource));
+		((SockSource *)source)->sock = sock;
+		g_source_set_priority(source, G_PRIORITY_DEFAULT);
+		g_source_set_can_recurse(source, FALSE);
+		return g_source_attach(source, NULL);
+	}
 #endif
 
 	return g_io_add_watch(sock->sock_ch, condition, sock_watch_cb, sock);
@@ -368,12 +424,14 @@ static gint fd_check_io(gint fd, GIOCondition cond)
 	}
 }
 
+#ifdef G_OS_UNIX
 static sigjmp_buf jmpenv;
 
 static void timeout_handler(gint sig)
 {
 	siglongjmp(jmpenv, 1);
 }
+#endif
 
 static gint sock_connect_with_timeout(gint sock,
 				      const struct sockaddr *serv_addr,
@@ -381,6 +439,7 @@ static gint sock_connect_with_timeout(gint sock,
 				      guint timeout_secs)
 {
 	gint ret;
+#ifdef G_OS_UNIX
 	void (*prev_handler)(gint);
 
 	alarm(0);
@@ -392,11 +451,14 @@ static gint sock_connect_with_timeout(gint sock,
 		return -1;
 	}
 	alarm(timeout_secs);
+#endif
 
 	ret = connect(sock, serv_addr, addrlen);
 
+#ifdef G_OS_UNIX
 	alarm(0);
 	signal(SIGALRM, prev_handler);
+#endif
 
 	return ret;
 }
@@ -404,6 +466,7 @@ static gint sock_connect_with_timeout(gint sock,
 struct hostent *my_gethostbyname(const gchar *hostname)
 {
 	struct hostent *hp;
+#ifdef G_OS_UNIX
 	void (*prev_handler)(gint);
 
 	alarm(0);
@@ -416,17 +479,22 @@ struct hostent *my_gethostbyname(const gchar *hostname)
 		return NULL;
 	}
 	alarm(io_timeout);
+#endif
 
 	if ((hp = gethostbyname(hostname)) == NULL) {
+#ifdef G_OS_UNIX
 		alarm(0);
 		signal(SIGALRM, prev_handler);
+#endif
 		fprintf(stderr, "%s: unknown host.\n", hostname);
 		errno = 0;
 		return NULL;
 	}
 
+#ifdef G_OS_UNIX
 	alarm(0);
 	signal(SIGALRM, prev_handler);
+#endif
 
 	return hp;
 }
@@ -528,15 +596,24 @@ static gint sock_connect_by_getaddrinfo(const gchar *hostname, gushort	port)
 
 SockInfo *sock_connect(const gchar *hostname, gushort port)
 {
+#ifdef G_OS_WIN32
+	SOCKET sock;
+#else
 	gint sock;
+#endif
 	SockInfo *sockinfo;
 
 #ifdef INET6
 	if ((sock = sock_connect_by_getaddrinfo(hostname, port)) < 0)
 		return NULL;
+#else
+#ifdef G_OS_WIN32
+	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
+		g_warning("socket() failed: %ld\n", WSAGetLastError());
 #else
 	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 		perror("socket");
+#endif /* G_OS_WIN32 */
 		return NULL;
 	}
 
@@ -554,11 +631,12 @@ SockInfo *sock_connect(const gchar *hostname, gushort port)
 	sockinfo->port = port;
 	sockinfo->state = CONN_ESTABLISHED;
 
-	usleep(100000);
+	g_usleep(100000);
 
 	return sockinfo;
 }
 
+#ifdef G_OS_UNIX
 static void sock_address_list_free(GList *addr_list)
 {
 	GList *cur;
@@ -580,7 +658,7 @@ static gboolean sock_connect_async_cb(GIOChannel *source,
 	SockConnectData *conn_data = (SockConnectData *)data;
 	gint fd;
 	gint val;
-	gint len;
+	guint len;
 	SockInfo *sockinfo;
 
 	fd = g_io_channel_unix_get_fd(source);
@@ -739,7 +817,7 @@ static gint sock_connect_address_list_async(SockConnectData *conn_data)
 	conn_data->cur_addr = conn_data->cur_addr->next;
 
 	conn_data->channel = g_io_channel_unix_new(sock);
-	conn_data->io_tag = g_io_add_watch(conn_data->channel, G_IO_IN|G_IO_OUT,
+	conn_data->io_tag = g_io_add_watch(conn_data->channel, G_IO_OUT,
 					   sock_connect_async_cb, conn_data);
 
 	return 0;
@@ -754,7 +832,7 @@ static gboolean sock_get_address_info_async_cb(GIOChannel *source,
 	SockLookupData *lookup_data = (SockLookupData *)data;
 	GList *addr_list = NULL;
 	SockAddrData *addr_data;
-	guint bytes_read;
+	gsize bytes_read;
 	gint ai_member[4];
 	struct sockaddr *addr;
 
@@ -952,6 +1030,7 @@ static gint sock_get_address_info_async_cancel(SockLookupData *lookup_data)
 
 	return 0;
 }
+#endif /* G_OS_UNIX */
 
 
 gint sock_printf(SockInfo *sock, const gchar *format, ...)
@@ -982,13 +1061,17 @@ gint fd_read(gint fd, gchar *buf, gint len)
 	if (fd_check_io(fd, G_IO_IN) < 0)
 		return -1;
 
+#ifdef G_OS_WIN32
+	return recv(fd, buf, len, 0);
+#else
 	return read(fd, buf, len);
+#endif
 }
 
 #if USE_SSL
 gint ssl_read(SSL *ssl, gchar *buf, gint len)
 {
-	gint ret;
+	gint err, ret;
 
 	if (SSL_pending(ssl) == 0) {
 		if (fd_check_io(SSL_get_rfd(ssl), G_IO_IN) < 0)
@@ -997,14 +1080,19 @@ gint ssl_read(SSL *ssl, gchar *buf, gint len)
 
 	ret = SSL_read(ssl, buf, len);
 
-	switch (SSL_get_error(ssl, ret)) {
+	switch ((err = SSL_get_error(ssl, ret))) {
 	case SSL_ERROR_NONE:
 		return ret;
 	case SSL_ERROR_WANT_READ:
 	case SSL_ERROR_WANT_WRITE:
 		errno = EAGAIN;
 		return -1;
+	case SSL_ERROR_ZERO_RETURN:
+		return 0;
 	default:
+		g_warning("SSL_read() returned error %d, ret = %d\n", err, ret);
+		if (ret == 0)
+			return 0;
 		return -1;
 	}
 }
@@ -1026,7 +1114,11 @@ gint fd_write(gint fd, const gchar *buf, gint len)
 	if (fd_check_io(fd, G_IO_OUT) < 0)
 		return -1;
 
+#ifdef G_OS_WIN32
+	return send(fd, buf, len, 0);
+#else
 	return write(fd, buf, len);
+#endif
 }
 
 #if USE_SSL
@@ -1065,9 +1157,7 @@ gint fd_write_all(gint fd, const gchar *buf, gint len)
 	gint n, wrlen = 0;
 
 	while (len) {
-		if (fd_check_io(fd, G_IO_OUT) < 0)
-			return -1;
-		n = write(fd, buf, len);
+		n = fd_write(fd, buf, len);
 		if (n <= 0)
 			return -1;
 		len -= n;
@@ -1161,61 +1251,68 @@ gint sock_gets(SockInfo *sock, gchar *buf, gint len)
 	return fd_gets(sock->sock, buf, len);
 }
 
-gchar *fd_getline(gint fd)
+gint fd_getline(gint fd, gchar **line)
 {
 	gchar buf[BUFFSIZE];
 	gchar *str = NULL;
 	gint len;
-	gulong size = 1;
+	gulong size = 0;
+	gulong cur_offset = 0;
 
 	while ((len = fd_gets(fd, buf, sizeof(buf))) > 0) {
 		size += len;
-		if (!str)
-			str = g_strdup(buf);
-		else {
-			str = g_realloc(str, size);
-			strcat(str, buf);
-		}
+		str = g_realloc(str, size + 1);
+		memcpy(str + cur_offset, buf, len + 1);
+		cur_offset += len;
 		if (buf[len - 1] == '\n')
 			break;
 	}
 
-	return str;
+	*line = str;
+
+	if (!str)
+		return -1;
+	else
+		return (gint)size;
 }
 
 #if USE_SSL
-gchar *ssl_getline(SSL *ssl)
+gint ssl_getline(SSL *ssl, gchar **line)
 {
 	gchar buf[BUFFSIZE];
 	gchar *str = NULL;
 	gint len;
-	gulong size = 1;
+	gulong size = 0;
+	gulong cur_offset = 0;
 
 	while ((len = ssl_gets(ssl, buf, sizeof(buf))) > 0) {
 		size += len;
-		if (!str)
-			str = g_strdup(buf);
-		else {
-			str = g_realloc(str, size);
-			strcat(str, buf);
-		}
+		str = g_realloc(str, size + 1);
+		memcpy(str + cur_offset, buf, len + 1);
+		cur_offset += len;
 		if (buf[len - 1] == '\n')
 			break;
 	}
 
-	return str;
+	*line = str;
+
+	if (!str)
+		return -1;
+	else
+		return (gint)size;
 }
 #endif
 
-gchar *sock_getline(SockInfo *sock)
+gint sock_getline(SockInfo *sock, gchar **line)
 {
-	g_return_val_if_fail(sock != NULL, NULL);
+	g_return_val_if_fail(sock != NULL, -1);
+	g_return_val_if_fail(line != NULL, -1);
 
 #if USE_SSL
 	if (sock->ssl)
-		return ssl_getline(sock->ssl);
+		return ssl_getline(sock->ssl, line);
 #endif
-	return fd_getline(sock->sock);
+	return fd_getline(sock->sock, line);
 }
 
 gint sock_puts(SockInfo *sock, const gchar *buf)
@@ -1231,7 +1328,7 @@ gint sock_puts(SockInfo *sock, const gchar *buf)
 #if USE_SSL
 gint ssl_peek(SSL *ssl, gchar *buf, gint len)
 {
-	gint ret;
+	gint err, ret;
 
 	if (SSL_pending(ssl) == 0) {
 		if (fd_check_io(SSL_get_rfd(ssl), G_IO_IN) < 0)
@@ -1240,14 +1337,19 @@ gint ssl_peek(SSL *ssl, gchar *buf, gint len)
 
 	ret = SSL_peek(ssl, buf, len);
 
-	switch (SSL_get_error(ssl, ret)) {
+	switch ((err = SSL_get_error(ssl, ret))) {
 	case SSL_ERROR_NONE:
 		return ret;
 	case SSL_ERROR_WANT_READ:
 	case SSL_ERROR_WANT_WRITE:
 		errno = EAGAIN;
 		return -1;
+	case SSL_ERROR_ZERO_RETURN:
+		return 0;
 	default:
+		g_warning("SSL_peek() returned error %d, ret = %d\n", err, ret);
+		if (ret == 0)
+			return 0;
 		return -1;
 	}
 }
