@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +39,37 @@
 #  include <wchar.h>
 #endif
 
+/* Wrappers for C library function that take pathname arguments. */
+#if GLIB_CHECK_VERSION(2, 6, 0)
+#  include <glib/gstdio.h>
+#else
+
+#define g_open		open
+#define g_rename	rename
+#define g_mkdir		mkdir
+#define g_stat		stat
+#define g_lstat		lstat
+#define g_unlink	unlink
+#define g_remove	remove
+#define g_rmdir		rmdir
+#define g_fopen		fopen
+#define g_freopen	freopen
+
+#endif /* GLIB_CHECK_VERSION */
+
+#if !GLIB_CHECK_VERSION(2, 7, 0)
+
+#ifdef G_OS_UNIX
+#define g_chdir		chdir
+#define g_chmod		chmod
+#else
+gint g_chdir	(const gchar	*path);
+gint g_chmod	(const gchar	*path,
+		 gint		 mode);
+#endif /* G_OS_UNIX */
+
+#endif /* !GLIB_CHECK_VERSION */
+
 /* The AC_CHECK_SIZEOF() in configure fails for some machines.
  * we provide some fallback values here */
 #if !SIZEOF_UNSIGNED_SHORT
@@ -141,6 +172,7 @@
 #define FILE_OP_ERROR(file, func) \
 { \
 	fprintf(stderr, "%s: ", file); \
+	fflush(stderr); \
 	perror(func); \
 }
 
@@ -234,8 +266,8 @@ wchar_t *wcscasestr		(const wchar_t *haystack,
 				 const wchar_t *needle);
 gint get_mbs_len		(const gchar	*s);
 
-gboolean is_next_nonascii	(const guchar *s);
-gint get_next_word_len		(const guchar *s);
+gboolean is_next_nonascii	(const gchar *s);
+gint get_next_word_len		(const gchar *s);
 
 /* functions for string parsing */
 gint subject_compare			(const gchar	*s1,
@@ -273,6 +305,8 @@ void extract_list_id_str		(gchar		*str);
 
 GSList *address_list_append		(GSList		*addr_list,
 					 const gchar	*str);
+GSList *references_list_prepend		(GSList		*msgid_list,
+					 const gchar	*str);
 GSList *references_list_append		(GSList		*msgid_list,
 					 const gchar	*str);
 GSList *newsgroup_list_append		(GSList		*group_list,
@@ -290,10 +324,16 @@ void subst_char				(gchar		*str,
 void subst_chars			(gchar		*str,
 					 gchar		*orig,
 					 gchar		 subst);
+void subst_null				(gchar		*str,
+					 gint		 len,
+					 gchar		 subst);
 void subst_for_filename			(gchar		*str);
 gboolean is_header_line			(const gchar	*str);
-gboolean is_ascii_str			(const guchar	*str);
+gboolean is_ascii_str			(const gchar	*str);
 gint get_quote_level			(const gchar	*str);
+gint check_line_length			(const gchar	*str,
+					 gint		 max_chars,
+					 gint		*line);
 
 gchar *strstr_with_skip_quote		(const gchar	*haystack,
 					 const gchar	*needle);
@@ -319,8 +359,10 @@ gchar *trim_string_before		(const gchar	*str,
 GList *uri_list_extract_filenames	(const gchar	*uri_list);
 gboolean is_uri_string			(const gchar	*str);
 gchar *get_uri_path			(const gchar	*uri);
+gint get_uri_len			(const gchar	*str);
 void decode_uri				(gchar		*decoded_uri,
 					 const gchar	*encoded_uri);
+gchar *encode_uri			(const gchar	*filename);
 gint scan_mailto_url			(const gchar	*mailto,
 					 gchar	       **to,
 					 gchar	       **cc,
@@ -329,15 +371,17 @@ gint scan_mailto_url			(const gchar	*mailto,
 					 gchar	       **body);
 
 /* return static strings */
-gchar *get_home_dir		(void);
-gchar *get_rc_dir		(void);
-gchar *get_news_cache_dir	(void);
-gchar *get_imap_cache_dir	(void);
-gchar *get_mime_tmp_dir		(void);
-gchar *get_template_dir		(void);
-gchar *get_tmp_dir		(void);
-gchar *get_tmp_file		(void);
-gchar *get_domain_name		(void);
+const gchar *get_home_dir		(void);
+const gchar *get_rc_dir			(void);
+const gchar *get_old_rc_dir		(void);
+const gchar *get_mail_base_dir		(void);
+const gchar *get_news_cache_dir		(void);
+const gchar *get_imap_cache_dir		(void);
+const gchar *get_mime_tmp_dir		(void);
+const gchar *get_template_dir		(void);
+const gchar *get_tmp_dir		(void);
+gchar *get_tmp_file			(void);
+const gchar *get_domain_name		(void);
 
 /* file / directory handling */
 off_t get_file_size		(const gchar	*file);
@@ -365,9 +409,13 @@ gint remove_all_numbered_files	(const gchar	*dir);
 gint remove_expired_files	(const gchar	*dir,
 				 guint		 hours);
 gint remove_dir_recursive	(const gchar	*dir);
+gint rename_force		(const gchar	*oldpath,
+				 const gchar	*newpath);
 gint copy_file			(const gchar	*src,
 				 const gchar	*dest,
 				 gboolean	 keep_backup);
+gint copy_dir			(const gchar	*src,
+				 const gchar	*dest);
 gint move_file			(const gchar	*src,
 				 const gchar	*dest,
 				 gboolean	 overwrite);
@@ -415,6 +463,11 @@ gchar *tzoffset			(time_t		*now);
 void get_rfc822_date		(gchar		*buf,
 				 gint		 len);
 
+size_t my_strftime		(gchar			*s,
+				 size_t			 max,
+				 const gchar		*format,
+				 const struct tm	*tm);
+
 /* logging */
 void set_log_file	(const gchar *filename);
 void close_log_file	(void);
